SystemOrganization addCategory: #'DeltaStreams-Visitors'!
SystemOrganization addCategory: #'DeltaStreams-Model-Base'!
SystemOrganization addCategory: #'DeltaStreams-Model-Class'!
SystemOrganization addCategory: #'DeltaStreams-Model-Composite'!
SystemOrganization addCategory: #'DeltaStreams-Model-Method'!
SystemOrganization addCategory: #'DeltaStreams-TestClasses'!
SystemOrganization addCategory: #'DeltaStreams-Tests'!
SystemOrganization addCategory: #'DeltaStreams-Logging'!
SystemOrganization addCategory: #'DeltaStreams-TestClasses-new'!
SystemOrganization addCategory: #'DeltaStreams-GUI'!
SystemOrganization addCategory: #'DeltaStreams-Storing'!

!SystemChangeNotifier methodsFor: '*DeltaStreams' stamp: 'gk 8/14/2007 23:47'!
hasNotificationsFor: anObject
	"Do we send system notifications to anObject?"

	^eventSource hasActionsWithReceiver: anObject! !

!String methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 09:21'!
nullMethodSource
	"Assuming I am a selector, answer a blank method that implements me"

	| numArgs |
	numArgs := self numArgs.
	numArgs < 0 ifTrue: [^ nil].
	numArgs = 0 ifTrue: [^ self asString].
	^ String streamContents: [:stream | self keywords with: (1 to: numArgs)
		do: [:keyword :index | stream
			nextPutAll: keyword;
			space;
			nextPut: $t;
			print: index;
			space]]! !

!String methodsFor: '*DeltaStreams' stamp: 'gk 8/23/2007 23:30'!
scanNames

	^Scanner new scanFieldNames: self! !

!Class methodsFor: '*DeltaStreams' stamp: 'gk 8/21/2007 01:17'!
changeSuperclass: newSuperclass
	"Change superclass of the receiver."

	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: newSuperclass
		type: self typeOfClass
		instanceVariableNames: self instanceVariablesString
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category! !

!Class methodsFor: '*DeltaStreams' stamp: 'mtf 9/4/2007 10:51'!
classVariableNames: classVarString
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: self instanceVariablesString  
		classVariableNames: classVarString
		poolDictionaries: self sharedPoolsString
		category: self category! !

!Class methodsFor: '*DeltaStreams' stamp: 'mtf 9/4/2007 10:49'!
instanceVariableNames: instVarString
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: instVarString 
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category! !

!Class methodsFor: '*DeltaStreams' stamp: 'mtf 9/4/2007 10:52'!
poolDictionaries: sharedPoolsString
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: self instanceVariablesString 
		classVariableNames: self classVariablesString
		poolDictionaries: sharedPoolsString
		category: self category! !

MCReader subclass: #DSReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

!DSReader commentStamp: 'mtf 9/26/2007 11:08' prior: 0!
My subclasses represent readers for streamed representations of DeltaStream-related objects. I inherit from MCReader since it has some useful class methods!

DSReader subclass: #DSDeltaReader
	instanceVariableNames: 'delta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

!DSDeltaReader commentStamp: 'mtf 9/26/2007 11:04' prior: 0!
My instances can read deltas from streams written by DSDeltaWriters. I am modeled quite heavily after MCVersionReader!

!DSDeltaReader class methodsFor: 'reading' stamp: 'mtf 9/26/2007 12:32'!
deltaFromFile: aFileName
	^ FileStream readOnlyFileNamed: aFileName
		do: [:stream | self deltaFromStream: stream]! !

!DSDeltaReader class methodsFor: 'reading' stamp: 'mtf 9/26/2007 11:28'!
deltaFromStream: aStream
	^ (self on: aStream) delta! !

!DSDeltaReader class methodsFor: 'file services' stamp: 'mtf 9/26/2007 11:00'!
fileReaderServicesForFile: fullName suffix: suffix
	self isAbstract ifTrue: [^ #()].
	^ ((suffix = self extension) or: [ suffix = '*' ])
		ifTrue: [self services]
		ifFalse: [Array new: 0]
		! !

!DSDeltaReader class methodsFor: 'file services' stamp: 'mtf 9/26/2007 11:09'!
initialize
	"DSDeltaReader initialize"
	self concreteSubclasses do: [:aClass | FileList registerFileReader: aClass]! !

!DSDeltaReader class methodsFor: 'file services' stamp: 'mtf 9/26/2007 12:34'!
registerDeltaFile: aFileName
	DSDeltaRepository current addAsking: (self deltaFromFile: aFileName)! !

!DSDeltaReader class methodsFor: 'file services' stamp: 'mtf 9/26/2007 12:33'!
serviceRegisterDelta
	^ SimpleServiceEntry
		provider: self
		label: 'register delta'
		selector: #registerDeltaFile:
		description: 'Register delta with the image so that it can be viewed in the change sorter'
		buttonLabel: 'register'! !

!DSDeltaReader class methodsFor: 'file services' stamp: 'mtf 9/26/2007 11:18'!
services
	^ Array
		with: self serviceRegisterDelta! !

!DSDeltaReader methodsFor: 'accessing' stamp: 'mtf 9/26/2007 11:11'!
delta
	delta ifNil: [self loadDelta].
	^ delta! !

!DSDeltaReader methodsFor: 'loading' stamp: 'mtf 9/26/2007 11:11'!
loadDelta
	self subclassResponsibility! !

DSDeltaReader subclass: #DSDszReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

!DSDszReader commentStamp: 'mtf 9/25/2007 14:05' prior: 0!
My instances are able to decode and install deltas written by DSBinStWriter. They are meant to be compressed and installable via file-in, but they are currently uncompressed due to my non-understanding of DeflateStreams/InflateStreams!

!DSDszReader class methodsFor: 'as yet unclassified' stamp: 'mtf 9/26/2007 10:53'!
extension
	^ 'dsz'! !

!DSDszReader methodsFor: 'as yet unclassified' stamp: 'mtf 9/26/2007 12:27'!
loadDelta
	delta := DataStream unStream: stream binary upToEnd asString unzipped! !

!MethodEditor methodsFor: '*DeltaStreams' stamp: 'mtf 8/29/2007 13:20'!
category: catString
	category := catString! !

!MethodEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 13:13'!
source
	^ source! !

!MethodEditor methodsFor: '*DeltaStreams' stamp: 'mtf 8/29/2007 13:20'!
source: sourceString
	source := sourceString.! !

!MethodEditor methodsFor: '*DeltaStreams' stamp: 'mtf 8/29/2007 13:21'!
stamp: timestamp
	stamp := timestamp.! !

!SequenceableCollection methodsFor: '*DeltaStreams' stamp: 'gk 8/23/2007 22:59'!
asSpaceString
	^ String streamContents: [:stream | 
		self
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!SequenceableCollection methodsFor: '*DeltaStreams' stamp: 'mtf 8/31/2007 11:46'!
reverseCollect: aBlock 
	"Evaluate aBlock with each of the receiver's elements, in reverse order,
	 as the argument.  Collect the resulting values into a collection like the 
	receiver. Answer the new collection."

	| newCollection |
	newCollection := self species new: self size.
	self size to: 1 by: -1 do:
		[:index |
		newCollection at: index put: (aBlock value: (self at: index))].
	^ newCollection! !

!SequenceableCollection methodsFor: '*DeltaStreams' stamp: 'mtf 9/13/2007 09:03'!
reverseSelect: aBlock 
	"Evaluate aBlock with each of the receiver's elements, in reverse order,
	 as the argument.  Collect the resulting values into a collection like the 
	receiver. Answer the new collection."

	| newCollection |
	newCollection := self species new: self size.
	self reverseDo: [:elem | (aBlock value: elem) ifTrue: [newCollection add: elem]].
	^ newCollection! !

!SequenceableCollection methodsFor: '*DeltaStreams' stamp: 'gk 8/23/2007 23:30'!
scanNames
	^self! !

TestCase subclass: #DSTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSTestCase commentStamp: '<historical>' prior: 0!
An abstract base class for DeltaStreams test cases - just offers some convenience methods.!

DSTestCase subclass: #DSDeltaClassifyTest
	instanceVariableNames: 'delta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaClassifyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 20:36'!
setUp
	delta := DSDelta new.! !

!DSDeltaClassifyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 20:37'!
tearDown
	delta stopLogging.
	self removeTestClasses! !

!DSDeltaClassifyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 23:58'!
testClassifierAcrossRename
	| class classified classChanges |
	delta startLogging.
	class := self createClassAWithMethods.
	class := class rename: self newClassAName.
	self createMethodBIn: class.
	delta stopLogging.
	classified := DSChangeCategorizer forClassesAndMethods withAll: delta.
	self assert: classified dictionary size = 1.
	classChanges := classified at: self newClassAName.
	self assert: classChanges methodChanges dictionary size = 2.
	self 
		assert: (classChanges methodChanges dictionary includesKey: self selectorA).
	self 
		assert: (classChanges methodChanges dictionary includesKey: self selectorB).
	self assert: classChanges classMethodChanges dictionary size = 1.
	self assert: (classChanges classMethodChanges dictionary 
				includesKey: self selectorA)! !

!DSDeltaClassifyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 23:58'!
testClassifierForClasses
	| classified classChanges |
	delta startLogging.
	self createClassAWithMethods.
	self createClassB.
	delta stopLogging.
	classified := DSChangeCategorizer forClassesAndMethods withAll: delta.
	self assert: classified dictionary size = 2.
	classChanges := classified at: self oldClassAName.
	self assert: classChanges methodChanges dictionary size = 1.
	self 
		assert: (classChanges methodChanges dictionary includesKey: self selectorA).
	self assert: classChanges classMethodChanges dictionary size = 1.
	self assert: (classChanges classMethodChanges dictionary 
				includesKey: self selectorA).
	classChanges := classified at: self classBName.
	self assert: classChanges methodChanges dictionary size = 0.
	self assert: classChanges classMethodChanges dictionary size = 0! !

DSTestCase subclass: #DSDeltaCleanApplyTest
	instanceVariableNames: 'class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaCleanApplyTest commentStamp: 'mtf 9/14/2007 10:32' prior: 0!
A set of tests about applying deltas to the system. These tests don't check that the deltas are valid, but they are. See DSDirtyApplyTest for tests on forceful application of dirty deltas. Most of these tests need to be commented.!

!DSDeltaCleanApplyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/12/2007 09:15'!
setUp
	self createTestPools.
	class := self createClassAFull.! !

!DSDeltaCleanApplyTest methodsFor: 'as yet unclassified' stamp: 'mtf 8/30/2007 11:45'!
tearDown
	class := nil.
	self removeTestClasses; removeTestPools! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 12:09'!
testAddClassMethod
	
	self deny: (class class selectors includes: self selectorB).
	DSDelta new addChange: (DSMethodAddedChange new
		setClass: class class;
		selector: self selectorB;
		stamp: self oldStamp;
		source: self methodBSource;
		protocol: self oldProtocol;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorB).
	self assert: (class class organization categoryOfElement: self selectorB) = self oldProtocol.
	self assert: (class class sourceCodeAt: self selectorB) = self methodBSource.
	self assert: (class class compiledMethodAt: self selectorB) timeStamp = self oldStamp! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 9/12/2007 10:28'!
testAddInstanceMethod
	self deny: (class selectors includes: self selectorB).
	DSDelta new addChange: (DSMethodAddedChange new
		setClass: class;
		selector: self selectorB;
		stamp: self oldStamp;
		source: self methodBSource;
		protocol: self oldProtocol;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorB).
	self assert: (class organization categoryOfElement: self selectorB) = self oldProtocol.
	self assert: (class sourceCodeAt: self selectorB) = self methodBSource.
	self assert: (class compiledMethodAt: self selectorB) timeStamp = self oldStamp! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:09'!
testChangeClassCategory
	"Change the category of a class. Should succeed"
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassCategoryChange
		className: self oldClassAName
		from: self oldClassCategory
		to: self newClassCategory
	); apply.
	
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self newClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:09'!
testChangeClassComment
	"Change a class comment. Should succeed"
	
	DSDelta new addChange: (DSClassCommentChange new
		className: self oldClassAName;
		oldComment: self oldComment;
		newComment: self newComment;
		oldStamp: self oldStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self newComment.
	self assert: class organization commentStamp = self newStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:10'!
testChangeClassInstVars
	"Change the insance variables on the metaclass. Should succeed"
	
	DSDelta new addChange: (DSClassInstVarsChange new
		className: self oldClassAName;
		oldVars: self oldInstVarString scanNames;
		newVars: self newInstVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self newInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:18'!
testChangeClassMethodProtocol
	"Classify a class method under a new protocol. should succeed"
	
	DSDelta new addChange: (DSMethodProtocolChange new
		setClass: class class;
		selector: self selectorA;
		oldProtocol: self oldProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self newProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:19'!
testChangeClassMethodSource
	"Change the source of a class method"
	
	DSDelta new addChange: (DSMethodSourceChange new
		setClass: class class;
		selector: self selectorA;
		oldSource: self oldMethodASource;
		newSource: self newMethodASource;
		oldStamp: self oldStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self newMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self newStamp.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:12'!
testChangeClassSuperclass
	"Change the Superclass of a class. The superclass exists, so this is valid and should succeed"
	
	DSDelta new addChange: (DSClassSuperclassChange new
		className: self oldClassAName;
		oldSuperclass: self oldSuperclass;
		newSuperclass: self newSuperclass;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self newSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:13'!
testChangeClassVars
	"Change the class pool variable names of a class. Valid; should succeed"
	
	DSDelta new addChange: (DSClassVarsChange new
		className: self oldClassAName;
		oldVars: self oldClassVarString scanNames;
		newVars: self newClassVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self newClassVarString scanNames sort asSpaceString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:15'!
testChangeInstVars
	"Change the instance variables of a class. Should succeed"
	
	DSDelta new addChange: (DSInstVarsChange new
		className: self oldClassAName;
		oldVars: self oldInstVarString scanNames;
		newVars: self newInstVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self newInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:18'!
testChangeInstanceMethodProtocol
	"Classify an instance method under a new protocol. should succeed"
	
	DSDelta new addChange: (DSMethodProtocolChange new
		setClass: class;
		selector: self selectorA;
		oldProtocol: self oldProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self newProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:19'!
testChangeInstanceMethodSource
	"Change the source of a class method"
	
	DSDelta new addChange: (DSMethodSourceChange new
		setClass: class;
		selector: self selectorA;
		oldSource: self oldMethodASource;
		newSource: self newMethodASource;
		oldStamp: self oldStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self newMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self newStamp.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:16'!
testChangeSharedPools
	"Change the shared pools of a class. The pools exist, so this should succeed"
	
	DSDelta new addChange: (DSSharedPoolVarsChange new
		className: self oldClassAName;
		oldVars: self oldSharedPoolString scanNames;
		newVars: self newSharedPoolString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self newSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 11:19'!
testCreateNormalClass
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #variable;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self assert: newClass isFixed.
	self deny:  newClass isVariable.
	self assert: newClass isPointers.
	self deny:  newClass isBits.
	self deny:  newClass isBytes.
	self assert: newClass isWords.
	self deny:  newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == self oldSuperclass.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self oldSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 11:21'!
testCreateVariableByteClass
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #bytes;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self deny:  newClass isFixed.
	self assert: newClass isVariable.
	self deny:  newClass isPointers.
	self assert: newClass isBits.
	self assert: newClass isBytes.
	self deny:  newClass isWords.
	self deny:  newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == self oldSuperclass.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self oldSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 11:19'!
testCreateVariableClass
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #variable;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self deny:  newClass isFixed.
	self assert: newClass isVariable.
	self assert: newClass isPointers.
	self deny:  newClass isBits.
	self deny:  newClass isBytes.
	self assert: newClass isWords.
	self deny:  newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == self oldSuperclass.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self oldSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 11:22'!
testCreateVariableWordClass
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #words;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self deny:  newClass isFixed.
	self assert: newClass isVariable.
	self deny:  newClass isPointers.
	self assert: newClass isBits.
	self deny:  newClass isBytes.
	self assert: newClass isWords.
	self deny:  newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == self oldSuperclass.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self oldSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 11:25'!
testCreateWeakClass
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #weak;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self deny:  newClass isFixed.
	self assert: newClass isVariable.
	self assert: newClass isPointers.
	self deny:  newClass isBits.
	self deny:  newClass isBytes.
	self assert: newClass isWords.
	self assert: newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == self oldSuperclass.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self oldSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 12:44'!
testRemoveClass
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassRemovedChange new
		className: self oldClassAName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldLocalVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #normal;
	yourself); apply.
	
	self deny: (Smalltalk includesKey: self classBName).! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 12:12'!
testRemoveClassMethod
	self assert: (class class selectors includes: self selectorA).
	DSDelta new addChange: (DSMethodRemovedChange new
		setClass: class class;
		selector: self selectorA;
		stamp: self oldStamp;
		source: self oldMethodASource;
		protocol: self oldProtocol;
	yourself); apply.
	
	self deny: (class class selectors includes: self selectorA).! !

!DSDeltaCleanApplyTest methodsFor: 'method tests' stamp: 'mtf 9/12/2007 10:28'!
testRemoveInstanceMethod
	self assert: (class selectors includes: self selectorA).
	DSDelta new addChange: (DSMethodRemovedChange new
		setClass: class;
		selector: self selectorA;
		stamp: self oldStamp;
		source: self oldMethodASource;
		protocol: self oldProtocol;
	yourself); apply.
	
	self deny: (class selectors includes: self selectorA).! !

!DSDeltaCleanApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:11'!
testRenameClass
	"Rename a class to a new, free name. Should succeed"
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassNameChange
		className: self oldClassAName
		to: self newClassAName
	); apply.
	
	self deny:  (Smalltalk includesKey: self oldClassAName).
	self assert: (Smalltalk includesKey: self newClassAName).
	class := Smalltalk classNamed: self newClassAName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self newClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

DSTestCase subclass: #DSDeltaDirtyApplyTest
	instanceVariableNames: 'class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaDirtyApplyTest commentStamp: 'mtf 9/14/2007 10:31' prior: 0!
A set of tests about applying deltas to the system. These tests don't check that the deltas are valid, and they are not. This tests what happens when a dirty delta is forcefully applied, presumably after the user has confirmed "Yes, I know this is stupid, but do it anyway". Each test is commented with the expected behavior.!

!DSDeltaDirtyApplyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/14/2007 10:37'!
setUp
	self createTestPools.
	class := self createClassAFull.! !

!DSDeltaDirtyApplyTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/14/2007 10:37'!
tearDown
	class := nil.
	self removeTestClasses; removeTestPools! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 10:38'!
testAddPreExistingClassMethod
	"Add a method that already exists. Invalid, but should succeed"
	
	self assert: (class class selectors includes: self selectorA).
	DSDelta new addChange: (DSMethodAddedChange new
		setClass: class class;
		selector: self selectorA;
		stamp: self newStamp;
		source: self newMethodASource;
		protocol: self newProtocol;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self newProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self newMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self newStamp! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 10:38'!
testAddPreExistingInstanceMethod
	"Add a method that already exists. Invalid, but should succeed"
	
	self assert: (class class selectors includes: self selectorA).
	DSDelta new addChange: (DSMethodAddedChange new
		setClass: class;
		selector: self selectorA;
		stamp: self newStamp;
		source: self newMethodASource;
		protocol: self newProtocol;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self newProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self newMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self newStamp! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:15'!
testBadClassAddClassMethod
	"Add a class side method to a non-existant class. Invalid; should create the class and the method"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSMethodAddedChange new
		className: self badClassName;
		meta: true;
		selector: self badSelector;
		stamp: self badStamp;
		source: self badSource;
		protocol: self badProtocol;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: (class class selectors includes: self badSelector).
	self assert: (class class organization categoryOfElement: self badSelector) = self badProtocol.
	self assert: (class class sourceCodeAt: self badSelector) = self badSource.
	self assert: (class class compiledMethodAt: self badSelector) timeStamp = self badStamp! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:14'!
testBadClassAddInstanceMethod
	"Add an instance side method to a non-existant class. Invalid; should create the class and the method"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSMethodAddedChange new
		className: self badClassName;
		meta: false;
		selector: self badSelector;
		stamp: self badStamp;
		source: self badSource;
		protocol: self badProtocol;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: (class selectors includes: self badSelector).
	self assert: (class organization categoryOfElement: self badSelector) = self badProtocol.
	self assert: (class sourceCodeAt: self badSelector) = self badSource.
	self assert: (class compiledMethodAt: self badSelector) timeStamp = self badStamp! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:44'!
testBadClassChangeClassBadSuperclass
	"Change the Superclass of a non-existing class to a non-existing superclass. Should create the class with a superclass of Object"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSClassSuperclassChange new
		className: self badClassName;
		oldSuperclass: self oldSuperclass;
		newSuperclass: self badSuperclassName;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString isEmpty.
	self deny:  class hasComment.
! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:52'!
testBadClassChangeClassCategory
	"Change the category of a non-existant class. Not sure if the class should be created or not. The test assumes the test should be created"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSClassCategoryChange
		className: self badClassName
		from: self oldClassCategory
		to: self newClassCategory
	); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == self newClassCategory.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString isEmpty.
	self deny:  class hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:43'!
testBadClassChangeClassComment
	"Change the comment of a non-existing class. Should create the class and give it the comment"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSClassCommentChange new
		className: self badClassName;
		oldComment: self oldComment;
		newComment: self newComment;
		oldStamp: self oldStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString isEmpty.
	self assert: class hasComment.
	self assert: class organization classComment = self newComment.
	self assert: class organization commentStamp = self newStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:43'!
testBadClassChangeClassInstVars
	"Change the insance variables on the metaclass of a non-existant class. Should create the class and give it's metaclass the instance variables'"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSClassInstVarsChange new
		className: self badClassName;
		oldVars: self oldInstVarString scanNames;
		newVars: self newInstVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString = self newInstVarString.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString isEmpty.
	self deny:  class hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/20/2007 09:32'!
testBadClassChangeClassMethodProtocol
	"Classyfy a class method in a non-existant class. Should create the class and a null method"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSMethodProtocolChange new
		className: self badClassName;
		meta: true;
		selector: self badSelector;
		oldProtocol: self badProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self deny:  (class class selectors includes: self badSelector).
	self assert: (class class selectors includes: self badSelector).
	self assert: (class class organization categoryOfElement: self badSelector) = self newProtocol.
	self assert: (class class sourceCodeAt: self badSelector) = self badSelector nullMethodSource.
	self assert: (class class compiledMethodAt: self badSelector) timeStamp = self newStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/20/2007 09:33'!
testBadClassChangeClassMethodSource
	"change the source for a class method in a non-existant class. Should create the class and the method, categorizing each under the default category"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSMethodSourceChange new
		className: self badClassName;
		meta: true;
		selector: self badSelector;
		oldSource: self badOldSource;
		newSource: self badSource;
		oldStamp: self badStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self deny: (class class selectors includes: self badSelector).
	self assert: (class class selectors includes: self badSelector).
	self assert: (class class organization categoryOfElement: self badSelector) = Categorizer default.
	self assert: (class class sourceCodeAt: self badSelector) = self badSource.
	self assert: (class class compiledMethodAt: self badSelector) timeStamp = self newStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:44'!
testBadClassChangeClassSuperclass
	"Change the Superclass of a non-existing class. Should create the class with the givin superclass"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSClassSuperclassChange new
		className: self badClassName;
		oldSuperclass: self oldSuperclass;
		newSuperclass: self newSuperclass;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == self newSuperclass.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString isEmpty.
	self deny:  class hasComment.
! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:44'!
testBadClassChangeClassVars
	"Change the class pool variable names of a non-existant class. Should create the class and give it the class vars"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSClassVarsChange new
		className: self badClassName;
		oldVars: self oldClassVarString scanNames;
		newVars: self newClassVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString = self newClassVarString.
	self assert: class sharedPoolsString isEmpty.
	self deny:  class hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:45'!
testBadClassChangeInstVars
	"Change the insance variables of a non-existant class. Should create the class and give it the instance variables"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSInstVarsChange new
		className: self badClassName;
		oldVars: self oldInstVarString scanNames;
		newVars: self newInstVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString = self newInstVarString.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString isEmpty.
	self deny:  class hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/20/2007 09:30'!
testBadClassChangeInstanceMethodProtocol
	"Classyfy a class method in a non-existant class. Should create the class and a null method"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSMethodProtocolChange new
		className: self badClassName;
		meta: true;
		selector: self badSelector;
		oldProtocol: self badProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self deny: (class selectors includes: self badSelector).
	self assert: (class selectors includes: self badSelector).
	self assert: (class organization categoryOfElement: self badSelector) = self newProtocol.
	self assert: (class sourceCodeAt: self badSelector) = self badSelector nullMethodSource.
	self assert: (class compiledMethodAt: self badSelector) timeStamp = self newStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/20/2007 09:32'!
testBadClassChangeInstanceMethodSource
	"change the source for an instance method in a non-existant class. Should create both the method and the class, categorizing each under the default category"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSMethodSourceChange new
		className: self badClassName;
		meta: false;
		selector: self badSelector;
		oldSource: self badOldSource;
		newSource: self badSource;
		oldStamp: self badStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self deny: (class selectors includes: self badSelector).
	self assert: (class selectors includes: self badSelector).
	self assert: (class organization categoryOfElement: self badSelector) = Categorizer default.
	self assert: (class sourceCodeAt: self badSelector) = self badSource.
	self assert: (class compiledMethodAt: self badSelector) timeStamp = self newStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:45'!
testBadClassChangeSharedPools
	"Change the shared pools of a class. Should create the class with the givin pools"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSSharedPoolVarsChange new
		className: self badClassName;
		oldVars: self oldSharedPoolString scanNames;
		newVars: self newSharedPoolString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString = self newSharedPoolString.
	self deny:  class hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:45'!
testBadClassChangeSharedPoolsBad
	"Change the shared pools of a non-existing class to non-existing pools. Should create the class, create the pools, and add the pools to the class"
	
	self deny: (Smalltalk includesKey: self badClassName).
	DSDelta new addChange: (DSSharedPoolVarsChange new
		className: self badClassName;
		oldVars: self oldSharedPoolString scanNames;
		newVars: self badSharedPoolString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self badClassName).
	class := Smalltalk classNamed: self badClassName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self badClassName.
	self assert: class superclass == Object.
	self assert: class category == Categorizer default.
	self assert: class instanceVariablesString isEmpty.
	self assert: class class instanceVariablesString isEmpty.
	self assert: class classVariablesString isEmpty.
	self assert: class sharedPoolsString = self badSharedPoolString.
	self deny:  class hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/19/2007 15:00'!
testBadClassRemoveClassMethod
	"Remove an class-side method in a non-existant class. Should do nothing; don't create a method or a class'"
	
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSMethodRemovedChange new
		className: self badClassName;
		meta: true;
		selector: self badSelector;
		protocol: self badProtocol;
		source: self badSource;
		stamp: self badStamp;
	yourself); apply.
	
	self deny: (Smalltalk includesKey: self badClassName).! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/19/2007 15:00'!
testBadClassRemoveInstanceMethod
	"Remove an instance-side method in a non-existant class. Should do nothing; don't create a method or a class"
	
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSMethodRemovedChange new
		className: self badClassName;
		meta: false;
		selector: self badSelector;
		protocol: self badProtocol;
		source: self badSource;
		stamp: self badStamp;
	yourself); apply.
	
	self deny: (Smalltalk includesKey: self badClassName).! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:47'!
testBadClassRenameToBadClass
	"Rename a non-existant class to another non-existent class. Invalid; should do nothing"
	
	self deny: (Smalltalk includesKey: self badClassAName).
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassNameChange
		className: self badClassAName
		to: self classBName
	); apply.
	
	self deny:  (Smalltalk includesKey: self badClassAName).
	self deny:  (Smalltalk includesKey: self classBName).
	self assert: (Smalltalk includesKey: self oldClassAName).
	class := Smalltalk classNamed: self oldClassAName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self newClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/20/2007 09:48'!
testBadClassRenameToGoodClass
	"Rename a non-existant class to an existing class. Invalid; should do nothing"
	
	self deny:  (Smalltalk includesKey: self badClassAName).
	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassNameChange
		className: self badClassAName
		to: self oldClassAName
	); apply.
	
	self deny:  (Smalltalk includesKey: self badClassAName).
	self deny:  (Smalltalk includesKey: self classBName).
	self assert: (Smalltalk includesKey: self oldClassAName).
	class := Smalltalk classNamed: self oldClassAName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self newClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 10:38'!
testBadSharedPools
	"Uses a non-existant pool"
	
	DSDelta new addChange: (DSSharedPoolVarsChange new
		className: self oldClassAName;
		oldVars: self oldSharedPoolString scanNames;
		newVars: self badSharedPoolString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self badSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:00'!
testChangeClassCategoryBadHistory
	"Change the category of a class, where the history is wrong. Should succeed"
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassCategoryChange
		className: self oldClassAName
		from: self badClassCategory
		to: self newClassCategory
	); apply.
	
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self newClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:00'!
testChangeClassCommentBadHistory
	"Change a class comment, where the history is wrong. Should succeed"
	
	DSDelta new addChange: (DSClassCommentChange new
		className: self oldClassAName;
		oldComment: self badComment;
		newComment: self newComment;
		oldStamp: self badStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self newComment.
	self assert: class organization commentStamp = self newStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:01'!
testChangeClassInstVarsBadHistory
	"Change the insance variables on the metaclass, where the history is wrong. Should succeed"
	
	DSDelta new addChange: (DSClassInstVarsChange new
		className: self oldClassAName;
		oldVars: self badInstVarString scanNames;
		newVars: self newInstVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self newInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:22'!
testChangeClassMethodProtocolBadHistory
	"Classify a class method under a new protocol, where the change history is not correct. Invalid; should succeed"
	
	DSDelta new addChange: (DSMethodProtocolChange new
		setClass: class class;
		selector: self selectorA;
		oldProtocol: self badProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self newProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:22'!
testChangeClassMethodSourceBadHistory
	"Change the source of an instance method, where the change history is wrong. Should succeed"
	
	DSDelta new addChange: (DSMethodSourceChange new
		setClass: class class;
		selector: self selectorA;
		oldSource: self badMethodASource;
		newSource: self newMethodASource;
		oldStamp: self badStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self newMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self newStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:01'!
testChangeClassSuperclassBadHistory
	"Change the Superclass of a class, where the history is wrong. The superclass exists, so this is valid and should succeed"
	
	DSDelta new addChange: (DSClassSuperclassChange new
		className: self oldClassAName;
		oldSuperclassName: self badSuperclassName;
		newSuperclass: self newSuperclass;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self newSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:01'!
testChangeClassVarsBadHistory
	"Change the class pool variable names of a class, where the history is wrong. should succeed"
	
	DSDelta new addChange: (DSClassVarsChange new
		className: self oldClassAName;
		oldVars: self badClassVarString scanNames;
		newVars: self newClassVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self newClassVarString scanNames sort asSpaceString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:01'!
testChangeInstVarsBadHistory
	"Change the instance variables of a class, where the history is wrong. Should succeed"
	
	DSDelta new addChange: (DSInstVarsChange new
		className: self oldClassAName;
		oldVars: self badInstVarString scanNames;
		newVars: self newInstVarString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self newInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:22'!
testChangeInstanceMethodProtocolBadHistory
	"Classify a class method under a new protocol, where the change history is not correct. Invalid; should succeed"
	
	DSDelta new addChange: (DSMethodProtocolChange new
		setClass: class;
		selector: self selectorA;
		oldProtocol: self badProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self newProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:22'!
testChangeInstanceMethodSourceBadHistory
	"Change the source of a class method, where the change history is wrong. Should succeed"
	
	DSDelta new addChange: (DSMethodSourceChange new
		setClass: class;
		selector: self selectorA;
		oldSource: self badMethodASource;
		newSource: self newMethodASource;
		oldStamp: self badStamp;
		newStamp: self newStamp;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self newMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self newStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 18:58'!
testChangeNonExistantClassMethodProtocol
	"Classyfy a non-existant method. Should do nothing"
	DSDelta new addChange: (DSMethodProtocolChange new
		setClass: class class;
		selector: self badSelector;
		oldProtocol: self oldProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self deny: (class class selectors includes: self badSelector).
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:02'!
testChangeNonExistantClassMethodSource
	"Change source of unknown class method. Should succeed"
	DSDelta new addChange: (DSMethodSourceChange new
		setClass: class class;
		selector: self badSelector;
		oldSource: '';
		newSource: self badSource;
		oldStamp: self oldStamp;
		newStamp: self badStamp;
	yourself); apply.
	
	self assert: (class class selectors includes: self badSelector).
	self assert: (class class organization categoryOfElement: self badSelector) = Categorizer default.
	self assert: (class class sourceCodeAt: self badSelector) = self badSource.
	self assert: (class class compiledMethodAt: self badSelector) timeStamp = self badStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 18:58'!
testChangeNonExistantInstanceMethodProtocol
	"Classyfy a non-existant method. Should do nothing"
	DSDelta new addChange: (DSMethodProtocolChange new
		setClass: class;
		selector: self badSelector;
		oldProtocol: self oldProtocol;
		newProtocol: self newProtocol;
	yourself); apply.
	
	self deny: (class selectors includes: self badSelector).
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 19:02'!
testChangeNonExistantInstanceMethodSource
	"Change source of unknown instance method. Should succeed"
	DSDelta new addChange: (DSMethodSourceChange new
		setClass: class;
		selector: self badSelector;
		oldSource: '';
		newSource: self badSource;
		oldStamp: self oldStamp;
		newStamp: self badStamp;
	yourself); apply.
	
	self assert: (class selectors includes: self badSelector).
	self assert: (class organization categoryOfElement: self badSelector) = Categorizer default.
	self assert: (class sourceCodeAt: self badSelector) = self badSource.
	self assert: (class compiledMethodAt: self badSelector) timeStamp = self badStamp.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 23:01'!
testChangeSharedPoolsBadHistory
	"Change the shared pools of a class, where the history is wrong. The pools exist, so this should succeed"
	
	DSDelta new addChange: (DSSharedPoolVarsChange new
		className: self oldClassAName;
		oldVars: self badSharedPoolString scanNames;
		newVars: self newSharedPoolString scanNames;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self newSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 10:40'!
testClassRenameConflict
"Rename an existing class to the name of another existing class. Invalid. Not sure what the result should be:
	Give an error?
	Merge the two classes?
	Should methods merged from old class overwrite those in new class having the same name?
	Should instance variables/class variables/shared pools/class instance variables/class comment be moved?
	Should class variables/class instance variables/shared pools/class comment overwrite those in the new class having the same name?
	Should class with the old name be removed?
My best guess as to the most appropriate forceful action is to merge the classes, moving everything from old name to new name, overwriting anything in new name with the old name. I definitely don't want to give an error, because presumably, the user has already been told about this and said 'Yes, I really want to do this, even though it is dumb'."

	self error: 'This test is not yet well-defined'.
	self shouldBeImplemented.

	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassNameChange
		className: self newClassAName
		to: self oldClassAName
	); apply.
	
	self deny:  (Smalltalk includesKey: self newClassAName).
	self deny:  (Smalltalk includesKey: self classBName).
	self assert: (Smalltalk includesKey: self oldClassAName).
	class := Smalltalk classNamed: self newClassAName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self newClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 10:39'!
testCreateClassWithBadPools
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self badSharedPoolString;
		category: self oldClassCategory;
		type: #variable;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self assert: newClass isFixed.
	self deny:  newClass isVariable.
	self assert: newClass isPointers.
	self deny:  newClass isBits.
	self deny:  newClass isBytes.
	self assert: newClass isWords.
	self deny:  newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == self oldSuperclass.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self badSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 22:06'!
testCreateClassWithBadSuperclass
	"Create a Class with a non-existant superclass. Should create the class, but it's superclass should be Object instead'"
	
	| newClass |
	self deny: (Smalltalk includesKey: self classBName).
	DSDelta new addChange: (DSClassCreatedChange new
		className: self classBName;
		superclassName: self badSuperclassName;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldInstVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #variable;
	yourself); apply.
	
	self assert: (Smalltalk includesKey: self classBName).
	newClass := Smalltalk at: self classBName.
	self assert: newClass isBehavior.
	self deny:  newClass isMeta.
	self assert: newClass class isMeta.
	self assert: newClass isFixed.
	self deny:  newClass isVariable.
	self assert: newClass isPointers.
	self deny:  newClass isBits.
	self deny:  newClass isBytes.
	self assert: newClass isWords.
	self deny:  newClass isWeak.
	self assert: newClass name == self classBName.
	self assert: newClass superclass == Object.
	self assert: newClass category == self oldClassCategory.
	self assert: newClass instanceVariablesString  = self oldInstVarString.
	"self assert: class class instanceVariablesString = self oldInstVarString."
	self assert: newClass classVariablesString = self oldClassVarString.
	self assert: newClass sharedPoolsString = self oldSharedPoolString.
	self assert: newClass methodDictionary isEmpty.
	self assert: newClass class methodDictionary isEmpty.
	self assert: newClass organization categories size = 1.
	self assert: newClass organization categories first = Categorizer nullCategory.
	self assert: newClass class organization categories size = 1.
	self assert: newClass class organization categories first = Categorizer nullCategory.
	self deny:  newClass hasComment.! !

!DSDeltaDirtyApplyTest methodsFor: 'class tests' stamp: 'mtf 9/14/2007 10:41'!
testRemoveNonExistantClass
	"Try to remove a class that isn't there. Should do nothing"
	
	self deny:  (Smalltalk includesKey: self classBName).
	self assert: (Smalltalk includesKey: self oldClassAName).
	DSDelta new addChange: (DSClassRemovedChange new
		className: self classBName;
		superclassName: self oldSuperclass name;
		instVarNames: self oldInstVarString;
		"classInstVarNames: self oldLocalVarString;"
		classVarNames: self oldClassVarString;
		poolDictionaryNames: self oldSharedPoolString;
		category: self oldClassCategory;
		type: #normal;
	yourself); apply.
	
	self deny:  (Smalltalk includesKey: self classBName).
	self assert: (Smalltalk includesKey: self oldClassAName).
	class := Smalltalk classNamed: self oldClassAName.
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests(slow)' stamp: 'mtf 9/14/2007 18:08'!
testRemoveNonExistantClassMethod
	"Remove a class-side method, where the method does not exist. Invalid; should do nothing"
	
	self assert: (class class selectors includes: self selectorA).
	self deny: (class class selectors includes: self selectorB).
	DSDelta new addChange: (DSMethodRemovedChange new
		setClass: class class;
		selector: self selectorB;
		stamp: self oldStamp;
		source: self oldMethodASource;
		protocol: self oldProtocol;
	yourself); apply.
	
	self assert: (class class selectors includes: self selectorA).
	self deny: (class class selectors includes: self selectorB).
! !

!DSDeltaDirtyApplyTest methodsFor: 'method tests' stamp: 'mtf 9/14/2007 18:08'!
testRemoveNonExistantInstanceMethod
	"Remove a instance-side method, where the method does not exist. Invalid; should do nothing"
	
	self assert: (class selectors includes: self selectorA).
	self deny: (class selectors includes: self selectorB).
	DSDelta new addChange: (DSMethodRemovedChange new
		setClass: class;
		selector: self selectorB;
		stamp: self oldStamp;
		source: self oldMethodASource;
		protocol: self oldProtocol;
	yourself); apply.
	
	self assert: (class selectors includes: self selectorA).
	self deny: (class selectors includes: self selectorB).
! !

DSTestCase subclass: #DSDeltaEditorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaEditorTest methodsFor: 'as yet unclassified' stamp: 'gk 8/24/2007 00:33'!
setUp! !

!DSDeltaEditorTest methodsFor: 'as yet unclassified' stamp: 'gk 8/24/2007 00:33'!
tearDown
.
	self removeTestClasses! !

!DSDeltaEditorTest methodsFor: 'as yet unclassified' stamp: 'gk 8/29/2007 23:53'!
testManualCreationApplyAndRevert

	| de newComment class className |.
	className := #DSTestClassA.
	newComment := 'A class comment for classA.'.

	de := DSDelta new editor.

	"Make three changes."
	de createClassName: className.
	de comment: newComment.
	de instVars: 'one two'.

	self assert: de delta changes size = 3.
	self deny: (Smalltalk hasClassNamed: className).
	de apply. "should create the class"

	class := Smalltalk classNamed: className.
	self assert: class notNil.

	self assert: class organization classComment = newComment.
	self assert: class instVarNames asSpaceString = 'one two'.

	de revert. "should make the class go poof"

	self deny: (Smalltalk hasClassNamed: className)! !

DSTestCase subclass: #DSDeltaLoggingTest
	instanceVariableNames: 'delta class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaLoggingTest commentStamp: '<historical>' prior: 0!
Testing change logging with a DSDelta.!

!DSDeltaLoggingTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/4/2007 09:32'!
setUp
	delta := DSDelta new.
	class := self createClassAWithMethods.
	self createTestPools.
	delta startLogging.! !

!DSDeltaLoggingTest methodsFor: 'as yet unclassified' stamp: 'gk 8/26/2007 22:33'!
tearDown
	delta stopLogging.
	delta := class := nil.
	self removeTestClasses; removeTestPools! !

!DSDeltaLoggingTest methodsFor: 'as yet unclassified' stamp: 'gk 8/22/2007 00:47'!
testAddedRemoved
	delta from: '' to: 'c d e' do: [:old :new |
		self assert: (old = #()).
		self assert: (new = #(c d e))].

	! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/5/2007 08:27'!
testClassCategory
	| change |
	class category: self newClassCategory.
	self assert: delta changes size = 1.
	
	change := delta changes first.
	self assert: change isClassCategory.
	self assert: change oldCategory = self oldClassCategory.
	self assert: change newCategory = self newClassCategory.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'gk 8/23/2007 22:02'!
testClassComment
	| old new |
	old := class organization classComment.
	new := old, 'NEW'.
	class classComment: new.
	self assert: (delta classCommentChanges size = 1).
	self assert: (delta classCommentChanges first oldComment = old).
	self assert: (delta classCommentChanges first newComment = new)! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'gk 9/3/2007 13:13'!
testClassCreated

	| change |
	self createClassB.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isClassCreated.
	self assert: change className == self classBName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames isEmpty.
	self assert: change classVarNames isEmpty.
	self assert: change poolDictionaryNames isEmpty.
	self assert: change type == #normal.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'gk 9/3/2007 13:13'!
testClassCreatedRemoved

	| classB |
	classB := self createClassB.
	self assert: (delta changes size = 1).
	classB removeFromSystem.
	self assert: (delta changes size = 2).
	
	self assert: delta changes first isClassCreated.
	self assert: delta changes second isClassRemoved.
	
	delta changes do: [ :change |
		self assert: change className == self classBName.
		self assert: change superclassName == self oldSuperclass name.
		self assert: change category == self oldClassCategory.
		self assert: change instVarNames isEmpty.
		self assert: change classVarNames isEmpty.
		self assert: change poolDictionaryNames isEmpty.
		self assert: change type == #normal.
	]! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 14:42'!
testClassInstVars

	| change |
	delta from: '' to: 'c d e' do: [:old :new |
		self assert: (old = #()).
		self assert: (new = #(c d e))].
	
	class class addInstVarName: 'bark'; removeInstVarName: 'bark'.
	self assert: (delta changes size = 2).
	change := delta changes first.
	self assert: change isClassInstVars.
	self assert: change className == self oldClassAName.
	self assert: (change newVars = #(bark)).
	self assert: (change oldVars = #()).
	self assert: (change added = #(bark)).
	self assert: (change removed = #()).
	self assert: (change moved = #()).

	change := delta changes second.
	self assert: change isClassInstVars.
	self assert: change className == self oldClassAName.
	self assert: (change newVars = #()).
	self assert: (change oldVars = #(bark)).
	self assert: (change added = #()).
	self assert: (change removed = #(bark)).
	self assert: (change moved = #()).
! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:00'!
testClassMethodAdded

	| change |
	self createMethodBIn: class class.
	self assert: delta changes size = 1.
	
	change := delta changes first.
	self assert: change isMethodAdded.
	self assert: change selector == self selectorB.
	self assert: change getClass == class class.
	self assert: change className == class theNonMetaClass name.
	self assert: change isMeta.
	self assert: change source = self methodBSource.
	self assert: change stamp = self oldStamp.
	self assert: change protocol = self oldProtocol.
! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:29'!
testClassMethodClassify

	| change |
	class class organization classify: self selectorA under: self newProtocol.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isMethodProtocol.
	self assert: change selector == self selectorA.
	self assert: change getClass == class class.
	self assert: change className == class theNonMetaClass name.
	self assert: change isMeta.
	self assert: change oldProtocol = self oldProtocol.
	self assert: change newProtocol = self newProtocol.! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:18'!
testClassMethodRemoved

	| change |
	class class removeSelector: self selectorA.
	self assert: delta changes size = 1.
	
	change := delta changes first.
	self assert: change isMethodRemoved.
	self assert: change selector == self selectorA.
	self assert: change getClass == class class.
	self assert: change className == class theNonMetaClass name.
	self assert: change isMeta.
	self assert: change source = self oldMethodASource.
	self assert: change stamp = self oldStamp.
	self assert: change protocol = self oldProtocol.! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:12'!
testClassMethodSourceAndProtocolChange

	| change |
	class class
		compile: self newMethodASource
		classified: self newProtocol
		withStamp: self newStamp
		notifying: nil.
	self assert: delta changes size = 2.

	change := delta changes first.
	self assert: change isMethodProtocol.
	self assert: change selector == self selectorA.
	self assert: change getClass == class class.
	self assert: change className == class theNonMetaClass name.
	self assert: change isMeta.
	self assert: change oldProtocol = self oldProtocol.
	self assert: change newProtocol = self newProtocol.

	change := delta changes second.
	self assert: change isMethodSource.
	self assert: change selector == self selectorA.
	self assert: change getClass == class class.
	self assert: change className == class theNonMetaClass name.
	self assert: change isMeta.
	self assert: change oldSource = self oldMethodASource.
	self assert: change newSource = self newMethodASource.
	self assert: change oldStamp = self oldStamp.
	self assert: change newStamp = self newStamp.! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:19'!
testClassMethodSourceChange

	| change |
	class class
		compile: self newMethodASource
		classified: self oldProtocol
		withStamp: self newStamp
		notifying: nil.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isMethodSource.
	self assert: change selector == self selectorA.
	self assert: change getClass == class class.
	self assert: change className == class theNonMetaClass name.
	self assert: change isMeta.
	self assert: change oldSource = self oldMethodASource.
	self assert: change newSource = self newMethodASource.
	self assert: change oldStamp = self oldStamp.
	self assert: change newStamp = self newStamp.
! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'gk 8/20/2007 23:20'!
testClassName
	| old new |
	old := class name.
	new := old, 'NEW'.
	class rename: new.
	self assert: (delta classNameChanges size = 1).
	self assert: (delta classNameChanges first oldName = old).
	self assert: (delta classNameChanges first newName = new)! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 14:07'!
testClassRemoved
	| compositeChange change |
	class removeFromSystem.
	self assert: (delta classRemovedChanges size = 1).
	
	compositeChange := delta changes first.
	self assert: compositeChange isClassRemoved.
	self assert: compositeChange isComposite.
	self assert: compositeChange changes size = 3.
	
	change := compositeChange changes first.
	self assert: change isMethodRemoved.
	
	change := compositeChange changes second.
	self assert: change isMethodRemoved.
	
	change := compositeChange changes third.
	self assert: change isClassRemoved.
	self deny:  change isComposite.
	self assert: change className == self oldClassAName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames isEmpty.
	self assert: change classVarNames isEmpty.
	self assert: change poolDictionaryNames isEmpty.
	self assert: change type == #normal.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'gk 8/21/2007 00:06'!
testClassSuperclass
	|  oldSuper |
	oldSuper := class superclass. 
	class changeSuperclass: Model. "just any one"
	self assert: (delta classSuperclassChanges size = 1).
	self assert: (delta classSuperclassChanges first oldSuperclass = oldSuper).
	self assert: (delta classSuperclassChanges first newSuperclass = Model)! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:54'!
testClassVars

	| change |
	class classVariableNames: 'A B C D'.
	self assert: (delta changes size = 1).
	change := delta changes first.
	self assert: (change newVars = #(A B C D)).
	self assert: (change oldVars = #()).
	self assert: (change added = #(A B C D)).
	self assert: (change removed = #()).
	self assert: (change moved = #()).

	class classVariableNames: 'C A E B F'.
	change := delta changes second.
	self assert: (change newVars = #(A B C E F)).
	self assert: (change oldVars = #(A B C D)).
	self assert: (change added = #(E F)).
	self assert: (change removed = #(D)).
	self assert: (change kept = #(A B C)).
	self assert: (change moved = #()).
! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'gk 8/22/2007 00:50'!
testInstVars

	| change |
	delta from: '' to: 'c d e' do: [:old :new |
		self assert: (old = #()).
		self assert: (new = #(c d e))].

	class addInstVarName: 'bark'; removeInstVarName: 'bark'.
	self assert: (delta changes size = 2).
	change := delta changes first.
	self assert: (change newVars = #(bark)).
	self assert: (change oldVars = #()).
	self assert: (change added = #(bark)).
	self assert: (change removed = #()).
	self assert: (change moved = #()).

	change := delta changes second.
	self assert: (change newVars = #()).
	self assert: (change oldVars = #(bark)).
	self assert: (change added = #()).
	self assert: (change removed = #(bark)).
	self assert: (change moved = #()).
! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:56'!
testInstVarsComplex

	| change |
	class instanceVariableNames: 'a b c d'.
	self assert: (delta changes size = 1).
	change := delta changes first.
	self assert: (change newVars = #(a b c d)).
	self assert: (change oldVars = #()).
	self assert: (change added = #(a b c d)).
	self assert: (change removed = #()).
	self assert: (change moved = #()).

	class instanceVariableNames: 'c a e b f'.
	change := delta changes second.
	self assert: (change newVars = #(c a e b f)).
	self assert: (change oldVars = #(a b c d)).
	self assert: (change added = #(e f)).
	self assert: (change removed = #(d)).
	self assert: (change kept = #(a b c)).
	self assert: (change moved = #(a b c)).
! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:14'!
testInstanceMethodAdded

	| change |
	self createMethodBIn: class.
	self assert: delta changes size = 1.
	
	change := delta changes first.
	self assert: change isMethodAdded.
	self assert: change selector == self selectorB.
	self assert: change getClass == class.
	self assert: change className == class theNonMetaClass name.
	self deny:  change isMeta.
	self assert: change source = self methodBSource.
	self assert: change stamp = self oldStamp.
	self assert: change protocol = self oldProtocol.
! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:14'!
testInstanceMethodClassify

	| change |
	class organization classify: self selectorA under: self newProtocol.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isMethodProtocol.
	self assert: change selector == self selectorA.
	self assert: change getClass == class.
	self assert: change className == class theNonMetaClass name.
	self deny:  change isMeta.
	self assert: change oldProtocol = self oldProtocol.
	self assert: change newProtocol = self newProtocol.! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:18'!
testInstanceMethodRemoved

	| change |
	class removeSelector: self selectorA.
	self assert: delta changes size = 1.
	
	change := delta changes first.
	self assert: change isMethodRemoved.
	self assert: change selector == self selectorA.
	self assert: change getClass == class.
	self assert: change className == class theNonMetaClass name.
	self deny:  change isMeta.
	self assert: change source = self oldMethodASource.
	self assert: change stamp = self oldStamp.
	self assert: change protocol = self oldProtocol.
! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:13'!
testInstanceMethodSourceAndProtocolChange

	| change |
	class
		compile: self newMethodASource
		classified: self newProtocol
		withStamp: self newStamp
		notifying: nil.
	self assert: delta changes size = 2.

	change := delta changes first.
	self assert: change isMethodProtocol.
	self assert: change selector == self selectorA.
	self assert: change getClass == class.
	self assert: change className == class theNonMetaClass name.
	self deny:  change isMeta.
	self assert: change oldProtocol = self oldProtocol.
	self assert: change newProtocol = self newProtocol.

	change := delta changes second.
	self assert: change isMethodSource.
	self assert: change selector == self selectorA.
	self assert: change getClass == class.
	self assert: change className == class theNonMetaClass name.
	self deny:  change isMeta.
	self assert: change oldSource = self oldMethodASource.
	self assert: change newSource = self newMethodASource.
	self assert: change oldStamp = self oldStamp.
	self assert: change newStamp = self newStamp.! !

!DSDeltaLoggingTest methodsFor: 'method tests' stamp: 'mtf 8/30/2007 11:20'!
testInstanceMethodSourceChange

	| change |
	class
		compile: self newMethodASource
		classified: self oldProtocol
		withStamp: self newStamp
		notifying: nil.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isMethodSource.
	self assert: change selector == self selectorA.
	self assert: change getClass == class.
	self assert: change className == class theNonMetaClass name.
	self deny:  change isMeta.
	self assert: change oldSource = self oldMethodASource.
	self assert: change newSource = self newMethodASource.
	self assert: change oldStamp = self oldStamp.
	self assert: change newStamp = self newStamp.
! !

!DSDeltaLoggingTest methodsFor: 'as yet unclassified' stamp: 'gk 8/20/2007 23:07'!
testLogging
	| d1 |
	d1 := DSDelta new.
	self assert: d1 isLogging not.
	d1 startLogging.
	self assert: d1 isLogging.
	d1 startLogging. "should still be logging"
	self assert: d1 isLogging.
	d1 stopLogging.
	self assert: d1 isLogging not! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 09:25'!
testNormalClassCreated

	| change |
	self oldSuperclass
		subclass: self classBName
		instanceVariableNames: self oldInstVarString
		classVariableNames: self oldClassVarString
		poolDictionaries: self oldSharedPoolString
		category: self oldClassCategory.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isClassCreated.
	self assert: change className == self classBName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames asSpaceString = self oldInstVarString.
	self assert: change classVarNames asSpaceString = self oldClassVarString.
	self assert: change poolDictionaryNames asSpaceString = self oldSharedPoolString.
	self assert: change type == #normal.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:57'!
testSharedPools

	| change |
	class poolDictionaries: 'DSa DSb DSc DSd'.
	self assert: (delta changes size = 1).
	change := delta changes first.
	self assert: (change newVars = #(DSa DSb DSc DSd)).
	self assert: (change oldVars = #()).
	self assert: (change added = #(DSa DSb DSc DSd)).
	self assert: (change removed = #()).
	self assert: (change moved = #()).

	class poolDictionaries: 'DSc DSa DSe DSb DSf'.
	change := delta changes second.
	self assert: (change newVars = #(DSc DSa DSe DSb DSf)).
	self assert: (change oldVars = #(DSa DSb DSc DSd)).
	self assert: (change added = #(DSe DSf)).
	self assert: (change removed = #(DSd)).
	self assert: (change kept = #(DSa DSb DSc)).
	self assert: (change moved = #(DSa DSb DSc)).
! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 09:34'!
testVariableByteClassCreated

	| change |
	self oldSuperclass
		variableByteSubclass: self classBName
		instanceVariableNames: ''
		classVariableNames: self oldClassVarString
		poolDictionaries: self oldSharedPoolString
		category: self oldClassCategory.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isClassCreated.
	self assert: change className == self classBName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames isEmpty.
	self assert: change classVarNames asSpaceString = self oldClassVarString.
	self assert: change poolDictionaryNames asSpaceString = self oldSharedPoolString.
	self assert: change type == #bytes.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 09:26'!
testVariableClassCreated

	| change |
	self oldSuperclass
		variableSubclass: self classBName
		instanceVariableNames: self oldInstVarString
		classVariableNames: self oldClassVarString
		poolDictionaries: self oldSharedPoolString
		category: self oldClassCategory.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isClassCreated.
	self assert: change className == self classBName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames asSpaceString = self oldInstVarString.
	self assert: change classVarNames asSpaceString = self oldClassVarString.
	self assert: change poolDictionaryNames asSpaceString = self oldSharedPoolString.
	self assert: change type == #variable.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 09:35'!
testVariableWordClassCreated

	| change |
	self oldSuperclass
		variableWordSubclass: self classBName
		instanceVariableNames: ''
		classVariableNames: self oldClassVarString
		poolDictionaries: self oldSharedPoolString
		category: self oldClassCategory.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isClassCreated.
	self assert: change className == self classBName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames isEmpty.
	self assert: change classVarNames asSpaceString = self oldClassVarString.
	self assert: change poolDictionaryNames asSpaceString = self oldSharedPoolString.
	self assert: change type == #words.! !

!DSDeltaLoggingTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 09:27'!
testWeakClassCreated

	| change |
	self oldSuperclass
		weakSubclass: self classBName
		instanceVariableNames: self oldInstVarString
		classVariableNames: self oldClassVarString
		poolDictionaries: self oldSharedPoolString
		category: self oldClassCategory.
	self assert: delta changes size = 1.

	change := delta changes first.
	self assert: change isClassCreated.
	self assert: change className == self classBName.
	self assert: change superclassName == self oldSuperclass name.
	self assert: change category == self oldClassCategory.
	self assert: change instVarNames asSpaceString = self oldInstVarString.
	self assert: change classVarNames asSpaceString = self oldClassVarString.
	self assert: change poolDictionaryNames asSpaceString = self oldSharedPoolString.
	self assert: change type == #weak.! !

DSTestCase subclass: #DSDeltaNormalizeTest
	instanceVariableNames: 'class delta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaNormalizeTest methodsFor: 'as yet unclassified' stamp: 'gk 9/7/2007 00:09'!
setUp
	delta := DSDelta new.
	class := self createClassAWithMethods.
	self createTestPools.
	delta startLogging.! !

!DSDeltaNormalizeTest methodsFor: 'as yet unclassified' stamp: 'gk 9/7/2007 00:09'!
tearDown
	delta stopLogging.
	delta := class := nil.
	self removeTestClasses; removeTestPools! !

!DSDeltaNormalizeTest methodsFor: 'as yet unclassified' stamp: 'gk 9/7/2007 00:24'!
testCreateAndRemove

	self createClassB; removeTestClasses; createClassB.
	delta stopLogging; normalize.
	self assert: delta changes size = 2.
	self assert: delta changes first isClassDeleted.
	self assert: delta changes second isClassCreated.! !

!DSDeltaNormalizeTest methodsFor: 'as yet unclassified(slow)' stamp: 'gk 9/7/2007 16:35'!
testMoveClassNameChanges

	| de |
	delta stopLogging.
	de := delta editor.

	"Make three changes."
	de createClassName: 'A'; comment: 'A comment'; instVars: 'one two'.
	de createClassName: 'Z'; comment: 'Z comment'.

	de delta renameClassFrom: 'A' to: 'B'.
	de className: 'B'; instVars: 'one two'.

	de delta renameClassFrom: 'B' to: 'C'.
	de delta renameClassFrom: 'C' to: 'D'.

	delta moveClassNameChanges.
self halt. 
	self assert: delta changes size = 2.
	self assert: delta changes first isClassDeleted.
	self assert: delta changes second isClassCreated.! !

DSTestCase subclass: #DSDeltaRevertTest
	instanceVariableNames: 'delta class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Tests'!

!DSDeltaRevertTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/4/2007 14:01'!
expectedFailures
	"Changing class type at runtime cannot be done in Squeak, but DeltaStreams
	can log such changes anyway"
	^ #(
		testClassTypeVariable
		testClassTypeVariableByte
		testClassTypeVariableWord
		testClassTypeWeak
	)! !

!DSDeltaRevertTest methodsFor: 'as yet unclassified' stamp: 'mtf 8/31/2007 14:32'!
setUp
	delta := DSDelta new.
	self createTestPools.
	class := self createClassAFull.
	delta startLogging.! !

!DSDeltaRevertTest methodsFor: 'as yet unclassified' stamp: 'mtf 8/31/2007 14:32'!
tearDown
	delta stopLogging.
	delta := class := nil.
	self removeTestClasses; removeTestPools! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 21:59'!
testClassAdded
	self createClassB.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 9/5/2007 08:14'!
testClassCategory
	class category: self newClassCategory.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:04'!
testClassComment
	class classComment: self newComment.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:56'!
testClassInstVars
	class instanceVariableNames: self newInstVarString.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:22'!
testClassMethodAdded
	self createMethodBIn: class class.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:28'!
testClassMethodClassify
	class class organization classify: self selectorA under: self newProtocol.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:34'!
testClassMethodProtocol
	class class
		compile: self oldMethodASource
		classified: self newProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:29'!
testClassMethodRemove
	class class removeSelector: self selectorA.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:33'!
testClassMethodSource
	class class
		compile: self newMethodASource
		classified: self oldProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:33'!
testClassMethodSourceAndProtocol
	class class
		compile: self newMethodASource
		classified: self newProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:32'!
testClassMethodStamp
	class class
		compile: self oldMethodASource
		classified: self oldProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:08'!
testClassName
	class rename: self newClassAName.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 14:35'!
testClassRemove
	class removeFromSystem.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:20'!
testClassTypeVariable
	self changeTypeTo: #variable in: class.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:20'!
testClassTypeVariableByte
	self changeTypeTo: #bytes in: class.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:20'!
testClassTypeVariableWord
	self changeTypeTo: #words in: class.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:20'!
testClassTypeWeak
	self changeTypeTo: #weak in: class.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:55'!
testClassVars
	class classVariableNames: self newClassVarString.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:56'!
testInstVars
	class instanceVariableNames: self newInstVarString.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 9/12/2007 10:27'!
testInstanceMethodAdded
	self createMethodBIn: class.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:29'!
testInstanceMethodClassify
	class organization classify: self selectorA under: self newProtocol.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests(slow)' stamp: 'gk 9/4/2007 23:42'!
testInstanceMethodProtocol
	class
		compile: self oldMethodASource
		classified: self newProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
self halt: 'This test fails because the empty newProtocol is not removed when we change the category of the method back to aProtocol'.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 9/12/2007 10:27'!
testInstanceMethodRemove
	class removeSelector: self selectorA.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:31'!
testInstanceMethodSource
	class
		compile: self newMethodASource
		classified: self oldProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:31'!
testInstanceMethodSourceAndProtocol
	class
		compile: self newMethodASource
		classified: self newProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'method tests' stamp: 'mtf 8/31/2007 22:32'!
testInstanceMethodStamp
	class
		compile: self oldMethodASource
		classified: self oldProtocol
		withStamp: self newStamp
		notifying: nil.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'as yet unclassified' stamp: 'mtf 9/4/2007 09:53'!
testNoChange
	"Make sure the system is exactly the way it was at the beginning of the test"

	"System Tests"
	self assert: (Smalltalk includesKey: self oldClassAName).
	self deny:  (Smalltalk includesKey: self newClassAName).
	self deny:  (Smalltalk includesKey: self classBName).

	"Class Tests"
	class := (Smalltalk classNamed: self oldClassAName).
	self assert: class isBehavior.
	self deny:  class isMeta.
	self assert: class class isMeta.
	self assert: class isFixed.
	self assert: class name == self oldClassAName.
	self assert: class superclass == self oldSuperclass.
	self assert: class category == self oldClassCategory.
	self assert: class instanceVariablesString  = self oldInstVarString.
	self assert: class class instanceVariablesString = self oldInstVarString.
	self assert: class classVariablesString = self oldClassVarString.
	self assert: class sharedPoolsString = self oldSharedPoolString.
	self assert: class methodDictionary size = 1.
	self assert: class class methodDictionary size = 1.

	"Organization Tests"
	self assert: class organization categories size = 1.
	self assert: class organization categories first = self oldProtocol.
	self assert: class class organization categories size = 1.
	self assert: class class organization categories first = self oldProtocol.
	self assert: class hasComment.
	self assert: class organization classComment = self oldComment.
	self assert: class organization commentStamp = self oldStamp.
	
	"Instance method test"
	self assert: (class selectors includes: self selectorA).
	self assert: (class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class compiledMethodAt: self selectorA) timeStamp = self oldStamp.
	
	"Class method test"
	self assert: (class class selectors includes: self selectorA).
	self assert: (class class organization categoryOfElement: self selectorA) = self oldProtocol.
	self assert: (class class sourceCodeAt: self selectorA) = self oldMethodASource.
	self assert: (class class compiledMethodAt: self selectorA) timeStamp = self oldStamp.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 9/4/2007 10:57'!
testSharedPools
	class poolDictionaries: self newSharedPoolString.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSDeltaRevertTest methodsFor: 'class tests' stamp: 'mtf 8/31/2007 22:08'!
testSuperclass
	class changeSuperclass: self newSuperclass.
	delta stopLogging.
	delta revert.
	self testNoChange.! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:51'!
badClassAName
	^ #DSTestClassAWithABadName! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:51'!
badClassCategory
	^ #'DeltaStreams-TestClasses-bad'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/17/2007 11:26'!
badClassName
	^ #DSTestClassThatDoesntExist! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:52'!
badClassVarString
	^ 'BadVar' ! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:50'!
badComment
	^ 'This comment is not what it should be'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:52'!
badInstVarString
	^ 'badVar'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:53'!
badMethodASource
	^ 'methodA ^ #no'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 19:08'!
badOldSource
	^ 'theMethodThatIsnt ^ #old'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:54'!
badProtocol
	^ #theBadProtocol! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:59'!
badSelector
	^ #theMethodThatIsnt! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/13/2007 20:37'!
badSharedPoolString
	^ 'DSBadPool'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 19:08'!
badSource
	^ 'theMethodThatIsnt ^ #new'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:55'!
badStamp
	^ 'nil 4/01/1337 13:37'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 9/14/2007 18:56'!
badSuperclassName
	^ #ThereIsNoSuperclassByThisName! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/31/2007 22:19'!
changeTypeTo: aSymbol in: aClass 

	^(ClassBuilder new)
		name: aClass name
		inEnvironment: aClass environment
		subclassOf: aClass superclass
		type: aSymbol
		instanceVariableNames: aClass instanceVariablesString
		classVariableNames: aClass class instanceVariablesString
		poolDictionaries: aClass sharedPoolsString
		category: aClass category! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:34'!
classBName
	^ #DSTestClassB! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/23/2007 13:50'!
compileMethod: source class: aClass category: category
	"Adapted from MCTestCase"
	aClass compile: source classified: category! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/30/2007 10:23'!
createClassA
	^ self createClassNamed: self oldClassAName superClass: self oldSuperclass
! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/31/2007 13:16'!
createClassAFull
	| class |
	class := self oldSuperclass
		subclass: self oldClassAName
		instanceVariableNames: self oldInstVarString
		classVariableNames: self oldClassVarString
		poolDictionaries: self oldSharedPoolString
		category: self oldClassCategory.
	class class instanceVariableNames: self oldInstVarString.
	class classComment: self oldComment stamp: self oldStamp.
	self createMethodAIn: class.
	self createMethodAIn: class class.
	^ class! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/30/2007 10:53'!
createClassAWithMethods
	| class |
	class := self createClassA.
	self createMethodAIn: class.
	self createMethodAIn: class class.
	^ class! !

!DSTestCase methodsFor: 'private' stamp: 'gk 8/20/2007 22:38'!
createClassB
	^ self createClassNamed: #DSTestClassB superClass: Object! !

!DSTestCase methodsFor: 'private' stamp: 'gk 8/18/2007 11:46'!
createClassNamed: aClassname 
	^ self createClassNamed: aClassname superClass: Object! !

!DSTestCase methodsFor: 'private' stamp: 'gk 8/18/2007 11:46'!
createClassNamed: aClassname superClass: aClass 

	^self createClassNamed: aClassname superClass: aClass instanceVariables: ''! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/30/2007 10:04'!
createClassNamed: aClassname superClass: aClass instanceVariables: instvarString

	^aClass
		subclass: aClassname
		instanceVariableNames: instvarString
		classVariableNames: ''
		poolDictionaries: ''
		category: self oldClassCategory! !

!DSTestCase methodsFor: 'private' stamp: 'test 8/26/2007 00:00'!
createMethod: aSelector toReturn: anObject class: aClass category: aCategory
	"Adapted from MCTestCase"
	self 
		compileMethod: aSelector, ' ^ ', anObject printString
		class: aClass
		category: aCategory.
	^ aSelector! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/30/2007 10:53'!
createMethodAIn: aClass
	^ aClass
		compile: self oldMethodASource
		classified: self oldProtocol
		withStamp: self oldStamp
		notifying: nil
! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/30/2007 10:53'!
createMethodBIn: aClass
	^ aClass
		compile: self methodBSource
		classified: self oldProtocol
		withStamp: self oldStamp
		notifying: nil
! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 8/30/2007 10:50'!
createTestPools

	#(a b c d e f) do: [:sym | Smalltalk at: (#DS, sym) asSymbol put: Dictionary new].! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:51'!
methodBSource
	^ 'methodB ^ 5'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:35'!
newClassAName
	^ #DSTestClassAWithANewName! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 12:41'!
newClassCategory
	^ #'DeltaStreams-TestClasses-new'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 10:23'!
newClassVarString
	^ 'C A E B F' ! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:35'!
newComment
	^ 'This comment is new'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 12:43'!
newInstVarString
	^ 'c a e b f'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:36'!
newMethodASource
	^ 'methodA ^ 2'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:37'!
newProtocol
	^ #theNewProtocol! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 10:23'!
newSharedPoolString
	^ 'DSc DSa DSe DSb DSf' ! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:37'!
newStamp
	^ 'n00b 4/31/2007 23:45'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:38'!
newSuperclass
	^ Model! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:38'!
oldClassAName
	^ #DSTestClassA! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 12:40'!
oldClassCategory
	^ #'DeltaStreams-TestClasses'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 10:24'!
oldClassVarString
	^ 'A B C D'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:38'!
oldComment
	^ 'This comment is old'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 12:44'!
oldInstVarString
	^ 'a b c d'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:40'!
oldMethodASource
	^ 'methodA ^ 1'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:41'!
oldProtocol
	^ #aProtocol! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/31/2007 10:24'!
oldSharedPoolString
	^ 'DSa DSb DSc DSd'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:41'!
oldStamp
	^ 'elder 3/21/1996 12:34'! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:37'!
oldSuperclass
	^ Object! !

!DSTestCase methodsFor: 'private' stamp: 'gk 8/26/2007 22:30'!
removeTestClasses

	Smalltalk flushClassNameCache.
	Smalltalk do: [:cl | (cl name beginsWith: #DSTestClass) ifTrue: [cl removeFromSystem]]! !

!DSTestCase methodsFor: 'private' stamp: 'mtf 9/13/2007 20:36'!
removeTestPools

	#(a b c d e f BadPool) do: [:sym | Smalltalk removeKey: (#DS, sym) asSymbol ifAbsent: []]! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:36'!
selectorA
	^ #methodA! !

!DSTestCase methodsFor: 'mocks' stamp: 'mtf 8/30/2007 09:34'!
selectorB
	^ #methodB! !

Exception subclass: #DSDeltaNotRevertableException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Base'!

Object subclass: #DSChange
	instanceVariableNames: 'timeStamp number'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Base'!

!DSChange commentStamp: '<historical>' prior: 0!
An abstract super class for all fine granular changes that a DSDelta holds. A DSChange object should only hold "simple data" and not refer to the classes themselves. This makes them easy to construct, manipulate and serialize without affecting (or need) the real classes.!

!DSChange methodsFor: 'comparing' stamp: 'gk 9/7/2007 00:29'!
<= aChange
	"Sort by number per default."

	^number <= aChange number! !

!DSChange methodsFor: 'anti' stamp: 'mtf 8/31/2007 11:07'!
antiChangeClass
	^ self class! !

!DSChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 13:03'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	anApplier applyChange: self! !

!DSChange methodsFor: 'anti' stamp: 'gk 8/23/2007 17:59'!
asAntiChange
	"Return a new Change that reverts this one."

	self subclassResponsibility! !

!DSChange methodsFor: 'testing' stamp: 'gk 9/7/2007 10:37'!
canShadow
	"Can a change of this kind shadow other previous changes?"

	^false! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:14'!
classGroupKey
"see the comment in #inClassGroupFor:. Changes are not groupable by default, so return nil"
	^ nil! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:19'!
inClassGroupFor: aChange
"Answers true if I am an the same class as aChange, false otherwise"
	| myKey otherKey |
	myKey := self classGroupKey ifNil: [^ false].
	otherKey := aChange classGroupKey ifNil: [^ false].
	^ myKey = otherKey! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:18'!
inClassGroupFor: aChange withClassName: className
"Answers true if I am an the same class as aChange, false otherwise. For the purposes of comparison, aChange is assumed to have the given class name rather than its actual class name to work around renaming issues"
	| myKey otherKey |
	myKey := self classGroupKey ifNil: [^ false].
	otherKey := aChange classGroupKey ifNil: [^ false].
	^ myKey = (otherKey className: className)! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:20'!
inMergeGroupFor: aChange
"See the comment in #inMergeGroupFor:withClassName:"
	| myKey otherKey |
	myKey := self mergeGroupKey ifNil: [^ false].
	otherKey := aChange mergeGroupKey ifNil: [^ false].
	^ myKey = otherKey! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:19'!
inMergeGroupFor: aChange withClassName: className
"Answers true if both aChange and I:
	1. are primitive changes (ie not composite)
	2. are of the same class
	3. are non-shadowing changes
	4. satisfy all the conditions listed in the comment of #inMethodGroupFor:withClassName:
	
As in #inClassGroupFor:withClassName:, aChange is treated as having the given name rather its real name to work around class renames

If I answer true, aChange and I can be merged into a single change, though not necesarilly cleanly. Shadowing changes (additions and removals) do not have the before/after symmetry required to be in this equivilance group"
	| myKey otherKey |
	myKey := self mergeGroupKey ifNil: [^ false].
	otherKey := aChange mergeGroupKey ifNil: [^ false].
	^ myKey = (otherKey className: className)! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:21'!
inMethodGroupFor: aChange
"See the comment in #inMethodGroupFor:withClassChange: for the meaning of this method"
	| myKey otherKey |
	myKey := self methodGroupKey ifNil: [^ false].
	otherKey := aChange methodGroupKey ifNil: [^ false].
	^ myKey = otherKey! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:20'!
inMethodGroupFor: aChange withClassName: className
"Answers true if I am in the same method group as aChange. I answer true on two conditions: Either:
	1. both aChange and I are method changes, and we are both part of the same class or metaclass and have the same selector, or
	2. both aChange and I are class changes, are not method changes, and are part of the same class/metaclass pair
	
As in #inClassGroupFor:withClassName:, aChange is treated as having the given name rather its real name to work around class renames"
	| myKey otherKey |
	myKey := self methodGroupKey ifNil: [^ false].
	otherKey := aChange methodGroupKey ifNil: [^ false].
	^ myKey = (otherKey className: className)! !

!DSChange methodsFor: 'initialize-release' stamp: 'gk 9/6/2007 23:24'!
initialize
	timeStamp := TimeStamp now! !

!DSChange methodsFor: 'testing' stamp: 'mtf 9/12/2007 11:32'!
isChange
	^true! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:45'!
isClassAdded
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 9/7/2007 00:00'!
isClassAddedOrRemoved
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 9/5/2007 08:16'!
isClassCategory
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:45'!
isClassChange
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:45'!
isClassComment
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 9/3/2007 13:20'!
isClassCreated
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 9/7/2007 00:24'!
isClassDeleted
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:06'!
isClassInstVars
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:45'!
isClassName
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:45'!
isClassRemoved
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:45'!
isClassSuperclass
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/21/2007 02:27'!
isClassType
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:06'!
isClassVars
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:22'!
isComposite
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/15/2007 01:36'!
isDoIt
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:06'!
isInstVars
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/23/2007 16:55'!
isMethodAdded
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/23/2007 16:54'!
isMethodChange
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/23/2007 16:55'!
isMethodName
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/23/2007 16:56'!
isMethodProtocol
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/23/2007 16:55'!
isMethodRemoved
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 13:45'!
isMethodSource
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 21:56'!
isName
	^false! !

!DSChange methodsFor: 'testing' stamp: 'mtf 9/12/2007 18:17'!
isRecord
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/17/2007 08:09'!
isRevertable
	"By default we answer yes."

	^true! !

!DSChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:07'!
isSharedPools
	^false! !

!DSChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:44'!
isSuperclass
	^false! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:14'!
mergeGroupKey
"see the comment in #inMergeGroupFor:. Changes are not groupable by default, so return nil"
	^ nil! !

!DSChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:13'!
methodGroupKey
"see the comment in #inMethodGroupFor:. Changes are not groupable by default, so return nil"
	^ nil! !

!DSChange methodsFor: 'accessing' stamp: 'gk 9/7/2007 00:28'!
number
	^number! !

!DSChange methodsFor: 'accessing' stamp: 'mtf 10/9/2007 14:42'!
number: aNumber
	number := aNumber! !

!DSChange methodsFor: 'private' stamp: 'gk 9/7/2007 16:18'!
remapClassNames: map
	"By default we do nothing. Subclasses reimplement."! !

!DSChange methodsFor: 'accessing' stamp: 'gk 9/6/2007 23:24'!
timeStamp
	^timeStamp! !

DSChange subclass: #DSClassChange
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassChange commentStamp: '<historical>' prior: 0!
An abstract super class for all actions affecting a specific class. A class change do not hold onto any "real" objects, like for example the class itself, instead we refer to it by name. This means a class change can be created even though the class does not exist.!

DSClassChange subclass: #DSClassCategoryChange
	instanceVariableNames: 'oldCategory newCategory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassCategoryChange class methodsFor: 'as yet unclassified' stamp: 'mtf 9/5/2007 08:32'!
class: aClass from: oldCategory to: newCategory
	^(self class: aClass)
		oldCategory: oldCategory;
		newCategory: newCategory;
		yourself! !

!DSClassCategoryChange class methodsFor: 'as yet unclassified' stamp: 'mtf 9/5/2007 08:31'!
className: aName from: oldCategory to: newCategory
	^(self className: aName)
		oldCategory: oldCategory;
		newCategory: newCategory;
		yourself! !

!DSClassCategoryChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 13:31'!
applyUsing: anApplier
	^ anApplier applyClassCategoryChanged: self! !

!DSClassCategoryChange methodsFor: 'anti' stamp: 'mtf 9/5/2007 08:21'!
asAntiChange
	^ self antiChangeClass
		className: className
		from: newCategory
		to: oldCategory! !

!DSClassCategoryChange methodsFor: 'testing' stamp: 'mtf 9/5/2007 08:18'!
isClassCategory
	^ true! !

!DSClassCategoryChange methodsFor: 'accessing' stamp: 'mtf 9/5/2007 08:18'!
newCategory
	^ newCategory! !

!DSClassCategoryChange methodsFor: 'accessing' stamp: 'mtf 9/5/2007 08:39'!
newCategory: aString
	newCategory := aString asSymbol! !

!DSClassCategoryChange methodsFor: 'accessing' stamp: 'mtf 9/5/2007 08:18'!
oldCategory
	^ oldCategory! !

!DSClassCategoryChange methodsFor: 'accessing' stamp: 'mtf 9/5/2007 08:40'!
oldCategory: aString
	oldCategory := aString asSymbol! !

!DSClassChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 23:12'!
class: aClass
	^self new initializeFrom: aClass; yourself! !

!DSClassChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/29/2007 00:31'!
className: aClassName
	^self new className: aClassName; yourself! !

!DSClassChange class methodsFor: 'as yet unclassified' stamp: 'mtf 9/13/2007 13:47'!
fromChange: aChange
	^ self new className: aChange className! !

!DSClassChange methodsFor: 'comparing' stamp: 'mtf 9/13/2007 14:27'!
= aChange
"This is mostly used for grouping (see the methods in the grouping category). This is able to compare group keys, but not regular instances. 

Keys are regular instances of DSClassChange and subclasses, but have no variables other than className set. Class group keys and method group keys only depend on className, but merge group keys depend on the class of the change as well, so compare that"

	^ self class == aChange class and: [className == aChange className]! !

!DSClassChange methodsFor: 'printing' stamp: 'mtf 9/26/2007 16:16'!
action
	^ 'changed'! !

!DSClassChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 13:31'!
applyUsing: anApplier
	^ anApplier applyClassChange: self! !

!DSClassChange methodsFor: 'testing' stamp: 'gk 9/7/2007 10:37'!
canShadow
	"Can a change of this kind shadow other previous changes?"

	^true! !

!DSClassChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 13:51'!
classGroupKey
	"See the comment in DSChange>>inClassGroupFor:"
	^ DSClassChange fromChange: self! !

!DSClassChange methodsFor: 'accessing' stamp: 'gk 8/28/2007 23:04'!
className
	^className! !

!DSClassChange methodsFor: 'accessing' stamp: 'mtf 8/28/2007 18:09'!
className: aString
	className := aString! !

!DSClassChange methodsFor: 'accessing' stamp: 'mtf 9/13/2007 19:41'!
getClass
	^ self getClassFrom: Smalltalk! !

!DSClassChange methodsFor: 'accessing' stamp: 'mtf 9/13/2007 19:50'!
getClassFrom: environment
	| class |
	class := environment at: self className.
	^ self isMeta
		ifTrue: [class class]
		ifFalse: [class]! !

!DSClassChange methodsFor: 'comparing' stamp: 'mtf 9/13/2007 14:23'!
hash
"This is just for grouping. See the comment in #="
	^ self class hash bitXor: className hash! !

!DSClassChange methodsFor: 'initialize-release' stamp: 'gk 8/28/2007 23:11'!
initializeFrom: aClass

	className := aClass name! !

!DSClassChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:48'!
isClassChange
	^true! !

!DSClassChange methodsFor: 'testing' stamp: 'mtf 9/4/2007 09:57'!
isMeta
	^false! !

!DSClassChange methodsFor: 'testing' stamp: 'gk 9/7/2007 10:39'!
isSameClass: anotherChange

	^className = anotherChange className! !

!DSClassChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:32'!
mergeGroupKey
"See the comment in DSChange>>inMergeGroupFor: Creates a blank instance of this class; only className is set; all other vars are nil. This sends the class message to determine the class to create rather than using a well-known class, like class group key and method group key do. Thus, merge group keys are specific to the change type, rather than generic for all class changes"
	^ self class new className: className! !

!DSClassChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 13:52'!
methodGroupKey
	"See the comment in DSChange>>inMethodGroupFor:"
	^ self classGroupKey! !

!DSClassChange methodsFor: 'printing' stamp: 'mtf 9/26/2007 16:19'!
printOn: aStream
	aStream nextPutAll: self className.
	self isMeta ifTrue: [aStream nextPutAll: ' class'].
	aStream space; nextPutAll: self action! !

!DSClassChange methodsFor: 'private' stamp: 'gk 9/7/2007 16:19'!
remapClassNames: map
	"Remap className."

	className := map at: className ifAbsent: [className]! !

DSClassChange subclass: #DSClassCommentChange
	instanceVariableNames: 'newComment oldComment newStamp oldStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassCommentChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 23:30'!
class: aClass from: oldComment to: newComment oldStamp: oldStamp newStamp: newStamp
	^(self class: aClass) oldComment: oldComment; newComment: newComment;
		oldStamp: oldStamp; newStamp: newStamp; yourself! !

!DSClassCommentChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 22:42'!
className: aName from: oldComment to: newComment oldStamp: oldStamp newStamp: newStamp
	^(self className: aName) oldComment: oldComment; newComment: newComment;
		oldStamp: oldStamp; newStamp: newStamp; yourself! !

!DSClassCommentChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 13:31'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^ anApplier applyClassCommentChanged: self! !

!DSClassCommentChange methodsFor: 'anti' stamp: 'gk 9/3/2007 22:31'!
asAntiChange
	"Reverse it."

	^self antiChangeClass className: className from: newComment to: oldComment oldStamp: newStamp newStamp: oldStamp! !

!DSClassCommentChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:47'!
isClassComment
	^ true! !

!DSClassCommentChange methodsFor: 'testing' stamp: 'gk 9/7/2007 15:31'!
isShadowing: anotherChange
	"The simplest logic - last one wins."

	^(self isSameClass: anotherChange) and: [ anotherChange isClassComment]! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:44'!
newComment
	^newComment! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:25'!
newComment: aString
	newComment := aString! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:45'!
newStamp
	^newStamp! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:45'!
newStamp: aString
	newStamp := aString! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:44'!
oldComment
	^oldComment! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:25'!
oldComment: aString
	oldComment := aString! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:45'!
oldStamp
	^oldStamp! !

!DSClassCommentChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:45'!
oldStamp: aString
	oldStamp := aString! !

DSClassChange subclass: #DSClassFullChange
	instanceVariableNames: 'superclassName instVarNames classVarNames poolDictionaryNames category type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassFullChange commentStamp: '<historical>' prior: 0!
This action must hold all information that is mandatory to supply in order to create (or remove) a class - which means quite a lot of fields. When we later change a class (the DSClassChanged* classes) we are much more fine granular.

Note that this change only creates the class - it does not add methods or a class comment. See DSCompositeClassAddedChange.!

DSClassFullChange subclass: #DSClassCreatedChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassCreatedChange commentStamp: 'mtf 9/12/2007 11:12' prior: 0!
This action must hold all information that is mandatory to supply in order to create a class - which means quite a lot of fields.
When we later change a class (the DSClassChanged* classes) we are much more fine granular.

Note that this change only creates the class - it does not add methods or a class comment. See DSCompositeClassAddedChange.!

!DSClassCreatedChange methodsFor: 'anti' stamp: 'mtf 8/31/2007 11:01'!
antiChangeClass
	^ DSClassRemovedChange! !

!DSClassCreatedChange methodsFor: 'applying' stamp: 'gk 9/3/2007 13:11'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applyClassCreated: self! !

!DSClassCreatedChange methodsFor: 'testing' stamp: 'gk 9/3/2007 13:20'!
isClassAdded
	^true! !

!DSClassCreatedChange methodsFor: 'testing' stamp: 'gk 9/3/2007 13:12'!
isClassCreated
	^true! !

!DSClassCreatedChange methodsFor: 'testing' stamp: 'gk 9/7/2007 10:41'!
isShadowing: anotherChange
	"A create removes all previous creates, adds or removes."

	^(self isSameClass: anotherChange) and: [anotherChange isClassAddedOrRemoved]! !

!DSClassFullChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/29/2007 00:32'!
className: aClassName
	^self new className: aClassName; initializeFromNone; yourself! !

!DSClassFullChange methodsFor: 'anti' stamp: 'gk 9/3/2007 13:06'!
asAntiChange
	"The anti change has the same information."

	^ self antiChangeClass new
		className: className;
		superclassName: superclassName;
		category: category;
		instVarNames: instVarNames;
		classVarNames: classVarNames;
		poolDictionaryNames: poolDictionaryNames;
		type: type;
		yourself! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:48'!
category
	^category! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'mtf 9/13/2007 22:15'!
category: aCategory
	category := aCategory ifNotNil: [aCategory asSymbol]! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:47'!
classVarNames
	^classVarNames! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 23:32'!
classVarNames: aStringOrArray

	classVarNames := aStringOrArray scanNames sort! !

!DSClassFullChange methodsFor: 'initialize-release' stamp: 'mtf 8/31/2007 10:53'!
initializeFrom: aClass
	"We extract all needed attributes from the class so that we can recreate it."

	super initializeFrom: aClass.
	superclassName := aClass superclass name.
	instVarNames := aClass instVarNames.
	classVarNames := aClass classVarNames asArray sort.
	poolDictionaryNames := aClass poolDictionaryNames asArray.
	category := aClass category.
	type := aClass typeOfClass! !

!DSClassFullChange methodsFor: 'initialize-release' stamp: 'mtf 9/12/2007 12:22'!
initializeFromNone
	"We use defaults."

	superclassName := #Object.
	instVarNames := ''.
	classVarNames := ''.
	poolDictionaryNames := Array new.
	category := #Unknown.
	type := #normal! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:46'!
instVarNames
	^instVarNames! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 23:31'!
instVarNames: aStringOrArray

	instVarNames := aStringOrArray scanNames! !

!DSClassFullChange methodsFor: 'testing' stamp: 'gk 9/6/2007 23:59'!
isClassAddedOrRemoved
	^true! !

!DSClassFullChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:32'!
mergeGroupKey
"See the comment in DSDelta>>inMergeGroupFor:. My instances are asymmetric and cannot be merged, so return nil"
	^ nil! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:47'!
poolDictionaryNames
	^poolDictionaryNames! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 23:31'!
poolDictionaryNames: aStringOrArray

	poolDictionaryNames := aStringOrArray scanNames! !

!DSClassFullChange methodsFor: 'private' stamp: 'gk 9/7/2007 16:20'!
remapClassNames: map

	super remapClassNames: map.
	superclassName := map at: superclassName ifAbsent: [superclassName]! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:46'!
superclassName
	^superclassName! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 23:19'!
superclassName: aName

	superclassName := aName asSymbol! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:48'!
type
	^type! !

!DSClassFullChange methodsFor: 'accessing' stamp: 'mtf 8/30/2007 14:33'!
type: aSymbol
	type := aSymbol! !

DSClassFullChange subclass: #DSClassRecord
	instanceVariableNames: 'classInstVarNames comment stamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassRecord commentStamp: 'mtf 9/12/2007 18:17' prior: 0!
My instances are responsible for storing the state of the class if there is no other change that can do that. For example, if the only change in a delta is an inst var change, there is no way to get the old class comment. In that case, a record would be logged prior to the inst var change so that the other class data is in the delta. This is not a change.

A record can go in a Delta, and can be applied, but it should do nothing when applied, unless there is no other source available. So, it mostly serves as a marker in a stream of changes!

!DSClassRecord methodsFor: 'applying' stamp: 'mtf 9/21/2007 13:32'!
applyUsing: anApplier
	^ anApplier applyClassSnapshot: self! !

!DSClassRecord methodsFor: 'anti' stamp: 'mtf 9/12/2007 14:51'!
asAntiChange
	^ self copy! !

!DSClassRecord methodsFor: 'convenience' stamp: 'mtf 9/12/2007 12:34'!
classInstVarChange
	^ DSClassInstVarsChange className: className
		from: Array new
		to: classInstVarNames! !

!DSClassRecord methodsFor: 'accessing' stamp: 'mtf 9/12/2007 12:24'!
classInstVarNames
	^ classInstVarNames! !

!DSClassRecord methodsFor: 'accessing' stamp: 'mtf 9/12/2007 12:26'!
classInstVarNames: aStringOrArray
	classInstVarNames := aStringOrArray scanNames! !

!DSClassRecord methodsFor: 'accessing' stamp: 'mtf 9/12/2007 12:24'!
comment
	^ comment! !

!DSClassRecord methodsFor: 'accessing' stamp: 'mtf 9/12/2007 12:27'!
comment: aString
	comment := aString! !

!DSClassRecord methodsFor: 'convenience' stamp: 'mtf 9/12/2007 12:32'!
commentChange
	^ DSClassCommentChange className: className
		from: ''
		to: comment
		oldStamp: ''
		newStamp: stamp! !

!DSClassRecord methodsFor: 'initialize-release' stamp: 'mtf 9/12/2007 12:30'!
initializeFrom: aClass
	super initializeFrom: aClass.
	classInstVarNames := aClass class instVarNames.
	comment := aClass organization classComment.
	stamp := aClass organization commentStamp.! !

!DSClassRecord methodsFor: 'testing' stamp: 'mtf 9/12/2007 11:33'!
isChange
	^ false! !

!DSClassRecord methodsFor: 'testing' stamp: 'mtf 9/12/2007 18:17'!
isRecord
	^ true! !

!DSClassRecord methodsFor: 'accessing' stamp: 'mtf 9/12/2007 12:24'!
stamp
	^ stamp! !

!DSClassRecord methodsFor: 'accessing' stamp: 'mtf 9/12/2007 12:26'!
stamp: aString
	stamp := aString! !

DSClassFullChange subclass: #DSClassRemovedChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassRemovedChange commentStamp: 'mtf 8/31/2007 10:48' prior: 0!
This change represents a removal of an empty class. Removal of a class with methods is more complex and is handled by DSClassRemovedCompositeChange!

!DSClassRemovedChange methodsFor: 'anti' stamp: 'gk 9/3/2007 13:08'!
antiChangeClass
	^ DSClassCreatedChange! !

!DSClassRemovedChange methodsFor: 'applying' stamp: 'gk 8/23/2007 21:04'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applyClassRemoved: self! !

!DSClassRemovedChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:48'!
isClassRemoved
	^ true! !

!DSClassRemovedChange methodsFor: 'testing' stamp: 'gk 9/7/2007 10:41'!
isShadowing: anotherChange
	"A remove removes all previous changes of any kind."

	^self isSameClass: anotherChange! !

DSClassChange subclass: #DSClassNameChange
	instanceVariableNames: 'newName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassNameChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 23:33'!
class: aClass to: newName
	^(self class: aClass) newName: newName; yourself! !

!DSClassNameChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 23:33'!
className: aName to: newName
	^(self className: aName) newName: newName; yourself! !

!DSClassNameChange methodsFor: 'applying' stamp: 'gk 9/3/2007 22:19'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applyClassNameChanged: self! !

!DSClassNameChange methodsFor: 'anti' stamp: 'gk 9/3/2007 22:17'!
asAntiChange
	"Reverse it."

	^self antiChangeClass className: newName to: className! !

!DSClassNameChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:48'!
isClassName
	^ true! !

!DSClassNameChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 21:57'!
isName
	^ true! !

!DSClassNameChange methodsFor: 'testing' stamp: 'gk 9/7/2007 10:41'!
isShadowing: anotherChange
	"Remove all previous name changes."

	^(self isSameClass: anotherChange) and: [anotherChange isClassName]! !

!DSClassNameChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 23:20'!
newName
	^newName! !

!DSClassNameChange methodsFor: 'accessing' stamp: 'gk 8/15/2007 00:41'!
newName: aString
	newName := aString! !

!DSClassNameChange methodsFor: 'accessing' stamp: 'gk 8/28/2007 23:04'!
oldName
	^className! !

!DSClassNameChange methodsFor: 'accessing' stamp: 'mtf 9/4/2007 10:02'!
oldName: aSymbol
	self className: aSymbol! !

DSClassChange subclass: #DSClassSuperclassChange
	instanceVariableNames: 'oldSuperclassName newSuperclassName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassSuperclassChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 23:33'!
class: aClass from: oldSuperclass to: newSuperclass
	^(self class: aClass) oldSuperclass: oldSuperclass; newSuperclass: newSuperclass; yourself! !

!DSClassSuperclassChange class methodsFor: 'as yet unclassified' stamp: 'gk 9/3/2007 22:58'!
className: aName fromName: oldSuperclassName toName: newSuperclassName
	^(self className: aName) oldSuperclassName: oldSuperclassName; newSuperclassName: newSuperclassName; yourself! !

!DSClassSuperclassChange methodsFor: 'applying' stamp: 'gk 9/3/2007 23:09'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applySuperclassChanged: self! !

!DSClassSuperclassChange methodsFor: 'anti' stamp: 'gk 9/3/2007 22:59'!
asAntiChange
	"Reverse it."

	^self antiChangeClass className: className fromName: newSuperclassName toName: oldSuperclassName! !

!DSClassSuperclassChange methodsFor: 'testing' stamp: 'gk 8/20/2007 22:47'!
isClassSuperclass
	^true! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 22:53'!
newSuperclass
	^Smalltalk classNamed: newSuperclassName! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 23:52'!
newSuperclass: aClass
	^newSuperclassName := aClass name! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 22:53'!
newSuperclassName
	^newSuperclassName! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 9/3/2007 22:54'!
newSuperclassName: aName
	newSuperclassName := aName! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 22:53'!
oldSuperclass
	^Smalltalk classNamed: oldSuperclassName! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 23:52'!
oldSuperclass: aClass
	^oldSuperclassName := aClass name! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 8/20/2007 22:53'!
oldSuperclassName
	^oldSuperclassName! !

!DSClassSuperclassChange methodsFor: 'accessing' stamp: 'gk 9/3/2007 22:54'!
oldSuperclassName: aName
	oldSuperclassName := aName! !

!DSClassSuperclassChange methodsFor: 'private' stamp: 'gk 9/7/2007 16:21'!
remapClassNames: map

	super remapClassNames: map.
	oldSuperclassName := map at: oldSuperclassName ifAbsent: [oldSuperclassName].
	newSuperclassName := map at: newSuperclassName ifAbsent: [newSuperclassName].! !

DSClassChange subclass: #DSClassTypeChange
	instanceVariableNames: 'oldType newType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassTypeChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 22:44'!
className: aName from: old to: new
	^(self className: aName) oldType: old; newType: new; yourself! !

!DSClassTypeChange methodsFor: 'as yet unclassified' stamp: 'mtf 9/21/2007 13:34'!
applyUsing: anApplier
	^ anApplier applyClassTypeChanged: self! !

!DSClassTypeChange methodsFor: 'testing' stamp: 'gk 8/21/2007 02:27'!
isClassType
	^true! !

!DSClassTypeChange methodsFor: 'accessing' stamp: 'gk 8/21/2007 02:28'!
newType
	^newType! !

!DSClassTypeChange methodsFor: 'accessing' stamp: 'gk 8/21/2007 02:28'!
newType: type
	newType := type! !

!DSClassTypeChange methodsFor: 'accessing' stamp: 'gk 8/21/2007 02:28'!
oldType
	^oldType! !

!DSClassTypeChange methodsFor: 'accessing' stamp: 'gk 8/21/2007 02:28'!
oldType: type
	oldType := type! !

DSClassChange subclass: #DSMethodChange
	instanceVariableNames: 'selector meta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodChange commentStamp: 'gk 8/28/2007 22:46' prior: 0!
An abstract super class for all actions affecting a specific method:

selector: (Symbol) the method selector
className: (Symbol) the name of the defining class
metaclass: (Boolean) true if defined on the metaclass, false if defined on the class!

!DSMethodChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/31/2007 13:32'!
class: aClass selector: selector
	^ self new class: aClass selector: selector; yourself! !

!DSMethodChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/27/2007 22:56'!
fromChange: aChange
	^ self new
		className: aChange className;
		selector: aChange selector;
		meta: aChange meta! !

!DSMethodChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/31/2007 13:40'!
fromEvent: anEvent
	^ self new
		setClass: anEvent itemClass;
		selector: anEvent itemSelector;
		yourself! !

!DSMethodChange methodsFor: 'comparing' stamp: 'mtf 9/13/2007 14:40'!
= aChange
"This is just for comparing of group keys. See the comment in DSClassChange>>=.

Group keys for method changes depend on the class/metaclass separation and the selector, in addition to what is required for class changes to group together"
	^ super = aChange and: [meta == aChange meta] and: [selector == aChange selector]! !

!DSMethodChange methodsFor: 'initialize-release' stamp: 'mtf 8/31/2007 13:32'!
class: aClass selector: aSelector
	self setClass: aClass.
	self selector: aSelector! !

!DSMethodChange methodsFor: 'comparing' stamp: 'mtf 9/13/2007 14:42'!
hash
"This is just for comparing of group keys. See the comment in #=."
	^ (super hash bitXor: meta hash) bitXor: selector hash! !

!DSMethodChange methodsFor: 'initialize-release' stamp: 'mtf 8/31/2007 13:30'!
initializeFromClass: aClass selector: aSelector
	self setClass: aClass.
	self selector: aSelector! !

!DSMethodChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 13:25'!
isMeta
	^ meta! !

!DSMethodChange methodsFor: 'testing' stamp: 'mtf 8/23/2007 15:40'!
isMethodChange
	^ true! !

!DSMethodChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:25'!
meta
	^ meta! !

!DSMethodChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:25'!
meta: aBoolean
	meta := aBoolean! !

!DSMethodChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:44'!
methodGroupKey
"For grouping. See the comment in DSChange>>inMethodGroupFor:"
	^ DSMethodChange fromChange: self! !

!DSMethodChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:03'!
name
	^ selector! !

!DSMethodChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:15'!
name: aSymbol
	selector := aSymbol! !

!DSMethodChange methodsFor: 'as yet unclassified' stamp: 'mtf 9/26/2007 16:20'!
printOn: aStream
	aStream nextPutAll: self className.
	self isMeta ifTrue: [aStream nextPutAll: ' class'].
	aStream
		nextPutAll: '>>';
		nextPutAll: self selector;
		space; nextPutAll: self action! !

!DSMethodChange methodsFor: 'accessing' stamp: 'mtf 8/23/2007 16:53'!
selector
	^ selector! !

!DSMethodChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:15'!
selector: aSymbol
	selector := aSymbol! !

!DSMethodChange methodsFor: 'convenience' stamp: 'mtf 8/28/2007 12:50'!
setClass: aClass
	self className: aClass theNonMetaClass name.
	self meta: aClass isMeta! !

!DSMethodChange methodsFor: 'convenience' stamp: 'mtf 8/28/2007 12:56'!
sourceFromMethod: aCompiledMethod
	^ aCompiledMethod getSourceFor: self selector in: self getClass! !

DSMethodChange subclass: #DSMethodFullChange
	instanceVariableNames: 'source protocol stamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

DSMethodFullChange subclass: #DSMethodAddedChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodAddedChange methodsFor: 'anti' stamp: 'mtf 8/27/2007 22:50'!
antiChangeClass
	^ DSMethodRemovedChange! !

!DSMethodAddedChange methodsFor: 'applying' stamp: 'mtf 8/26/2007 19:08'!
applyUsing: anApplier
	^ anApplier applyMethodAdded: self! !

!DSMethodAddedChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 10:38'!
isMethodAdded
	^true! !

!DSMethodFullChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/28/2007 13:10'!
class: aClass selector: selector method: aCompiledMethod protocol: protocol
	^ (self class: aClass selector: selector)
		method: aCompiledMethod;
		protocol: protocol;
		yourself! !

!DSMethodFullChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/28/2007 13:14'!
fromEvent: anEvent
	^ (super fromEvent: anEvent)
		method: anEvent itemMethod;
		protocol: anEvent itemProtocol;
		yourself! !

!DSMethodFullChange methodsFor: 'anti' stamp: 'mtf 8/27/2007 22:57'!
asAntiChange
	^ (self antiChangeClass fromChange: self)
		protocol: protocol;
		source: source;
		stamp: stamp! !

!DSMethodFullChange methodsFor: 'initialize-release' stamp: 'mtf 8/31/2007 13:36'!
class: aClass selector: aSelector
	super class: aClass selector: aSelector.
	self method: (aClass methodDictionary at: aSelector).
	self protocol: (aClass organization categoryOfElement: aSelector)
	! !

!DSMethodFullChange methodsFor: 'grouping' stamp: 'mtf 9/13/2007 14:47'!
mergeGroupKey
"See the comment in DSDelta>>inMergeGroupFor:. My instances are not symetric enough to be mergable, so return nil"
	^ nil! !

!DSMethodFullChange methodsFor: 'initialize-release' stamp: 'mtf 8/28/2007 12:56'!
method: aCompiledMethod
	self source: (self sourceFromMethod: aCompiledMethod).
	self stamp: aCompiledMethod timeStamp.! !

!DSMethodFullChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:27'!
protocol
	^ protocol! !

!DSMethodFullChange methodsFor: 'accessing' stamp: 'mtf 9/11/2007 21:32'!
protocol: aProtocol
	protocol := aProtocol
		ifNil: [Categorizer default]
		ifNotNil: [aProtocol asSymbol]! !

!DSMethodFullChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:16'!
source
	^ source! !

!DSMethodFullChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:18'!
source: aString
	source := aString! !

!DSMethodFullChange methodsFor: 'accessing' stamp: 'mtf 8/26/2007 08:54'!
stamp
	^ stamp! !

!DSMethodFullChange methodsFor: 'accessing' stamp: 'mtf 8/26/2007 08:54'!
stamp: anObject
	stamp := anObject! !

DSMethodFullChange subclass: #DSMethodRecord
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodRecord commentStamp: 'mtf 9/12/2007 18:16' prior: 0!
See the comment in DSClassRecord. My instances serve the same purpose, but for methods!

!DSMethodRecord methodsFor: 'applying' stamp: 'mtf 9/12/2007 11:36'!
applyUsing: anApplier
	anApplier applyMethodSnapshot: self! !

!DSMethodRecord methodsFor: 'anti' stamp: 'mtf 9/12/2007 14:52'!
asAntiChange
	^ self! !

!DSMethodRecord methodsFor: 'testing' stamp: 'mtf 9/12/2007 11:34'!
isChange
	^ false! !

!DSMethodRecord methodsFor: 'testing' stamp: 'mtf 9/12/2007 18:17'!
isRecord
	^ true! !

!DSMethodRecord methodsFor: 'testing' stamp: 'mtf 9/12/2007 11:34'!
isRevertable
	^ false! !

DSMethodFullChange subclass: #DSMethodRemovedChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodRemovedChange methodsFor: 'anti' stamp: 'mtf 8/27/2007 22:54'!
antiChangeClass
	^ DSMethodAddedChange! !

!DSMethodRemovedChange methodsFor: 'applying' stamp: 'mtf 8/26/2007 19:09'!
applyUsing: anApplier
	^ anApplier applyMethodRemoved: self! !

!DSMethodRemovedChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 10:37'!
isMethodRemoved
	^true! !

DSMethodChange subclass: #DSMethodNameChange
	instanceVariableNames: 'newSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodNameChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/28/2007 13:08'!
class: aClass selector: selector from: oldSelector to: newSelector
	^ (self class: aClass selector: selector)
		newSelector: newSelector;
		yourself! !

!DSMethodNameChange methodsFor: 'applying' stamp: 'mtf 8/26/2007 19:08'!
applyUsing: anApplier
	^ anApplier applyMethodName: self! !

!DSMethodNameChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 14:37'!
isMethodName
	^true! !

!DSMethodNameChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 21:58'!
isName
	^true! !

!DSMethodNameChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:04'!
newName
	^ newSelector! !

!DSMethodNameChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:04'!
newName: anObject
	newSelector := anObject! !

!DSMethodNameChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:05'!
newSelector
	^ newSelector! !

!DSMethodNameChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:05'!
newSelector: anObject
	newSelector := anObject! !

!DSMethodNameChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:05'!
oldName
	^ selector! !

!DSMethodNameChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 10:05'!
oldSelector
	^ selector! !

DSMethodChange subclass: #DSMethodProtocolChange
	instanceVariableNames: 'oldProtocol newProtocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodProtocolChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/28/2007 13:07'!
class: aClass selector: selector from: oldProtocol to: newProtocol
	^ (self class: aClass selector: selector)
		oldProtocol: oldProtocol;
		newProtocol: newProtocol;
		yourself! !

!DSMethodProtocolChange methodsFor: 'applying' stamp: 'mtf 9/4/2007 09:41'!
applyUsing: anApplier
	^ anApplier applyMethodProtocolChanged: self! !

!DSMethodProtocolChange methodsFor: 'anti' stamp: 'mtf 8/27/2007 22:59'!
asAntiChange
	^ (self class fromChange: self)
		oldProtocol: newProtocol;
		newProtocol: oldProtocol! !

!DSMethodProtocolChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 13:51'!
isMethodProtocol
	^true! !

!DSMethodProtocolChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:51'!
newProtocol
	^ newProtocol! !

!DSMethodProtocolChange methodsFor: 'accessing' stamp: 'mtf 9/11/2007 21:34'!
newProtocol: aProtocol
	newProtocol := aProtocol
		ifNil: [Categorizer default]
		ifNotNil: [aProtocol asSymbol]! !

!DSMethodProtocolChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:51'!
oldProtocol
	^ oldProtocol! !

!DSMethodProtocolChange methodsFor: 'accessing' stamp: 'mtf 9/11/2007 21:34'!
oldProtocol: aProtocol
	oldProtocol := aProtocol
		ifNil: [Categorizer default]
		ifNotNil: [aProtocol asSymbol]! !

DSMethodChange subclass: #DSMethodSourceChange
	instanceVariableNames: 'oldSource newSource oldStamp newStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Method'!

!DSMethodSourceChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/28/2007 13:03'!
class: aClass selector: selector from: oldMethod to: newMethod
	^ (self class: aClass selector: selector)
		oldMethod: oldMethod;
		newMethod: newMethod;
		yourself! !

!DSMethodSourceChange methodsFor: 'applying' stamp: 'mtf 9/4/2007 09:41'!
applyUsing: anApplier
	^ anApplier applyMethodSourceChanged: self! !

!DSMethodSourceChange methodsFor: 'anti' stamp: 'mtf 8/27/2007 23:00'!
asAntiChange
	^ (self class fromChange: self)
		oldSource: newSource;
		newSource: oldSource;
		oldStamp: newStamp;
		newStamp: oldStamp! !

!DSMethodSourceChange methodsFor: 'testing' stamp: 'mtf 8/24/2007 13:46'!
isMethodSource
	^true! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'mtf 8/28/2007 12:59'!
newMethod: aCompiledMethod
	self newSource: (self sourceFromMethod: aCompiledMethod).
	self newStamp: aCompiledMethod timeStamp.! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:46'!
newSource
	^ newSource! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:46'!
newSource: anObject
	newSource := anObject! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'test 8/26/2007 00:22'!
newStamp
	^ newStamp! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'test 8/26/2007 00:22'!
newStamp: anObject
	newStamp := anObject! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'mtf 8/28/2007 12:58'!
oldMethod: aCompiledMethod
	self oldSource: (self sourceFromMethod: aCompiledMethod).
	self oldStamp: aCompiledMethod timeStamp.! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:45'!
oldSource
	^ oldSource! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'mtf 8/24/2007 13:45'!
oldSource: anObject
	oldSource := anObject! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'test 8/26/2007 00:22'!
oldStamp
	^ oldStamp! !

!DSMethodSourceChange methodsFor: 'accessing' stamp: 'test 8/26/2007 00:22'!
oldStamp: anObject
	oldStamp := anObject! !

DSClassChange subclass: #DSVarsChange
	instanceVariableNames: 'oldVars newVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSVarsChange commentStamp: '<historical>' prior: 0!
A change in variables (instance variables, class variables or shared pools) is reprented as two Arrays of Strings - before and after.
There are three different aspects to a change:
	- What variables were added?
	- What variables were removed?
	- What variables changed their slot number and is this relevant? (See subclass comments)

First it was tempting to represent these different changes with different change instances - but the developer often makes these changes all in one go. If we represented a single developer action as three different changes, it would be misleading.!

DSVarsChange subclass: #DSClassInstVarsChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassInstVarsChange commentStamp: 'gk 9/4/2007 22:50' prior: 0!
Class instance variables, just as instance variables do indeed have an order (I think) though probably totally irrelevant in 99.9% of the cases. If anyone can correct me on this - please do. :)!

!DSClassInstVarsChange methodsFor: 'applying' stamp: 'mtf 8/30/2007 16:58'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applyClassInstVarsChanged: self! !

!DSClassInstVarsChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:10'!
isClassInstVars
	^ true! !

!DSClassInstVarsChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 18:50'!
isMeta
	^ true! !

DSVarsChange subclass: #DSClassVarsChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSClassVarsChange commentStamp: '<historical>' prior: 0!
Class variables are held in a Dictionary so ordering has no effect but customary is to keep them sorted, see ClassDescription>>classVariablesString. This means we only care about added or removed variables.!

!DSClassVarsChange methodsFor: 'applying' stamp: 'gk 9/3/2007 23:34'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applyClassVarsChanged: self! !

!DSClassVarsChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:09'!
isClassVars
	^ true! !

!DSClassVarsChange methodsFor: 'accessing' stamp: 'gk 8/22/2007 00:28'!
newVars: anArray
	"We make sure it is sorted to maintain a specific ordering."

	newVars := anArray sort! !

!DSClassVarsChange methodsFor: 'accessing' stamp: 'gk 8/22/2007 00:28'!
oldVars: anArray
	"We make sure it is sorted to maintain a specific ordering."

	oldVars := anArray sort! !

DSVarsChange subclass: #DSInstVarsChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSInstVarsChange commentStamp: '<historical>' prior: 0!
Instance variables are held in an Array so ordering is important.
This means that additional to tracking added or removed variables we also keep track of moved variables (with a new index position in the array).!

!DSInstVarsChange methodsFor: 'applying' stamp: 'gk 8/24/2007 00:11'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applyInstVarsChanged: self! !

!DSInstVarsChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:08'!
isInstVars
	^ true! !

DSVarsChange subclass: #DSSharedPoolVarsChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Class'!

!DSSharedPoolVarsChange commentStamp: 'gk 9/5/2007 10:51' prior: 0!
Shared pools are held in an OrderedCollection so ordering is important.
This means that additional to tracking added or removed variables we also keep track of moved variables (with a new index position in the array).

NOTE: Currently we don't support "private" shared pools - or in other words, shared pools (Dictionaries) that aren't accessible through the SystemDictionary.!

!DSSharedPoolVarsChange methodsFor: 'applying' stamp: 'gk 9/3/2007 23:23'!
applyUsing: anApplier
	"Apply this change to the image. Double dispatch."

	^anApplier applySharedPoolVarsChanged: self! !

!DSSharedPoolVarsChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:07'!
isSharedPools
	^ true! !

!DSVarsChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/28/2007 23:40'!
class: aClass from: oldArray to: newArray

	^(self class: aClass) oldVars: oldArray; newVars: newArray; yourself! !

!DSVarsChange class methodsFor: 'as yet unclassified' stamp: 'gk 8/24/2007 08:01'!
className: aName from: oldArray to: newArray

	^(self className: aName) oldVars: oldArray; newVars: newArray; yourself! !

!DSVarsChange methodsFor: 'differences' stamp: 'gk 9/3/2007 13:58'!
added
	"Return any new variables added, in order."

	^newVars difference: oldVars! !

!DSVarsChange methodsFor: 'anti' stamp: 'gk 8/28/2007 23:03'!
asAntiChange
	"Reverse it."

	^self class className: className from: newVars to: oldVars! !

!DSVarsChange methodsFor: 'testing' stamp: 'gk 8/22/2007 00:39'!
hasAdded
	^self added notEmpty! !

!DSVarsChange methodsFor: 'testing' stamp: 'gk 8/22/2007 00:39'!
hasMoved
	^self moved notEmpty! !

!DSVarsChange methodsFor: 'testing' stamp: 'gk 8/22/2007 00:39'!
hasRemoved
	^self removed notEmpty! !

!DSVarsChange methodsFor: 'differences' stamp: 'gk 9/3/2007 13:58'!
kept
	"Return all old variables we still have, in order."

	^(oldVars intersection: newVars)! !

!DSVarsChange methodsFor: 'differences' stamp: 'gk 8/22/2007 00:59'!
moved
	"Return all old variables with a new index."

	^self kept select: [:var |
		(oldVars indexOf: var) ~~ (newVars indexOf: var)]! !

!DSVarsChange methodsFor: 'differences' stamp: 'gk 9/3/2007 14:43'!
movedWithIndices
	"Return an OrderedCollection of Associations for all moved variables with their new index."

	| result |
	result := OrderedCollection new.
	self moved do: [:ivar |
		result add: ivar -> (newVars indexOf: ivar)].
	^result! !

!DSVarsChange methodsFor: 'accessing' stamp: 'gk 8/22/2007 00:29'!
newVars
	^newVars! !

!DSVarsChange methodsFor: 'accessing' stamp: 'gk 8/21/2007 23:48'!
newVars: anArray
	newVars := anArray! !

!DSVarsChange methodsFor: 'accessing' stamp: 'gk 8/22/2007 00:29'!
oldVars
	^oldVars! !

!DSVarsChange methodsFor: 'accessing' stamp: 'gk 8/21/2007 23:48'!
oldVars: anArray
	oldVars := anArray! !

!DSVarsChange methodsFor: 'differences' stamp: 'gk 9/3/2007 13:58'!
removed
	"Return all old variables removed, in order."

	^oldVars difference: newVars! !

DSChange subclass: #DSCompositeChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Base'!

!DSCompositeChange commentStamp: 'mtf 9/6/2007 17:43' prior: 0!
A composite change is an abstract base class for changes that logically belong together. It is not necessarily ordered. My primary subclass is DSChangeSequence, which stores a sequence of changes!

DSCompositeChange subclass: #DSChangeCategorizer
	instanceVariableNames: 'dict defaultBlock classifyBlock categorizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

!DSChangeCategorizer commentStamp: 'mtf 9/6/2007 23:57' prior: 0!
My instances automatically group the changes added to them. I implement the Categorizer protocol, so I can be used directly by Browser and friends.!

!DSChangeCategorizer class methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 19:29'!
default: aDefaultBlock classify: aClassifyBlock
	^ self new default: aDefaultBlock classify: aClassifyBlock! !

!DSChangeCategorizer class methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 23:44'!
forClasses
	^ self
		default: [:aSymbol | DSCompositeClassChange new]
		classify: [:aChange | aChange className]! !

!DSChangeCategorizer class methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 23:44'!
forClassesAndMethods
	^ self
		default: [:aSymbol | DSGroupedClassChange new]
		classify: [:aChange | aChange className]! !

!DSChangeCategorizer class methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 23:41'!
forMethods
	^ self
		default: [:aSymbol | DSCompositeMethodChange  new]
		classify: [:aChange | aChange selector]! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/6/2007 22:07'!
addChange: aChange
	self addChange: aChange classified: (classifyBlock value: aChange).
	aChange isName ifTrue: [
		self dictionary at: aChange newName put: (self dictionary at: aChange oldName).
		self dictionary removeKey: aChange oldName].
	^ aChange
	! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/7/2007 10:27'!
addChange: aChange classified: aSymbol
	(self at: aSymbol) addChange: aChange.
	categorizer ifNotNil: [categorizer classify: aSymbol under: (self at: aSymbol) category].
	^ aChange! !

!DSChangeCategorizer methodsFor: 'applying' stamp: 'mtf 9/21/2007 12:41'!
applyUsing: anApplier
	anApplier applyCategorizedCompositeChange: self! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/6/2007 19:40'!
at: aSymbol
	^ self dictionary at: aSymbol ifAbsentPut: [defaultBlock value: aSymbol]! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/7/2007 11:08'!
buildCategorizer
	| aCategorizer |
	aCategorizer := Categorizer defaultList: Array new.
	self dictionary keysAndValuesDo: [:name :group | aCategorizer classify: name under: group category].
	^ aCategorizer! !

!DSChangeCategorizer methodsFor: 'initialize-release' stamp: 'mtf 9/6/2007 19:28'!
default: aDefaultBlock classify: aClassifyBlock
	defaultBlock := aDefaultBlock.
	classifyBlock := aClassifyBlock! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/6/2007 19:14'!
dictionary
	^ dict ifNil: [dict := Dictionary new]! !

!DSChangeCategorizer methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 20:06'!
do: aBlock
	self dictionary do: [:ea | ea do: aBlock]! !

!DSChangeCategorizer methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 22:47'!
groupsDo: aBlock
	self dictionary do: [:ea | ea groupsDo: aBlock]! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/6/2007 19:14'!
keys
	^ self dictionary keys! !

!DSChangeCategorizer methodsFor: 'accessing' stamp: 'mtf 9/7/2007 10:48'!
organization
	^ categorizer ifNil: [categorizer := self buildCategorizer]! !

!DSChangeCategorizer methodsFor: 'enumerating' stamp: 'mtf 9/13/2007 12:00'!
reverseDo: aBlock
	self dictionary do: [:ea | ea reverseDo: aBlock]! !

DSCompositeChange subclass: #DSChangeSequence
	instanceVariableNames: 'changes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

!DSChangeSequence commentStamp: 'mtf 9/6/2007 21:22' prior: 0!
I am a base class for a sequence of changes that logically belong together. The prime example is the antiChange for DSClassRemovedChange - since the anti change should revert the effects it needs in this case both recreate the class, set its comment and add methods on both instance and class side. Thus the class DSClassAddedChange is a composite change that does all these changes in sequence.!

!DSChangeSequence methodsFor: 'accessing' stamp: 'mtf 9/6/2007 17:40'!
addChange: aChange

	^ self changes add: aChange! !

!DSChangeSequence methodsFor: 'applying' stamp: 'mtf 9/21/2007 12:42'!
applyUsing: anApplier
	anApplier applyChangeSequence: self! !

!DSChangeSequence methodsFor: 'anti' stamp: 'mtf 9/6/2007 17:41'!
asAntiChange
	"Return a new Change that reverts this one."

	^ self antiChangeClass changes: 
		(self changes reverseCollect: [:change| change asAntiChange])! !

!DSChangeSequence methodsFor: 'accessing' stamp: 'mtf 9/6/2007 17:41'!
changes
	^ changes ifNil: [changes := OrderedCollection new]! !

!DSChangeSequence methodsFor: 'accessing' stamp: 'mtf 9/6/2007 17:41'!
changes: anOrderedCollection
	changes := anOrderedCollection! !

!DSChangeSequence methodsFor: 'enumerating' stamp: 'mtf 9/20/2007 19:25'!
do: aBlock
	^ self changes do: aBlock! !

!DSChangeSequence methodsFor: 'enumerating' stamp: 'mtf 9/20/2007 19:25'!
reverseDo: aBlock
	^ self changes reverseDo: aBlock! !

DSChangeSequence subclass: #DSCompositeClassChange
	instanceVariableNames: 'category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

DSCompositeClassChange subclass: #DSCompositeClassAddedChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

!DSCompositeClassAddedChange methodsFor: 'anti' stamp: 'gk 9/3/2007 13:22'!
antiChangeClass
	^DSCompositeClassDeletedChange! !

!DSCompositeClassAddedChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 12:44'!
applyUsing: anApplier
	^ anApplier applyCompositeClassAddedChange: self! !

!DSCompositeClassAddedChange methodsFor: 'initialize-release' stamp: 'gk 9/7/2007 00:16'!
initializeFrom: aClass
	"We add changes corresponding to adding every bit of the class individually.
	This change is normally used when adding a class to a Delta manually."

	self addChange: (DSClassCreatedChange class: aClass).

	aClass hasComment ifTrue: [self
		addChange: (DSClassCommentChange class: aClass
			from: nil
			to: aClass organization classComment
			oldStamp: nil
			newStamp: aClass organization commentStamp)].

	"This one below looks odd, but recall that we need to be able to do revert and this one is needed then"
	aClass theMetaClass instVarNames isEmpty ifFalse: [self 
		addChange: (DSClassInstVarsChange class: aClass
			from: #()
			to: aClass theMetaClass instVarNames)].

	aClass theNonMetaClass selectors do: [:sel | self
		addChange: (DSMethodAddedChange class: aClass theNonMetaClass selector: sel)].
	aClass theMetaClass selectors do: [:sel | self
		addChange: (DSMethodAddedChange class: aClass theMetaClass selector: sel)].


! !

!DSCompositeClassAddedChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:23'!
isClassAdded
	^ true! !

!DSCompositeClassChange class methodsFor: 'as yet unclassified' stamp: 'mtf 9/21/2007 12:57'!
class: aClass
	^ self new initializeFrom: aClass; yourself! !

!DSCompositeClassChange methodsFor: 'accessing' stamp: 'mtf 9/13/2007 08:50'!
addChange: aChange
	aChange isClassChange ifFalse: [^ nil].
	aChange isClassAdded | aChange isClassRemoved ifTrue: [category := aChange category].
	aChange isClassCategory ifTrue: [category := aChange newCategory].
	^ super addChange: aChange! !

!DSCompositeClassChange methodsFor: 'as yet unclassified' stamp: 'mtf 9/21/2007 13:00'!
applyUsing: anApplier
	anApplier applyCompositeClassChange: self! !

!DSCompositeClassChange methodsFor: 'accessing' stamp: 'mtf 9/7/2007 10:08'!
category
	category ifNotNil: [^ category].
	Smalltalk at: self className ifPresent: [:aClass | ^ aClass category].
	Smalltalk at: self changes first className ifPresent: [:aClass | ^ aClass category].
	^ Categorizer default! !

!DSCompositeClassChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:26'!
className
	^ self changes last className! !

!DSCompositeClassChange methodsFor: 'initialize-release' stamp: 'mtf 9/21/2007 12:58'!
initializeFrom: aClass
	^ self subclassResponsibility! !

!DSCompositeClassChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 23:29'!
isClassChange
	^ true! !

DSCompositeClassChange subclass: #DSCompositeClassDeletedChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

!DSCompositeClassDeletedChange methodsFor: 'anti' stamp: 'gk 9/3/2007 13:22'!
antiChangeClass
	^DSCompositeClassAddedChange! !

!DSCompositeClassDeletedChange methodsFor: 'applying' stamp: 'gk 9/3/2007 13:23'!
applyUsing: anApplier
	"We override superclass behavior and simply remove the whole class in one call."

	^anApplier applyClassRemoved: self! !

!DSCompositeClassDeletedChange methodsFor: 'initialize-release' stamp: 'gk 9/3/2007 13:30'!
initializeFrom: aClass
	"We add changes corresponding to removing every bit individually.
	This is mainly in order to be able to do a full revert later."

	aClass theNonMetaClass selectors do: [:sel | self
		addChange: (DSMethodRemovedChange class: aClass theNonMetaClass selector: sel)].
	aClass theMetaClass selectors do: [:sel | self
		addChange: (DSMethodRemovedChange class: aClass theMetaClass selector: sel)].

	"This one below looks odd, but recall that we need to be able to do revert and this one is needed then"
	aClass theMetaClass instVarNames isEmpty ifFalse: [self 
		addChange: (DSClassInstVarsChange class: aClass
			from: aClass theMetaClass instVarNames
			to: #())].

	aClass hasComment ifTrue: [self
		addChange: (DSClassCommentChange class: aClass
			from: aClass organization classComment
			to: nil
			oldStamp: aClass organization commentStamp
			newStamp: nil)].
	self addChange: (DSClassRemovedChange class: aClass)! !

!DSCompositeClassDeletedChange methodsFor: 'testing' stamp: 'gk 9/7/2007 00:24'!
isClassDeleted
	^ true! !

!DSCompositeClassDeletedChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:23'!
isClassRemoved
	^ true! !

DSCompositeClassChange subclass: #DSGroupedClassChange
	instanceVariableNames: 'methodChanges classMethodChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

!DSGroupedClassChange commentStamp: 'mtf 9/21/2007 12:55' prior: 0!
My instances represent changes made to a specific class, grouped by method for easy lookup. This is especially useful when browsing a delta

I may be superceded by DSChangeCategorizer, but am still referenced. I may be removed in the future!

!DSGroupedClassChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:46'!
addChange: aChange
	aChange isMethodChange ifTrue: [^ self addMethodChange: aChange].
	^ super addChange: aChange! !

!DSGroupedClassChange methodsFor: 'private' stamp: 'test 9/6/2007 22:28'!
addMethodChange: aChange
	^ (aChange isMeta
		ifFalse: [self methodChanges]
		ifTrue: [self classMethodChanges]
	) addChange: aChange! !

!DSGroupedClassChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:58'!
classMethodChanges
	^ classMethodChanges 
		ifNil: [classMethodChanges := DSChangeCategorizer forMethods]! !

!DSGroupedClassChange methodsFor: 'enumerating' stamp: 'mtf 9/7/2007 00:11'!
do: aBlock
	super do: aBlock.
	self methodChanges do: aBlock.
	self classMethodChanges do: aBlock! !

!DSGroupedClassChange methodsFor: 'enumerating' stamp: 'mtf 9/7/2007 00:11'!
groupsDo: aBlock
	super groupsDo: aBlock.
	self methodChanges groupsDo: aBlock.
	self classMethodChanges groupsDo: aBlock! !

!DSGroupedClassChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:58'!
methodChanges
	^ methodChanges ifNil: [methodChanges := DSChangeCategorizer forMethods]! !

!DSGroupedClassChange methodsFor: 'enumerating' stamp: 'mtf 9/13/2007 11:58'!
reverseDo: aBlock
	self classMethodChanges reverseDo: aBlock.
	self methodChanges reverseDo: aBlock.
	super reverseDo: aBlock.
! !

DSChangeSequence subclass: #DSCompositeMethodChange
	instanceVariableNames: 'protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Composite'!

!DSCompositeMethodChange methodsFor: 'accessing' stamp: 'mtf 9/13/2007 08:50'!
addChange: aChange
	aChange isMethodChange ifFalse: [^ nil].
	aChange isMethodAdded | aChange isMethodRemoved ifTrue: [protocol := aChange protocol].
	aChange isMethodProtocol ifTrue: [protocol := aChange newProtocol].
	^ super addChange: aChange! !

!DSCompositeMethodChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 12:54'!
applyUsing: anApplier
	^ anApplier applyCompositeMethodChange: self! !

!DSCompositeMethodChange methodsFor: 'accessing' stamp: 'mtf 9/7/2007 10:05'!
category
	"for compatability with DSChangeClassifier, letting it implement the Categorization protocol"
	^ self protocol! !

!DSCompositeMethodChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:39'!
className
	^ self changes last className! !

!DSCompositeMethodChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 23:40'!
isClassChange
	^ true! !

!DSCompositeMethodChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:39'!
isMeta
	^ self changes last isMeta! !

!DSCompositeMethodChange methodsFor: 'testing' stamp: 'mtf 9/6/2007 23:40'!
isMethodChange
	^ true! !

!DSCompositeMethodChange methodsFor: 'accessing' stamp: 'mtf 9/7/2007 10:07'!
protocol
	protocol ifNotNil: [^ protocol].
	Smalltalk at: self className ifPresent: [:aClass |
		^ aClass organization categoryOfElement: self selector].
	^ Categorizer default! !

!DSCompositeMethodChange methodsFor: 'accessing' stamp: 'mtf 9/6/2007 23:39'!
selector
	^ self changes last selector! !

DSChangeSequence subclass: #DSDelta
	instanceVariableNames: 'uuid test notRevertable attributes attachments changeCounter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Base'!

!DSDelta commentStamp: '<historical>' prior: 0!
A Delta is much like a ChangeSet but improved and simplified. It captures changes much like a ChangeSet does (see #event:), but unlike ChangeSet multiple Deltas can be logging changes at the same time. A DeltaSet also maintains a chronologically ordered collection of change objects that is a true log unlike a ChangeSet which tries to normalize on the fly and does not maintain an order.

Each change object carries more information than a record in a ChangeSet does - and they know how to produce their anti change which in turn gives a DSDelta the ability to produce its own anti Delta. Such an anti Delta can be used to revert a Delta.!

!DSDelta class methodsFor: 'as yet unclassified' stamp: 'gk 8/23/2007 20:55'!
changeApplier

	^DSSystemEditorApplier new! !

!DSDelta class methodsFor: 'utilities' stamp: 'gk 8/23/2007 22:57'!
collectionToString: collection
	^ String streamContents: [:stream | 
		collection
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!DSDelta class methodsFor: 'as yet unclassified' stamp: 'gk 8/23/2007 22:06'!
name: aString

	^self new name: aString; yourself! !

!DSDelta class methodsFor: 'utilities' stamp: 'mtf 9/21/2007 23:23'!
uuid: aUUID
	^ self basicNew uuid: aUUID; yourself! !

!DSDelta methodsFor: 'private' stamp: 'mtf 10/3/2007 11:55'!
addChange: aChange
	"Within a Delta we give a change a monotonically increasing number
	in order to be able to sort changes chronologicall. The TimeStamp does
	not work if changes are created very fast, they end up the same and sorting
	becomes undefined. the change is copied before it's number is changed since
	the change may be in multiple deltas under different numbers"

	| newChange |
	newChange := aChange copy number: self incrementChangeCounter; yourself.
	changes add: newChange.
	self grouper ifNotNil: [self grouper addChange: newChange].
	self changed: #changeList.
	^ newChange! !

!DSDelta methodsFor: 'applying' stamp: 'mtf 9/21/2007 13:38'!
apply
	"Apply this delta to the image. We let an applier do it."

	^ self class changeApplier apply: self! !

!DSDelta methodsFor: 'applying' stamp: 'mtf 9/21/2007 12:38'!
applyUsing: anApplier
	"Apply this delta to the image. We let an applier do it."

	^ anApplier applyDelta: self! !

!DSDelta methodsFor: 'applying' stamp: 'mtf 9/13/2007 21:30'!
asAntiDelta
	"Return a new Delta that, when applied, reverts each change of this Delta
	in reverse order. Each change should have enough information to create
	an anti change from itself."

	| anti |
	self isRevertable ifFalse: [DSDeltaNotRevertableException signal].
	anti := DSDelta name: 'anti-', self name.
	attributes associationsDo: [:assoc | anti attribAt: assoc key put: assoc value]. 
	self changes reverseDo: [:each | anti addChange: each asAntiChange]. 
	^anti! !

!DSDelta methodsFor: 'accessing' stamp: 'gk 8/15/2007 00:09'!
attachments
	^attachments! !

!DSDelta methodsFor: 'attributes' stamp: 'gk 8/23/2007 16:55'!
attribAt: aSymbol

	^attributes at: aSymbol ifAbsent: [nil]! !

!DSDelta methodsFor: 'attributes' stamp: 'gk 8/23/2007 16:55'!
attribAt: aSymbol ifAbsent: aBlock

	^attributes at: aSymbol ifAbsent: aBlock! !

!DSDelta methodsFor: 'attributes' stamp: 'gk 8/23/2007 16:54'!
attribAt: aSymbol put: aStringBooleanOrInteger

	^attributes at: aSymbol put: aStringBooleanOrInteger! !

!DSDelta methodsFor: 'accessing' stamp: 'gk 8/15/2007 00:10'!
attributes
	^attributes! !

!DSDelta methodsFor: 'attributes' stamp: 'mtf 9/13/2007 08:51'!
buildGrouper

	^self attribAt: #grouper put: (DSGroupedClassChange withAll: self; yourself)! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 9/5/2007 08:35'!
categorizeClass: aClass from: oldCategory to: newCategory
	"Class Category changed."

	self addChange: (DSClassCategoryChange class: aClass from: oldCategory to: newCategory)! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 8/31/2007 13:11'!
changeClassInstVars: aClass from: oldNames to: newNames
	"Instvars changed."

	self from: oldNames to: newNames do: [:old :new |
		self addChange: (DSClassInstVarsChange class: aClass theNonMetaClass from: old to: new)]! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 8/28/2007 23:36'!
changeClassType: aClass from: oldType to: newType
	"Type of class changed."

	self addChange: (DSClassTypeChange class: aClass from: oldType to: newType)! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 8/28/2007 23:39'!
changeClassVars: aClass from: oldNames to: newNames
	"Class vars changed."

	self from: oldNames asArray to: newNames asArray do: [:old :new |
		self addChange: (DSClassVarsChange class: aClass from: old to: new)]! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 8/28/2007 23:39'!
changeInstVars: aClass from: oldNames to: newNames
	"Instvars changed."

	self from: oldNames to: newNames do: [:old :new |
		self addChange: (DSInstVarsChange class: aClass from: old to: new)]! !

!DSDelta methodsFor: 'change editing' stamp: 'gk 8/29/2007 00:25'!
changeInstVarsName: aName from: oldNames to: newNames
	"Instvars changed."

	self from: oldNames to: newNames do: [:old :new |
		self addChange: (DSInstVarsChange className: aName from: old to: new)]! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 8/28/2007 23:39'!
changeSharedPools: aClass from: oldNames to: newNames
	"Shared pools changed."

	self from: oldNames to: newNames do: [:old :new |
		self addChange: (DSSharedPoolVarsChange class: aClass from: old to: new)]! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 8/28/2007 23:35'!
changeSuperclass: aClass from: oldSuperclass to: newSuperclass
	"Superclass changed."

	self addChange: (DSClassSuperclassChange class: aClass from: oldSuperclass to: newSuperclass)! !

!DSDelta methodsFor: 'accessing' stamp: 'gk 8/18/2007 11:03'!
changes
	^changes! !

!DSDelta methodsFor: 'testing' stamp: 'gk 8/18/2007 11:02'!
changesMaybeRevertable
	"Are all changes per definition revertable
	or marked by the developer as being revertable?"

	^changes allSatisfy: [:change | change isMaybeRevertable]! !

!DSDelta methodsFor: 'testing' stamp: 'gk 8/18/2007 11:02'!
changesRevertable
	"Are all changes per definition revertable?"

	^changes allSatisfy: [:change | change isRevertable]! !

!DSDelta methodsFor: 'queries' stamp: 'gk 8/20/2007 22:46'!
classCommentChanges
	^changes select: [:change | change isClassComment]! !

!DSDelta methodsFor: 'queries' stamp: 'mtf 10/2/2007 13:56'!
classGroupFor: protoChange
	"Answers all the changes to the same class as protoChange. Returns the list in reverse order from the delta"

	protoChange isClassChange ifFalse: [^ Array new].
	^ (self withClassName: protoChange className reverseSelect: [:change :className |
		change inClassGroupFor: protoChange withClassName: className])! !

!DSDelta methodsFor: 'queries' stamp: 'mtf 10/2/2007 14:01'!
classGroupNamed: aClassName
	"Answers all the changes to the class with this name"

	^ self classGroupFor: (DSClassChange className: aClassName)! !

!DSDelta methodsFor: 'applying' stamp: 'gk 9/7/2007 00:11'!
classGroupsDo: aBlock
	"Execute a block for all groups of changes per class."

	| groups coll |
	groups := Dictionary new.
	changes do: [:each |
		coll := groups at: each className ifAbsentPut: [SortedCollection new].
		coll add: each].
	groups values do: [:group | aBlock value: group]! !

!DSDelta methodsFor: 'queries' stamp: 'gk 8/21/2007 02:04'!
classInstVarsChanges
	^changes select: [:change | change isClassInstVars]! !

!DSDelta methodsFor: 'queries' stamp: 'gk 8/20/2007 23:18'!
classNameChanges
	^changes select: [:change | change isClassName]! !

!DSDelta methodsFor: 'queries' stamp: 'gk 8/21/2007 01:27'!
classRemovedChanges
	^changes select: [:change | change isClassRemoved]! !

!DSDelta methodsFor: 'queries' stamp: 'gk 8/21/2007 00:59'!
classSuperclassChanges
	^changes select: [:change | change isClassSuperclass]! !

!DSDelta methodsFor: 'applying' stamp: 'gk 8/23/2007 20:25'!
clearChanges
	"Remove all changes and return them."

	| oldChanges |
	oldChanges := changes.
	changes := OrderedCollection new.
	^oldChanges! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 8/28/2007 23:30'!
commentClass: aClass from: oldComment to: newComment oldStamp: oldStamp newStamp: newStamp
	"A class comment has been changed."

	self addChange: (DSClassCommentChange
						class: aClass
						from: oldComment
						to: newComment
						oldStamp: oldStamp
						newStamp: newStamp)! !

!DSDelta methodsFor: 'change editing' stamp: 'gk 8/29/2007 00:24'!
commentClassName: aName from: oldComment to: newComment oldStamp: oldStamp newStamp: newStamp
	"A class comment has been changed."

	self addChange: (DSClassCommentChange
						className: aName
						from: oldComment
						to: newComment
						oldStamp: oldStamp
						newStamp: newStamp)! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 9/3/2007 13:09'!
createClass: aClass
	"A new class was created."

	self addChange: (DSClassCreatedChange class: aClass)! !

!DSDelta methodsFor: 'queries' stamp: 'gk 8/18/2007 11:03'!
doIts

	^changes select: [:change | change isDoIt]! !

!DSDelta methodsFor: 'editing' stamp: 'gk 8/23/2007 22:17'!
editor
	^DSDeltaEditor on: self! !

!DSDelta methodsFor: 'private' stamp: 'mtf 9/5/2007 08:37'!
event: anEvent
	"This is where events come in from the SystemChangeNotifier.
	This code is similar to the old code in ChangeSet."

	| class |
	anEvent itemKind = SystemChangeNotifier classKind ifTrue: [
		class := anEvent item.
		class wantsChangeSetLogging ifFalse: [^self].
		anEvent isAdded ifTrue: [self createClass: class].
		anEvent isRemoved ifTrue: [self removeClass: class].
		anEvent isRenamed ifTrue: [self renameClassFrom: anEvent oldName to: anEvent newName].
		anEvent isCommented ifTrue: [self commentClass: class from: anEvent oldComment to: anEvent newComment oldStamp: anEvent oldStamp newStamp: anEvent newStamp].
		anEvent isModified ifTrue: [class isMeta
			ifTrue: [self modifyMetaclass: class event: anEvent]
			ifFalse: [self modifyClass: class event: anEvent]].
		anEvent isReorganized ifTrue: [self reorganizeClass: anEvent item].
		anEvent isRecategorized ifTrue: [self categorizeClass: anEvent item from: anEvent oldCategory to: anEvent item category].
	].

	anEvent itemKind = SystemChangeNotifier methodKind ifTrue: [
		class := anEvent itemClass.
		class wantsChangeSetLogging ifFalse: [^self].
		anEvent isAdded ifTrue: [self logMethodAddedEvent: anEvent].
		anEvent isRemoved ifTrue: [self logMethodRemovedEvent: anEvent].
		anEvent isRecategorized ifTrue: [self logMethodProtocolEvent: anEvent from: anEvent oldCategory to: anEvent itemProtocol].
		anEvent isModified ifTrue: [self logMethodModifyEvent: anEvent].
	].! !

!DSDelta methodsFor: 'utilities' stamp: 'mtf 10/3/2007 11:53'!
fixChangeNumbers
	"fix the change numbers to conform to the invariant that the number of the change is the same as it's index in the change array'"
	changes withIndexDo: [:change :index | change number: index].
	changeCounter := changes size! !

!DSDelta methodsFor: 'private' stamp: 'gk 8/22/2007 00:45'!
from: oldNames to: newNames do: aBlock
	"Scan both Strings of names into Arrays (or otherwise presume they are Arrays)
	and evaluate the block with them."

	| old new scanner |
	scanner := Scanner new.
	old := oldNames isString
			ifTrue: [scanner scanFieldNames: oldNames]
			ifFalse: [oldNames].
	new := newNames isString
			ifTrue: [scanner scanFieldNames: newNames]
			ifFalse: [newNames].
	^aBlock value: old value: new! !

!DSDelta methodsFor: 'attributes' stamp: 'mtf 9/13/2007 08:44'!
grouper

	^self attribAt: #grouper! !

!DSDelta methodsFor: 'testing' stamp: 'gk 8/18/2007 10:59'!
hasDoIts

	^self doIts notEmpty! !

!DSDelta methodsFor: 'private' stamp: 'gk 9/7/2007 09:49'!
incrementChangeCounter
	"Monotonically increasing change counter."

	^changeCounter := changeCounter + 1! !

!DSDelta methodsFor: 'initialize-release' stamp: 'mtf 9/21/2007 23:22'!
initialize
	self uuid: UUID new! !

!DSDelta methodsFor: 'testing' stamp: 'gk 8/23/2007 22:33'!
isEmpty
	^changes isEmpty! !

!DSDelta methodsFor: 'change logging' stamp: 'gk 8/18/2007 11:18'!
isLogging
	"Is this Delta logging system changes?"

	^SystemChangeNotifier uniqueInstance hasNotificationsFor: self! !

!DSDelta methodsFor: 'testing' stamp: 'gk 8/18/2007 11:01'!
isMaybeRevertable
	"It is maybe revertable if all changes are per definition revertable
	or marked by the developer as being revertable and this Delta
	is not flagged as notRevertable."

	^notRevertable not and: [self changesMaybeRevertable]! !

!DSDelta methodsFor: 'testing' stamp: 'gk 8/18/2007 11:01'!
isRevertable
	"It is revertable if all changes are per definition revertable
	and this Delta is not flagged as notRevertable."

	^notRevertable not and: [self changesRevertable]! !

!DSDelta methodsFor: 'change logging events' stamp: 'test 9/6/2007 22:28'!
logMethodAddedEvent: anEvent
	"A new method was created."

	self addChange: (DSMethodAddedChange fromEvent: anEvent)! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 8/28/2007 14:11'!
logMethodModifyEvent: anEvent
	anEvent isProtocolModified ifTrue:
		[self logMethodProtocolEvent: anEvent from: anEvent oldProtocol to: anEvent newProtocol].
	self logMethodSourceEvent: anEvent from: anEvent oldItem to: anEvent item! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 8/30/2007 11:30'!
logMethodProtocolEvent: anEvent from: oldProtocol to: newProtocol 
	"A method is being categorized under a new protocol."

	| selector |
	selector := anEvent itemSelector.
	selector ifNil: [selector := anEvent itemMethod sourceSelector].
	selector ifNil: [selector := anEvent itemMethod selector].
	self addChange: (DSMethodProtocolChange
		class: anEvent itemClass
		selector: selector
		from: oldProtocol
		to: newProtocol)! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 8/28/2007 14:11'!
logMethodRemovedEvent: anEvent
	"A new method was created."

	self addChange: (DSMethodRemovedChange fromEvent: anEvent)! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 8/28/2007 14:11'!
logMethodSourceEvent: anEvent from: oldMethod to: newMethod 
	"A method has new source."

	self addChange: (DSMethodSourceChange
		class: anEvent itemClass
		selector: anEvent itemSelector
		from: oldMethod
		to: newMethod)! !

!DSDelta methodsFor: 'applying' stamp: 'gk 9/7/2007 10:36'!
mergeChanges
	"Merge the changes in the group best we can."

	! !

!DSDelta methodsFor: 'queries' stamp: 'mtf 9/13/2007 16:05'!
methodGroupFor: protoChange
	"Answers all the changes to the same method in the same class, or to the same class if the change is not a method change"

	protoChange isClassChange ifFalse: [^ Array new].
	^ (self withClassName: protoChange className reverseSelect: [:change :className |
		change inMethodGroupFor: protoChange withClassName: className]) reverse! !

!DSDelta methodsFor: 'queries' stamp: 'mtf 9/13/2007 19:24'!
methodGroups
	^ DSChangeCategorizer forClassesAndMethods withAll: self! !

!DSDelta methodsFor: 'private' stamp: 'gk 8/28/2007 23:38'!
modifyClass: class event: anEvent
	"A class definition was modified. A lot of changes can be made this way:
		- class type
		- superclass
		- instvars
		- classvars
		- shared pools"

	anEvent isTypeModified ifTrue: [
		self changeClassType: class from: anEvent oldTypeOfClass to: anEvent typeOfClass].

	anEvent isSuperclassModified ifTrue: [
		self changeSuperclass: class from: anEvent oldSuperclass to: anEvent superclass].

	anEvent areInstVarsModified ifTrue: [
		self changeInstVars: class from: anEvent oldInstVarNames to: anEvent instVarNames].

	anEvent areClassVarsModified ifTrue: [
		self changeClassVars: class from: anEvent oldClassVarNames to: anEvent classVarNames].

	anEvent areSharedPoolsModified ifTrue: [
		self changeSharedPools: class from: anEvent oldItem poolDictionaryNames asArray to: anEvent item poolDictionaryNames asArray]! !

!DSDelta methodsFor: 'private' stamp: 'mtf 8/30/2007 16:59'!
modifyMetaclass: class event: anEvent
	"A metaclass definition was modified. This basicly just means the class instance variables were changed"

	anEvent areInstVarsModified ifTrue: [
		self changeClassInstVars: class theNonMetaClass from: anEvent oldInstVarNames to: anEvent instVarNames]! !

!DSDelta methodsFor: 'applying' stamp: 'gk 9/7/2007 16:17'!
moveClassNameChanges
	"Move all class name changes to the beginning as if they were made first."

	| nameChanges map |
	nameChanges := OrderedCollection new.
	map := Dictionary new.
	changes reverseDo: [:change |
		change isClassName
			ifTrue: [
				"Is there already a mapping in place?"
				(map includesKey: change newName)
					ifTrue: [
						"Then we remap it using the even older name as key from now on"
						map at: change oldName put: (map at: change newName).
						map removeKey: change newName]
					ifFalse: [
						"Then we just add a mapping."
						map at: change oldName put: change newName].
				nameChanges add: change ]
			ifFalse: [
				change remapClassNames: map]].
	changes removeAll: nameChanges.
	changes := nameChanges, changes! !

!DSDelta methodsFor: 'attributes' stamp: 'gk 8/24/2007 00:18'!
name

	^self attribAt: #name ifAbsent: ['<no name>']! !

!DSDelta methodsFor: 'attributes' stamp: 'gk 8/23/2007 16:55'!
name: aString

	^self attribAt: #name put: aString! !

!DSDelta methodsFor: 'applying' stamp: 'gk 9/7/2007 15:34'!
normalize
	"Modify changes so that they are compacted into the minimal form
	still producing the equivalent end result. In other words - remove redundancy."

	self moveClassNameChanges; removeShadows; mergeChanges! !

!DSDelta methodsFor: 'change logging events' stamp: 'gk 9/3/2007 13:18'!
removeClass: aClass 
	"A class was removed from the system. The default change
	is a composite class deleted change - since it needs to contain
	all individual changes needed to do a revert. If the resulting
	composite ends up containing just one change - then that
	can be used instead - a simple DSClassRemovedChange."

	| change |
	change := DSCompositeClassDeletedChange class: aClass.
	self addChange: (change changes size = 1
		ifTrue: [change changes first]
		ifFalse: [change])! !

!DSDelta methodsFor: 'applying' stamp: 'gk 9/7/2007 15:27'!
removeShadows
	"Remove all changes that are shadowed (redundant).
	We go in reverse and remove all changes before
	each change that it shadows. The logic is all based
	on #canShadow and #isShadowing:. To make it
	at least somewhat efficient we nil out the changes
	and then remove them in one big swoop at the end."

	| candidate change |
	changes size to: 2 by: -1 do: [:index |
		change := changes at: index.
		(change notNil and: [change canShadow]) ifTrue: [
			index - 1 to: 1 by: -1 do: [:i |
				candidate := changes at: i.
				(candidate notNil and: [change isShadowing: candidate])
					ifTrue: [changes at: i put: nil]]]].
	changes := changes select: [:each | each notNil]! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 8/29/2007 20:43'!
renameClassFrom: oldName to: newName
	"A class is being renamed."

	self addChange: (DSClassNameChange className: oldName to: newName)! !

!DSDelta methodsFor: 'change logging events' stamp: 'mtf 9/7/2007 22:36'!
reorganizeClass: aClass 
	self shouldBeImplemented! !

!DSDelta methodsFor: 'applying' stamp: 'gk 8/23/2007 20:26'!
revert
	"Revert this delta by applying its anti Delta to the image."

	^self asAntiDelta apply! !

!DSDelta methodsFor: 'private' stamp: 'mtf 10/3/2007 11:42'!
species
	"When selecting changes, use a plain change sequence, since DSDelta changes the number of all added changes"
	^ DSChangeSequence! !

!DSDelta methodsFor: 'change logging' stamp: 'gk 8/18/2007 11:18'!
startLogging
	"Make this Delta start logging system changes.
	ChangeSets only have one that is logging."

	self isLogging ifFalse: [
		SystemChangeNotifier uniqueInstance
			notify: self
			ofAllSystemChangesUsing: #event:]! !

!DSDelta methodsFor: 'change logging' stamp: 'gk 8/18/2007 11:19'!
stopLogging
	"Make this Delta stop logging system changes.
	ChangeSets only have one that is logging."

	self isLogging ifTrue: [
		SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self]! !

!DSDelta methodsFor: 'accessing' stamp: 'gk 8/15/2007 00:04'!
test

	^test! !

!DSDelta methodsFor: 'accessing' stamp: 'gk 8/15/2007 00:04'!
uuid

	^uuid! !

!DSDelta methodsFor: 'initialize-release' stamp: 'mtf 9/21/2007 23:21'!
uuid: aUUID
	super initialize.
	uuid := aUUID.
	changeCounter := 0.
	attributes := Dictionary new.
	changes := OrderedCollection new.
	notRevertable := false
	! !

!DSCompositeChange class methodsFor: 'as yet unclassified' stamp: 'mtf 8/31/2007 11:55'!
changes: anOrderedCollection
	^ self new changes: anOrderedCollection; yourself! !

!DSCompositeChange methodsFor: 'adding' stamp: 'mtf 9/6/2007 18:30'!
addChange: aChange
	self subclassResponsibility! !

!DSCompositeChange methodsFor: 'applying' stamp: 'mtf 9/21/2007 12:37'!
applyUsing: anApplier
	anApplier applyCompositeChange: self! !

!DSCompositeChange methodsFor: 'testing' stamp: 'gk 9/7/2007 15:32'!
canShadow
	"Can a change of this kind shadow other previous changes?"

	^true! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/20/2007 19:24'!
collect: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument.  
	Collect the resulting values into a collection like the receiver. Answer  
	the new collection."

	| newCollection |
	newCollection := self species new.
	self do: [:each | newCollection add: (aBlock value: each)].
	^ newCollection! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 19:48'!
do: aBlock
	self subclassResponsibility! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 22:33'!
groupsDo: aBlock
	^ aBlock value: self! !

!DSCompositeChange methodsFor: 'testing' stamp: 'mtf 8/31/2007 13:23'!
isComposite
	^ true! !

!DSCompositeChange methodsFor: 'private' stamp: 'gk 9/7/2007 16:22'!
remapClassNames: map
	"By default we do nothing. Subclasses reimplement."

	self do: [:each | each remapClassNames: map]! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/13/2007 10:28'!
reverseDo: aBlock
	^ self do: aBlock! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 20:13'!
visitLeaves: aBlock
	self do: [:aChange | aChange isComposite
		ifTrue: [aChange visitLeaves: aBlock]
		ifFalse: [aBlock value: aChange]]! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 19:55'!
withAll: aCompositeChange
	aCompositeChange do: [:change | self addChange: change]! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/6/2007 20:18'!
withAllLeaves: aCompositeChange
	aCompositeChange visitLeaves: [:change | self addChange: change]! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 9/13/2007 10:27'!
withClassName: aSymbol reverseDo: aBlock
	"For each change, evaluate a block with that change and a class name. The name will only be affected by class rename changes"

	| className |
	className := aSymbol.
	self reverseDo: [:change | 
		(change isClassName and: [change newName == className])
			ifTrue: [className := change oldName].
		aBlock value: change value: className]! !

!DSCompositeChange methodsFor: 'enumerating' stamp: 'mtf 10/2/2007 14:03'!
withClassName: aSymbol reverseSelect: aBlock
	"For each change, evaluate a block with that change and a class name. The name will only be affected by class rename changes"

	| newChanges |
	newChanges := self species new.
	self withClassName: aSymbol reverseDo: [:change :className | 
		(aBlock value: change value: className)
			ifTrue: [newChanges addChange: change]].
	^ newChanges! !

DSChange subclass: #DSDoIt
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Base'!

!DSDoIt methodsFor: 'testing' stamp: 'gk 8/21/2007 01:24'!
isDoIt
	^true! !

Object subclass: #DSDeltaEditor
	instanceVariableNames: 'delta class className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Model-Base'!

!DSDeltaEditor commentStamp: 'gk 8/29/2007 00:13' prior: 0!
A DSDeltaEditor is a temporary wrapper for a DSDelta in order to facilitate simpler manual editing/construction of a DSDelta.!

!DSDeltaEditor class methodsFor: 'as yet unclassified' stamp: 'gk 8/23/2007 22:18'!
on: aDelta

	^self new delta: aDelta; yourself! !

!DSDeltaEditor methodsFor: 'applying' stamp: 'gk 8/23/2007 22:44'!
apply
	^delta apply! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/29/2007 00:21'!
class

	^class ifNil: [className ifNil: [nil] ifNotNil: [Smalltalk classNamed: className]]! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/23/2007 22:20'!
class: aClass

	class := aClass! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 9/7/2007 16:33'!
className
	"Current class name"

	^className ifNil: [class ifNil: [self halt] ifNotNil: [class name]]! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 9/7/2007 16:31'!
className: aName
	"Select an existing class."

	className := aName! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 9/7/2007 16:32'!
classVars: newNames

	^delta changeClassVars: self class from: self class classVarNames to: newNames! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/23/2007 22:30'!
comment: aComment

	^self comment: aComment stamp: Utilities changeStamp! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/29/2007 00:24'!
comment: aComment stamp: aStamp

	class ifNotNil: [
		^delta commentClass: class
			from: class organization classComment
			to: aComment
			oldStamp: class organization commentStamp
			newStamp: aStamp]
		ifNil: [
		^delta commentClassName: className
			from: ''
			to: aComment
			oldStamp: nil
			newStamp: aStamp]! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/27/2007 00:15'!
createClass: aClass
	"Add an existing class."

	class := aClass.
	^delta createClassName: aClass name! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 9/3/2007 13:09'!
createClassName: aName
	"Add an existing class."

	className := aName.
	^delta addChange: (DSClassCreatedChange className: className)! !

!DSDeltaEditor methodsFor: 'accessing' stamp: 'gk 8/23/2007 23:03'!
delta
	^delta! !

!DSDeltaEditor methodsFor: 'accessing' stamp: 'gk 8/23/2007 22:38'!
delta: aDelta

	delta := aDelta! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/29/2007 00:27'!
instVars: newNames

	class ifNotNil: [
		^delta changeInstVars: class from: class instVarNames to: newNames]
	ifNil: [
		^delta changeInstVarsName: className from: '' to: newNames]
! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/28/2007 23:28'!
removeClass: aClass

	^delta removeClass: aClass! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/28/2007 23:29'!
removeClassName: aName

	^delta removeClass: (Smalltalk classNamed: aName)! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 9/7/2007 16:33'!
rename: aName

	^delta renameClassFrom: self className to: aName! !

!DSDeltaEditor methodsFor: 'applying' stamp: 'gk 8/23/2007 22:44'!
revert
	^delta revert! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/23/2007 22:23'!
sharedPools: newNames

	^delta changeSharedPools: class from: class poolDictionaryNames asArray to: newNames asArray! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/23/2007 22:26'!
superclass: aClass

	^delta changeSuperclass: class from: class superclass to: aClass! !

!DSDeltaEditor methodsFor: 'editing' stamp: 'gk 8/28/2007 23:37'!
type: aType

	^delta changeClassType: class from: class typeOfClass to: aType! !

Object subclass: #DSDeltaRepository
	instanceVariableNames: 'deltas'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'DeltaStreams-Logging'!

!DSDeltaRepository class methodsFor: 'as yet unclassified' stamp: 'mtf 9/5/2007 09:43'!
current
	^ Current ifNil: [Current := self new]! !

!DSDeltaRepository methodsFor: 'adding' stamp: 'mtf 9/26/2007 12:22'!
addAsking: aDelta
	(self named: aDelta name) ifNil: [^ self basicAdd: aDelta].
	(UIManager default chooseFrom: #('enter new name' 'overwrite' 'cancel') lines: #(2)
		title: 'The name "', aDelta name, '" is already in use') caseOf: {
	
	[1] -> [self addAsking: (self renameDelta: aDelta)].
	[2] -> [self addOverwriting: aDelta].
	[3] -> []}! !

!DSDeltaRepository methodsFor: 'adding' stamp: 'mtf 9/26/2007 11:52'!
addIfAbsent: aDelta
	(self named: aDelta name) ifNotNil: [^ nil].
	^ self basicAdd: aDelta! !

!DSDeltaRepository methodsFor: 'adding' stamp: 'mtf 9/26/2007 11:57'!
addInforming: aDelta
	^ (self addIfAbsent: aDelta) ifNil: [self inform: 'The name ''', aDelta name, ''' is already in use'. nil]! !

!DSDeltaRepository methodsFor: 'adding' stamp: 'mtf 9/6/2007 12:02'!
addNewNamed: newName
	| newDelta |
	(self named: newName) ifNotNil: [self inform: 'That name is already in use'. ^ nil].
	newDelta := self deltas add: (DSDelta name: newName).
	self changed: #deltaList.
	^ newDelta! !

!DSDeltaRepository methodsFor: 'adding' stamp: 'mtf 9/26/2007 11:54'!
addOverwriting: aDelta
	(self named: aDelta name) ifNotNil: [self remove: aDelta].
	^ self basicAdd: aDelta! !

!DSDeltaRepository methodsFor: 'adding' stamp: 'mtf 9/26/2007 11:54'!
addWithUniqueName: aDelta
	aDelta name: (self uniqueNameLike: aDelta name).
	^ self basicAdd: aDelta! !

!DSDeltaRepository methodsFor: 'private' stamp: 'mtf 9/26/2007 11:51'!
basicAdd: aDelta
	self deltas add: aDelta.
	self changed: #deltaList.
	^ aDelta! !

!DSDeltaRepository methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 10:37'!
defaultName
	^ self uniqueNameLike: 'Unnamed' translated! !

!DSDeltaRepository methodsFor: 'as yet unclassified' stamp: 'mtf 9/5/2007 09:45'!
deltas
	^ deltas ifNil: [deltas := OrderedCollection new]! !

!DSDeltaRepository methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 10:19'!
named: aName
	^ deltas detect: [:delta| delta name = aName] ifNone: [nil]! !

!DSDeltaRepository methodsFor: 'removing' stamp: 'mtf 9/26/2007 11:53'!
remove: aDelta
	self deltas remove: aDelta.
	self changed: #deltaList.
	^ aDelta! !

!DSDeltaRepository methodsFor: 'user interface' stamp: 'mtf 9/26/2007 12:21'!
renameDelta: aDelta
	| newName |
	
	newName := UIManager default
		request: 'Enter a new name for ', aDelta name
		initialAnswer: (self uniqueNameLike: aDelta name).
	newName isEmptyOrNil ifTrue: [^ aDelta].
	aDelta name: newName.
	^ aDelta
! !

!DSDeltaRepository methodsFor: 'as yet unclassified' stamp: 'mtf 10/9/2007 14:42'!
uniqueNameLike: aString

	| try |
	(self named: aString) ifNil: [^ aString].

	1 to: 999999 do:
		[:i | try := aString , i printString.
		(self named: try) ifNil: [^ try]]! !

Object subclass: #DSSystemLogger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Logging'!

!DSSystemLogger commentStamp: 'mtf 9/5/2007 09:48' prior: 0!
My instances are responsible for converting System events to changes to add to a DSDeltaRepository. In theory at least :)!

Object subclass: #DSVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSVisitor commentStamp: 'mtf 9/5/2007 10:00' prior: 0!
An abstract visitor for Deltas!

DSVisitor subclass: #DSApplier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSApplier commentStamp: 'mtf 9/20/2007 11:06' prior: 0!
An applier is an object that can apply DSDeltas. An applier is a "Big Hammer". It does not distinguish friend from foe, sane changes from nonsense changes. An applier can and should do everything possible to make the system match the end result of the changes handed to it. It is a validator's job to determine what changes an applier should make to the system.!

!DSApplier methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 13:13'!
applyCompositeChange: change
	"Apply a composite change by applying all changes in their order.
	Application is done using a double dispatch visitor approach."
	
	^ change do: [:each | each applyUsing: self]! !

!DSApplier methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 13:15'!
applyCompositeClassDeletedChange: change
	"We simply remove the whole class in one call rather than iterating through all the changes"

	^ self applyClassRemoved: change! !

DSApplier subclass: #DSSystemEditorApplier
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSSystemEditorApplier commentStamp: 'mtf 9/21/2007 13:23' prior: 0!
An applier is an object that can apply DSDeltas. One way to do this is to use SystemEditor - the tool for making source code changes in an atomic way developed for MC2. An applier is used like this:

	applier := DSSystemEditorApplier new.
	applier applyDelta: aDSDelta.

When it comes to making changes, an applier is a "Big Hammer". It does not distinguish friend from foe, sane changes from nonsense changes. An applier can and should do everything possible to make the system match the end result of the changes handed to it. It is the job of a not-yet-implemented validator to determine whether a change should be made and hand it off to the applier.!

!DSSystemEditorApplier class methodsFor: 'as yet unclassified' stamp: 'mtf 9/7/2007 20:36'!
on: aSystemEditor
	^ self new editor: aSystemEditor ! !

!DSSystemEditorApplier methodsFor: 'public' stamp: 'mtf 9/21/2007 13:29'!
apply: change
	"apply the change to the editor, then commit to the system"
	
	super apply: change.
	editor commit! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/5/2007 08:39'!
applyClassCategoryChanged: change
	"Changed a class category."

	(self classEditorForChange: change) category: change newCategory! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/4/2007 09:46'!
applyClassCommentChanged: change
	"Changed a class comment."

	(self classEditorForChange: change) classComment: change newComment stamp: change newStamp! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/13/2007 20:48'!
applyClassCreated: change
	"Add a class."

	| classEditor |
	self ensureSharedPools: change poolDictionaryNames.
	classEditor := editor at: change superclassName ifAbsent: [editor at: #Object].
	classEditor subclass: change className
			instanceVariableNames: change instVarNames asSpaceString
			classVariableNames: change classVarNames asSpaceString
			poolDictionaries: change poolDictionaryNames asSpaceString
			category: change category.
	classEditor typeOfClass: change type! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/20/2007 10:05'!
applyClassInstVarsChanged: change
	"Changed class instvars."

	| classEditor |
	classEditor := self classEditorForChange: change.
	change added do: [:each | classEditor addInstVarName: each].
	change removed do: [:each | classEditor removeInstVarName: each].
	change moved notEmpty ifTrue: [
		classEditor rearrangeInstVarNames: change movedWithIndices]! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/19/2007 15:12'!
applyClassNameChanged: change
	"Changed a class name. If the old class does not exist, do nothing. If the new name already exists, the behavior is not implemented"

	(editor includesKey: change oldName) ifFalse: [^ self].
	(editor includesKey: change newName) ifTrue: [^ self error: 'Class rename conflict. See the comment in DSDeltaDirtyApplyTest>>testClassRenameConfict'].
	(self classEditorForChange: change) rename: change newName.! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/13/2007 20:19'!
applyClassRemoved: change
	"Remove a class. Do nothing if it isn't there'"

	| classEditor |
	classEditor := editor at: change className ifAbsent: [^ self].
	classEditor removeFromSystem! !

!DSSystemEditorApplier methodsFor: 'as yet unclassified' stamp: 'mtf 9/21/2007 13:17'!
applyClassSnapshot: snapshot
	"Snapshots just record the old version; there is no new version. So, only apply them if there is nothing better."
	(editor includesKey: snapshot className) ifTrue: [^ self].
	self applyClassCreated: snapshot.
	snapshot classInstVarChange applyUsing: self.
	snapshot commentChange applyUsing: self.! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/20/2007 10:08'!
applyClassSuperclassChanged: change
	"Changed a class superclass."

	(self classEditorForChange: change)
		superclass: (editor at: change newSuperclassName ifAbsent: [editor at: #Object])! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/20/2007 09:57'!
applyClassVarsChanged: change
	"Changed class vars, we ignore ordering as it is irrelevant - 
	they are actually kept in a Dictionary."

	| classEditor |
	classEditor := self classEditorForChange: change.
	change added do: [:each | classEditor addClassVarName: each].
	change removed do: [:each | classEditor removeClassVarName: each]! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/20/2007 10:08'!
applyInstVarsChanged: change
	"Changed instvars."

	| classEditor |
	classEditor := self classEditorForChange: change.
	change added do: [:each | classEditor addInstVarName: each].
	change removed do: [:each | classEditor removeInstVarName: each].
	change moved notEmpty ifTrue: [
		classEditor rearrangeInstVarNames: change movedWithIndices]! !

!DSSystemEditorApplier methodsFor: 'method changes' stamp: 'mtf 8/31/2007 14:37'!
applyMethodAdded: change 
	(self classEditorForChange: change)
		compile: change source
		classified: change protocol
		withStamp: change stamp
		notifying: nil
		logSource: true! !

!DSSystemEditorApplier methodsFor: 'method changes' stamp: 'mtf 9/20/2007 10:19'!
applyMethodProtocolChanged: change
	"Change the protocol of a method, Does nothing if either the class or the method don't exist'"

	(self methodEditorForChange: change)
		category: change newProtocol! !

!DSSystemEditorApplier methodsFor: 'method changes' stamp: 'mtf 9/20/2007 10:10'!
applyMethodRemoved: change
	"remove a method from a class. Does nothing if class does not exist"
	
	| classEditor |
	classEditor := editor at: change className ifAbsent: [^ self].
	classEditor removeSelector: change selector! !

!DSSystemEditorApplier methodsFor: 'double dispatch' stamp: 'mtf 9/12/2007 11:43'!
applyMethodSnapshot: snapshot
	"Snapshots just record the old version; there is no new version. So, only apply them if there is nothing better."
	(self classEditorForChange: snapshot)
		edMethodAt: snapshot selector ifAbsent: [self applyMethodAdded: snapshot]! !

!DSSystemEditorApplier methodsFor: 'method changes' stamp: 'mtf 9/20/2007 10:18'!
applyMethodSourceChanged: change
	"Apply a method source change. If class or method is missing, create and categorize under default category"

	(self methodEditorForChange: change)
		source: change newSource;
		stamp: change newStamp! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/13/2007 20:58'!
applySharedPoolVarsChanged: change
	"Changed shared pool vars. Currently we just raise error if missing!!"

	| classEditor |
	self ensureSharedPools: change added.
	classEditor := editor at: change className.
	change added do: [:each |
		classEditor addSharedPool: (classEditor system edResolve: each asSymbol ifAbsent:[
			self error: 'Pool variable ',each,' not found!!'])].
	change removed do: [:each |
		classEditor removeSharedPool: (classEditor system edResolve: each asSymbol ifAbsent:[
			self error: 'Pool variable ',each,' not found!!'])].
	change moved notEmpty ifTrue: [
		classEditor rearrangeSharedPoolVarNames: change movedWithIndices]! !

!DSSystemEditorApplier methodsFor: 'class changes' stamp: 'mtf 9/20/2007 09:52'!
applySuperclassChanged: change
	"Change the superclass of a class."

	| classEditor superEditor |
	classEditor := editor ensureClassNamed: change className.
	superEditor := editor at: change newSuperclassName ifAbsent: [^ self].
	classEditor superclass: superEditor! !

!DSSystemEditorApplier methodsFor: 'private' stamp: 'mtf 9/17/2007 10:18'!
classEditorForChange: change
	| classEditor |
	classEditor := editor ensureClassNamed: change className.
	^ change isMeta
		ifTrue: [classEditor class]
		ifFalse: [classEditor].! !

!DSSystemEditorApplier methodsFor: 'initialize-release' stamp: 'mtf 9/7/2007 20:35'!
editor: aSystemEditor

	editor := aSystemEditor! !

!DSSystemEditorApplier methodsFor: 'private' stamp: 'mtf 9/13/2007 20:53'!
ensureSharedPools: anArray
	anArray do: [:name | Smalltalk at: name asSymbol ifAbsentPut: [Dictionary new]]! !

!DSSystemEditorApplier methodsFor: 'initialize-release' stamp: 'gk 8/23/2007 20:57'!
initialize

	editor := SystemEditor new! !

!DSSystemEditorApplier methodsFor: 'private' stamp: 'mtf 9/20/2007 10:17'!
methodEditorForChange: aMethodChange
	"answer the method editor for the change, creating a new, empty method if absent"
	
	| classEditor |
	classEditor := self classEditorForChange: aMethodChange.
	classEditor edMethodAt: aMethodChange selector ifAbsent: [classEditor
		compile: aMethodChange selector nullMethodSource
		classified: Categorizer default].
	^ classEditor edMethodAt: aMethodChange selector! !

!DSVisitor methodsFor: 'public' stamp: 'mtf 9/21/2007 13:27'!
apply: change
	"Apply the change using double-dispatch, then 'commit' if that makes sense. This is not a delegate"

	^ change applyUsing: self! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:41'!
applyCategorizedCompositeChange: change
	"Treat like a composite change by default"
	
	^ self applyCompositeChange: change! !

!DSVisitor methodsFor: 'basic changes' stamp: 'mtf 9/21/2007 13:15'!
applyChange: change
	"Handle an arbitrary change. By default, all changes eventually delegate here, so this is the place to put default behavior"

	^ self! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:37'!
applyChangeSequence: change
	"Treat like a composite change by default"
	
	^ self applyCompositeChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:07'!
applyClassCategoryChanged: change
	"Changed a class category."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:07'!
applyClassChange: change
	"Default handler for any class change."

	^ self applyChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:07'!
applyClassCommentChanged: change
	"Changed a class comment."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:07'!
applyClassCreated: change
	"Add a class."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:08'!
applyClassInstVarsChanged: change
	"Changed class instvars."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:08'!
applyClassNameChanged: change
	"Changed a class name."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:08'!
applyClassRemoved: change
	"Remove a class."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:08'!
applyClassSuperclassChanged: change
	"Changed a class superclass."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:33'!
applyClassTypeChanged: change
	"Changed the type of a class"

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:08'!
applyClassVarsChanged: change
	"Changed class vars, we ignore ordering as it is irrelevant - 
	they are actually kept in a Dictionary."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 13:16'!
applyCompositeChange: change
	"Visit a composite change"
	
	^ self applyChange: change! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:53'!
applyCompositeClassAddedChange: change
	"Treat like a composite class change by default"
	
	^ self applyCompositeClassChange: change! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:43'!
applyCompositeClassChange: change
	"Treat like a composite change by default"
	
	^ self applyCompositeChange: change! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:53'!
applyCompositeClassDeletedChange: change
	"Treat like a composite class change by default"
	
	^ self applyCompositeClassChange: change! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:53'!
applyCompositeMethodChange: change
	"Treat like a composite class change by default"
	
	^ self applyCompositeClassChange: change! !

!DSVisitor methodsFor: 'composite changes' stamp: 'mtf 9/21/2007 12:33'!
applyDelta: aDelta
	"Treat like a composite change by default"
	
	^ self applyCompositeChange: aDelta! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:08'!
applyInstVarsChanged: change
	"Changed instvars."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'method changes' stamp: 'mtf 9/21/2007 13:11'!
applyMethodAdded: change
	"Add a method"

	^ self applyMethodChange: change! !

!DSVisitor methodsFor: 'method changes' stamp: 'mtf 9/21/2007 13:10'!
applyMethodChange: change
	"Default handler for any method change."

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'method changes' stamp: 'mtf 9/21/2007 13:11'!
applyMethodProtocolChanged: change 
	"Change the protocol of a method"

	^ self applyMethodChange: change! !

!DSVisitor methodsFor: 'method changes' stamp: 'mtf 9/21/2007 13:11'!
applyMethodRemoved: change
	"Remove a method"

	^ self applyMethodChange: change! !

!DSVisitor methodsFor: 'method changes' stamp: 'mtf 9/21/2007 13:12'!
applyMethodSourceChanged: change
	"Change the source of a method"

	^ self applyMethodChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:09'!
applySharedPoolVarsChanged: change
	"Changed shared pool vars. Currently we just raise error if missing!!"

	^ self applyClassChange: change! !

!DSVisitor methodsFor: 'class changes' stamp: 'mtf 9/21/2007 13:09'!
applySuperclassChanged: change
	"Change the superclass of a class."

	^ self applyClassChange: change! !

DSVisitor subclass: #DSVisitorWrapper
	instanceVariableNames: 'visitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSVisitorWrapper commentStamp: 'mtf 9/29/2007 14:15' prior: 0!
My instances intercept change requests and act on them and optionally forward them to the wrapped visitor. we can be chained and composed together!

DSVisitorWrapper subclass: #DSChangeViewFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSChangeViewFilter commentStamp: 'mtf 9/29/2007 14:20' prior: 0!
I only apply changes that the UI wants to see when viewing the diff between an old and new system. I apply everything except class and method removals, since one wants to see that classes were removed, and not just have them not show up'!

!DSChangeViewFilter methodsFor: 'class changes' stamp: 'mtf 9/29/2007 14:20'!
applyClassRemoved: change
	"The UI wants to see the class. Do nothing"
	^ self! !

!DSChangeViewFilter methodsFor: 'method changes' stamp: 'mtf 9/29/2007 14:21'!
applyMethodRemoved: change
	"The UI wants to see the method. Do nothing"
	^ self! !

DSVisitorWrapper subclass: #DSValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSValidator commentStamp: 'mtf 9/21/2007 13:22' prior: 0!
A validator is a filter for changes. It rejects some changes and passes the rest to it's wrapped visitor. The default implementation accepts all changes!

DSValidator subclass: #DSEvilErrorIgnorer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Visitors'!

!DSEvilErrorIgnorer commentStamp: 'mtf 9/29/2007 14:24' prior: 0!
This class is a hack to get the UI working in spite of bugs in DSSystemEditorApplier. It delegates all messages to it's applier and ignores errors. Appliers should not raise errors, but DSSystemEditorApplier is still somewhat buggy, so it does (See DSDeltaApplyTest). This class is evil because it accepts *more* changes than the wrapped applier, not less!

!DSEvilErrorIgnorer methodsFor: 'as yet unclassified' stamp: 'mtf 9/21/2007 13:21'!
applyChange: change
	^ [super applyChange: change] on: Error do: []! !

!DSVisitorWrapper class methodsFor: 'as yet unclassified' stamp: 'mtf 9/29/2007 14:10'!
on: aVisitor
	^ self new visitor: aVisitor! !

!DSVisitorWrapper methodsFor: 'basic' stamp: 'mtf 9/29/2007 14:10'!
applyChange: change
	^ change applyUsing: visitor! !

!DSVisitorWrapper methodsFor: 'initialize-release' stamp: 'mtf 9/29/2007 14:10'!
visitor: aVisitor
	visitor := aVisitor! !

DSVisitor subclass: #DSWriter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

DSWriter subclass: #DSDeltaWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

!DSDeltaWriter commentStamp: 'mtf 9/21/2007 14:02' prior: 0!
My instances are capable of dumping a DSDelta to a stream

Instance Variables
	stream:		(WriteStream) the output stream

stream
	- xxxxx
!

!DSDeltaWriter class methodsFor: 'as yet unclassified' stamp: 'mtf 9/21/2007 13:59'!
on: aWriteStream
	^ self new stream: aWriteStream! !

!DSDeltaWriter methodsFor: 'initialize-release' stamp: 'mtf 9/21/2007 13:59'!
stream: aWriteStream
	stream := aWriteStream! !

DSDeltaWriter subclass: #DSDszWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

!DSDszWriter commentStamp: 'mtf 9/25/2007 14:05' prior: 0!
Creates a chunk file where most of the content is a gzipped DataStream of a DSDelta. It can be filed in normally.
Compression is not yet implemented!

!DSDszWriter methodsFor: 'composite changes' stamp: 'mtf 9/26/2007 12:25'!
applyDelta: aDelta

	self withBinaryDeflated: [:deflateStream | (DataStream on: deflateStream) nextPut: aDelta]! !

!DSDszWriter methodsFor: 'composite changes' stamp: 'mtf 9/26/2007 00:26'!
withBinary: aBlock
	"Perform aBlock with the stream in binary mode"

	[stream binary. aBlock value: stream] ensure: [stream text]! !

!DSDszWriter methodsFor: 'composite changes' stamp: 'mtf 9/26/2007 00:25'!
withBinaryDeflated: aBlock
	"Perform aBlock with the stream in both binary and deflated mode"

	self withBinary: [:binaryStream | self withDeflated: aBlock]! !

!DSDszWriter methodsFor: 'composite changes' stamp: 'mtf 9/26/2007 00:24'!
withDeflated: aBlock
	"Perform aBlock with the stream in deflated mode"

	| deflateStream |
	[aBlock value: (deflateStream := GZipWriteStream on: stream)] ensure: [deflateStream finish]
! !

DSDeltaWriter subclass: #DSStWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-Storing'!

!DSStWriter methodsFor: 'visiting' stamp: 'mtf 9/24/2007 11:43'!
applyChange: change

	self writeChangeHeader: change.
	self writeInstVars: (self instVarNamesFor: change) change: change.
	self writeChangeTail: change.! !

!DSStWriter methodsFor: 'visiting' stamp: 'mtf 9/21/2007 22:52'!
applyCompositeChange: compositeChange

	self writeChangeHeader: compositeChange.
	self writeChanges: compositeChange.
	self writeChangeTail: compositeChange! !

!DSStWriter methodsFor: 'visiting' stamp: 'mtf 9/24/2007 11:17'!
applyDelta: delta

	stream
		nextPutAll: 'DSDeltaRepository current addDeltaWithUniqueName: ((';
		nextPutAll: delta class name;
		nextPutAll: ' uuid: (UUID fromString: ';
		print: delta uuid asString;
		nextPutAll: ')) name: ';
		print: delta name;
		nextPutAll: '; '.
	self writeChanges: delta.
	self writeChangeTail: delta.
	stream nextPutAll: ') !!'; cr! !

!DSStWriter methodsFor: 'private' stamp: 'mtf 9/24/2007 11:38'!
instVarNamesFor: change

	^ change class allInstVarNames difference: DSChange allInstVarNames! !

!DSStWriter methodsFor: 'writing' stamp: 'mtf 9/21/2007 22:42'!
writeChangeHeader: change

	stream nextPutAll: change class name; nextPutAll: ' new'! !

!DSStWriter methodsFor: 'writing' stamp: 'mtf 9/24/2007 19:22'!
writeChangeTail: change

	stream nextPutAll: ' yourself'
	! !

!DSStWriter methodsFor: 'writing' stamp: 'mtf 9/21/2007 22:52'!
writeChanges: compositeChange

	compositeChange do: [:change |
		stream nextPutAll: 'addChange: ('.
		change applyUsing: self.
		stream nextPut: $).
		stream cr]! !

!DSStWriter methodsFor: 'writing' stamp: 'mtf 9/24/2007 19:20'!
writeInstVars: anArray change: change

	anArray do: [:var | stream
		space;
		nextPutAll: var asString;
		nextPutAll: ': ';
		store: (change perform: var asSymbol);
		nextPut: $;]! !

!DSWriter class methodsFor: 'as yet unclassified' stamp: 'mtf 9/26/2007 10:51'!
on: aWriteStream
	self subclassResponsibility! !

!DSWriter methodsFor: 'initialize-release' stamp: 'mtf 9/26/2007 10:51'!
stream: aWriteStream
	self subclassResponsibility! !

!Object methodsFor: '*DeltaStreams' stamp: 'gk 8/14/2007 23:53'!
hasActionsWithReceiver: anObject

	^self actionMap keys anySatisfy:
		[:eachEventSelector |
			(self actionSequenceForEvent: eachEventSelector)
				anySatisfy: [:anAction | anAction receiver == anObject]]! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 12:39'!
browserDefinition: style
	^ self definition! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 12:50'!
compiledMethodAt: aSelector ifAbsent: aBlock
	^ self edMethodAt: aSelector ifAbsent: aBlock! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 12:39'!
definition
	^ 'Someclass subclass: blah blah blah...'! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 13:15'!
parserClass
	^ self class parserClass! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 10/2/2007 09:48'!
selectors
	^ self organization allMethodSelectors! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 12:43'!
sourceCodeTemplate
	^ self class sourceCodeTemplate! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 13:03'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
	"Return the source code at selector, deferring to superclass if necessary"
	^ self sourceCodeAt: selector ifAbsent:
		[self superclass
			ifNil:
				[aBlock value]
			 ifNotNil:
				[self superclass ultimateSourceCodeAt: selector ifAbsent: aBlock]]! !

!ClassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 10/2/2007 09:48'!
whichCategoryIncludesSelector: aSymbol 
	^ self organization categoryOfElement: aSymbol! !

!OrderedCollection methodsFor: '*DeltaStreams' stamp: 'mtf 8/31/2007 11:48'!
reverseCollect: aBlock 
	"Evaluate aBlock with each of my elements, in reverse order, as the argument. Collect the 
	resulting values into a collection that is like me. Answer the new 
	collection. Override superclass in order to use addLast:, not at:put:."

	| newCollection |
	newCollection := self species new: self size.
	lastIndex to: firstIndex by: -1 do:
		[:index |
		newCollection addLast: (aBlock value: (array at: index))].
	^ newCollection! !

!SystemEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/26/2007 15:28'!
atChange: change ifAbsent: aBlock

	| classEditor |
	change isClassChange ifFalse: [^ aBlock value].
	classEditor := self at: change className ifAbsent: [^ aBlock value].
	change isMeta ifTrue: [classEditor := classEditor class].
	^ classEditor! !

!SystemEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/17/2007 10:17'!
ensureClassNamed: className
	^ self at: className ifAbsent: [(self at: #Object) subclass: className
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: Categorizer default]! !

!MetaclassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 12:34'!
browserDefinition: style
	^ self definition! !

!MetaclassEditor methodsFor: '*DeltaStreams' stamp: 'mtf 9/20/2007 12:40'!
definition
	^ 'Somemetaclass blah blah blah...'! !

ClassBrowserPlus subclass: #SystemEditorBrowser
	instanceVariableNames: 'editor showChangesOnly'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-GUI'!

SystemEditorBrowser subclass: #DSChangeSorter
	instanceVariableNames: 'repository deltaListIndex changeListIndex versionListIndex delta changeApplier shownChanges autoLimitChanges changeListSelection unsavedChanges changeListShowList changeListShowFilter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeltaStreams-GUI'!

!DSChangeSorter class methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 11:01'!
new
	^ super new repository: DSDeltaRepository current! !

!DSChangeSorter class methodsFor: 'as yet unclassified' stamp: 'mtf 9/6/2007 10:59'!
openOnRepository: aRepository
	^ self new repository: aRepository; open! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/2/2007 09:54'!
allChanges
	unsavedChanges ifNotNil: [^ unsavedChanges].
	delta ifNotNil: [^ delta changes].
	^ #()! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 9/27/2007 11:14'!
buildChangeListWith: aBuilder
	| listSpec |
	listSpec := builder pluggableMultiSelectionListSpec new.
	listSpec model: self;
		list: #changeList;
		listItem: #changeListAt:;
		listSize: #changeListSize;
		getIndex: #changeListIndex;
		setIndex: #changeListIndex:;
		menu: #changeListMenu:;
		keyPress: #changeListKey:from:;
		dragItem: #dragChange:;
		getSelectionList: #changeListSelectionAt:;
		setSelectionList: #changeListSelectionAt:put:;	
		
		frame: (0.2 @ 0 corner: 0.4 @ 1).
		
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/5/2007 22:17'!
buildClassListButtonsWith: aBuilder
	| listSpec |
	listSpec := super buildClassListButtonsWith: aBuilder.
  	listSpec frame: ((listSpec frame
		left: 0.6)
		right: 0.75).
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/5/2007 22:17'!
buildClassListWith: aBuilder
	| listSpec |
	listSpec := super buildClassListWith: aBuilder.
  	listSpec frame: ((listSpec frame
		left: 0.6)
		right: 0.75).
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/6/2007 08:46'!
buildCodePaneWith: aBuilder
	| listSpec |
	listSpec := super buildCodePaneWith: aBuilder.
  	listSpec frame: ((listSpec frame
		left: 0.6)
		right: 1).
	^ listSpec.! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/27/2007 15:29'!
buildDeltaListWith: aBuilder
	| listSpec |
	listSpec := builder pluggableTreeSpec new.
	listSpec model: self;
		label: #deltaTreeLabelFor:;
		roots: #deltaTreeRoots;
		hasChildren: #deltaTreeHasChildren:;
		getChildren: #deltaTreeChildrenAt:;
		getSelectedPath: #deltaTreeSelection;
		setSelected: #deltaTreeSelection:;
		menu: #deltaListMenu:;
		keyPress: #deltaListKey:from:;
		
		frame: (0 @ 0 corner: 0.2 @ 1).
		
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/5/2007 22:15'!
buildMessageCategoryListWith: aBuilder
	| listSpec |
	listSpec := super buildMessageCategoryListWith: aBuilder.
  	listSpec frame: ((listSpec frame
		left: 0.75)
		right: 0.85).
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/5/2007 22:16'!
buildMessageListWith: aBuilder
	| listSpec |
	listSpec := super buildMessageListWith: aBuilder.
  	listSpec frame: ((listSpec frame
		left: 0.85)
		right: 1.0).
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/5/2007 22:17'!
buildSystemCategoryBrowserWith: aBuilder
	| listSpec |
	listSpec := super buildSystemCategoryBrowserWith: aBuilder.
  	listSpec frame: ((listSpec frame
		left: 0.4)
		right: 0.6).
	^ listSpec.! !

!DSChangeSorter methodsFor: 'version list' stamp: 'mtf 9/6/2007 08:44'!
buildVersionListWith: aBuilder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec model: self;
		 list: #versionList;
		 getIndex: #versionListIndex;
		 setIndex: #versionListIndex:;
		 menu: #versionListMenu:;
		 keyPress: #versionListKey:from:;
		 dragItem: #dragVersion:;
		
		frame: (0.4 @ 0.5 corner: 0.6 @ 1).
		
	^ listSpec.! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/7/2007 10:41'!
buildWith: aBuilder 
	"Create the ui for the browser"
	| windowSpec panelSpec |
	builder := aBuilder.
	windowSpec := self buildWindowWith: aBuilder.

	windowSpec children add: (self buildDeltaListWith: aBuilder).
	windowSpec children add: (self buildChangeListWith: aBuilder).
	
	windowSpec children add: (self buildSystemCategoryBrowserWith: aBuilder).
	windowSpec children add: (self buildClassListWith: aBuilder).
	windowSpec children addLast: (self buildClassListButtonsWith: aBuilder).
	windowSpec children add: (self buildMessageCategoryListWith: aBuilder).
	windowSpec children add: (self buildMessageListWith: aBuilder).
	
	self wantsOptionalButtons
		ifTrue: [panelSpec := self buildOptionalButtons.
			panelSpec
				frame: (0.4 @ 0.42 corner: 1 @ 0.5).
			windowSpec children add: panelSpec].

	windowSpec children add: (self buildVersionListWith: aBuilder).
	windowSpec children add: (self buildCodePaneWith: aBuilder).
	window := builder build: windowSpec.
	^ window! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 9/27/2007 11:54'!
changeList
	Transcript show: 'fetch list'; cr.
	^ self shownChanges collect: [:change | self highlightChange: change]! !

!DSChangeSorter methodsFor: 'change list menu' stamp: 'mtf 10/8/2007 12:00'!
changeListApplySelected
	(self confirm: 'There is currently no sanity checking of applied changes. Proceed anyway?') ifFalse: [^ self].
	self selectedChangesAsDelta apply! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 10/3/2007 11:30'!
changeListAt: externalIndex
	^ self highlightChange: (self shownChangeAt: externalIndex)! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 9/6/2007 11:40'!
changeListIndex
	^ changeListIndex! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 10/3/2007 11:31'!
changeListIndex: externalIndex

	| change |
	changeListIndex = externalIndex ifTrue: [^ self]. "Quick return"
	changeListIndex := externalIndex.
	changeListIndex = 0 ifTrue: [^ self].
	change := self shownChangeAt: changeListIndex.
	self selectChangeSubject: change.
	self changed: #changeListIndex! !

!DSChangeSorter methodsFor: 'change list menu' stamp: 'mtf 10/8/2007 11:06'!
changeListMenu: aMenu
	self hasHiddenChanges ifTrue: [
		aMenu add: 'show all' action: #changeListShowAll].
	autoLimitChanges
		ifTrue: [aMenu add: 'don''t auto-limit changes (l)' action: #changeListAutoLimitOff]
		ifFalse: [aMenu add: 'auto-limit changes (l)' action: #changeListAutoLimitOn].
	aMenu addLine.
	aMenu add: 'select all (a)' action: #changeListSelectAll.
	aMenu add: 'select none (n)' action: #changeListSelectNone.
	aMenu add: 'toggle selection (t)' action: #changeListToggleSelection.
	aMenu addLine.
	aMenu add: 'apply selected changes...' action: #changeListApplySselected.
	aMenu add: 'revert selected changes...' action: #changeListRevertSselected.
	aMenu addLine.
	aMenu add: 'hide selected (h)' action: #changeListHideSselected.
	aMenu add: 'hide unselected' action: #changeListDeleteUnselected.
	aMenu addLine.
	aMenu add: 'save (s)' action: #changeListSave.
	aMenu add: 'save as newVersion' action: #changeListSaveAsNewVersion.
	aMenu add: 'save as...' action: #changeListSaveAs.
	aMenu addLine.
	aMenu add: 'move selected... (m)' action: #changeListMoveSelected.
	aMenu add: 'copy selected... (c)' action: #changeListCopySelected.
	aMenu add: 'delete selected (d)' action: #changeListDeleteSelected.
	
	^ aMenu! !

!DSChangeSorter methodsFor: 'change list menu' stamp: 'mtf 10/3/2007 11:45'!
changeListSelectNone
	changeListSelection := nil.
	self changed: #changeList! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 10/2/2007 14:24'!
changeListSelectionAt: externalIndex
	"Answers true if change is selected, false if not."
	
	^ self changeListSelectionInternalAt: (self internalChangeIndexFor: externalIndex)! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 10/2/2007 14:26'!
changeListSelectionAt: externalIndex put: aBoolean
	self changeListSelectionInternalAt: (self internalChangeIndexFor: externalIndex)
		put: aBoolean.
	self changed: #changeListIndex! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/3/2007 11:38'!
changeListSelectionAtChanges: aCompositeChange put: aBoolean
	"Select/unselect the changes in the change list. Depends on each change's number being set correctly"
	
	self halt.
	aCompositeChange do: [:change | self changeListSelectionInternalAt: change number put: aBoolean]! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/3/2007 11:10'!
changeListSelectionInternalAt: internalIndex
	"Answers true if change is selected, false if not."
	
	changeListSelection ifNil: [^ false].
	changeListSelection size <= internalIndex ifTrue: [^ false].
	^ changeListSelection at: internalIndex ifAbsentPut: [false]! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/2/2007 14:25'!
changeListSelectionInternalAt: internalIndex put: aBoolean
	aBoolean ifNil: [^ self].
	aBoolean == (self changeListSelectionInternalAt: internalIndex) ifTrue: [^ self].
	changeListSelection ifNil: [changeListSelection := OrderedCollection new].
	changeListSelection at: internalIndex ifAbsentPut: [aBoolean]. "make sure the list is big enough"
	changeListSelection at: internalIndex put: aBoolean! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 10/3/2007 11:34'!
changeListSize
	"efficiently query the size of the displayed change list. Called by PluggableListMorph on every event"
	
	^ shownChanges
		ifNotNil: [shownChanges size]
		ifNil: [self allChanges size]! !

!DSChangeSorter methodsFor: 'class list' stamp: 'mtf 10/1/2007 11:42'!
classListMenu: aMenu
	"Set up the menu to apply to the receiver's class list, honoring the #shifted boolean"
	
	delta ifNil: [^ super classListMenu: aMenu].

	aMenu addList: #(
		('select class changes (s)' 		selectClassChanges)
		('only show class changes (o)'	onlyClassChanges)
		('hide class changes (h)'		hideClassChanges)
		-
		('show definition'			editClass)
		('show comment'			editComment)
		-
		('rename class ...'			renameClass)
		('copy class'				copyClass)
		('remove class (x)'			removeClass)
		-
		('find method...'				findMethod)
		('find method wildcard...'	findMethodWithWildcard)).
	^ aMenu! !

!DSChangeSorter methodsFor: 'accessing' stamp: 'mtf 9/6/2007 13:47'!
delta
	^ delta! !

!DSChangeSorter methodsFor: 'accessing' stamp: 'mtf 10/2/2007 10:03'!
delta: aDelta
	delta == aDelta ifTrue: [^ self].
	changeListIndex := 0.
	versionListIndex := 0.
	delta ifNotNil: [delta removeDependent: self].
	delta := aDelta.
	delta ifNotNil: [delta addDependent: self].
	unsavedChanges := nil.
	shownChanges := nil.
	changeListShowList := nil.
	changeListShowFilter := nil.
	changeListSelection := nil.
	delta ifNil: 
			[self editor: nil.
			^ self].
	self editor: SystemEditor new.
	changeApplier := DSChangeViewFilter on: (DSEvilErrorIgnorer 
				on: (DSSystemEditorApplier on: self editor)).
	delta applyUsing: changeApplier! !

!DSChangeSorter methodsFor: 'delta list functions' stamp: 'mtf 9/25/2007 12:18'!
deltaListMenu: aMenu
	delta ifNotNil: [delta isLogging
		ifFalse: [aMenu add: 'start logging' action: #startLogging]
		ifTrue: [aMenu add: 'stop logging' action: #stopLogging]].
	aMenu addLine.
	aMenu add: 'new change set... (n)' action: #newDelta.
	aMenu add: 'file out delta (o)' action: #fileOutDelta.
	^ aMenu! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/27/2007 14:27'!
deltaTreeHasChildren: item
	^ false.! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/27/2007 14:31'!
deltaTreeLabelFor: item
	(item isKindOf: Association) ifTrue: [^ item value]. "One of the virtual deltas"
	^ item name! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/27/2007 15:32'!
deltaTreeRoots
	^ self virtualDeltaActions, repository deltas! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/29/2007 11:36'!
deltaTreeSelection: item
	(item isKindOf: Association) ifTrue: [^ self perform: item key]. "One of the virtual deltas"
	delta = item ifTrue: [^ self].
	self delta: item.
	self changed: #changeList.
	self changed: #systemCategoryList! !

!DSChangeSorter methodsFor: 'delta list functions' stamp: 'mtf 9/26/2007 10:54'!
fileOutDelta
	"Much stuff copied from ChangeSet>>fileOut"

	| nameToUse |
	ChangeSet promptForDefaultChangeSetDirectoryIfNecessary.
	nameToUse := Preferences changeSetVersionNumbers 
				ifTrue: 
					[ChangeSet defaultChangeSetDirectory nextNameFor: delta name
						extension: DSDszReader extension]
				ifFalse: 
					[delta name , FileDirectory dot , Utilities dateTimeSuffix 
						, FileDirectory dot , DSDszReader extension].
	nameToUse := ChangeSet defaultChangeSetDirectory fullNameFor: nameToUse.
	Cursor write showWhile: 
			[FileStream newFileNamed: nameToUse
				do: [:file | delta applyUsing: (DSDszWriter on: file)]]! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 9/27/2007 11:37'!
hasHiddenChanges
	shownChanges ifNil: [^ false].
	delta ifNil: [^ false].
	unsavedChanges 
		ifNil: [^ delta changes size ~= shownChanges size]
		ifNotNil: [^ unsavedChanges size ~= shownChanges size]! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 9/27/2007 11:52'!
highlightChange: change
	"Return a properly colored text for the change"

	^ change printString! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 9/5/2007 22:43'!
initialExtent
	^ 1000@400! !

!DSChangeSorter methodsFor: 'initialize-release' stamp: 'mtf 10/3/2007 10:55'!
initialize
	super initialize.
	autoLimitChanges := true.
	changeListIndex := 0.! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/3/2007 11:36'!
internalChangeIndexFor: externalIndex
	"Due to the ability to hide changes from view, we store a map from external index (that the pluggable list knows about) and internal indexes (that the delta understands). We also reverse the delta list so that new changes are on top"
	
	^ shownChanges
		ifNil: [self allChanges size - externalIndex + 1]
		ifNotNil: [shownChanges at: externalIndex]! !

!DSChangeSorter methodsFor: 'delta list functions' stamp: 'mtf 9/6/2007 12:10'!
newDelta
	| newName |
	self okToChange ifFalse: [^ nil].
	newName := UIManager default
		request: 'Enter a name'
		initialAnswer: repository defaultName.
	newName isEmptyOrNil ifTrue: [^ nil].
	^ repository addNewNamed: newName! !

!DSChangeSorter methodsFor: 'accessing' stamp: 'mtf 9/6/2007 11:03'!
repository
	^ repository! !

!DSChangeSorter methodsFor: 'accessing' stamp: 'mtf 9/6/2007 13:45'!
repository: aRepository
	deltaListIndex := 0.
	self delta: nil.
	
	repository ifNotNil: [repository removeDependent: self].
	repository := aRepository.
	repository ifNotNil: [repository addDependent: self].! !

!DSChangeSorter methodsFor: 'change list plug' stamp: 'mtf 9/27/2007 11:43'!
selectChangeSubject: change

	| classEditor methodEditor |
	change isClassChange ifFalse: [^ self].
	classEditor := editor atChange: change ifAbsent: [^ self].
	self selectCategoryForClass: classEditor theNonMetaClass.
	self selectClass: classEditor theNonMetaClass.
	self metaClassIndicated: classEditor isMeta.
	change isMethodChange ifFalse: [^ self].
	methodEditor := classEditor edMethodAt: change selector ifAbsent: [^ self].
	self selectMessageCategoryNamed: methodEditor category.
	self selectedMessageName: methodEditor selector.! !

!DSChangeSorter methodsFor: 'class list' stamp: 'mtf 10/2/2007 14:29'!
selectClassChanges
	"Select all changes for the selected class"
	
	self changeListSelectionAtChanges: (delta classGroupNamed: self selectedClassName) put: true.
	self changed: #changeList! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/29/2007 11:36'!
selectLiveImage
	self delta: nil.
	self systemOrganizer: SystemOrganization.
	self changed: #changeList.
	self changed: #systemCategoryList! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/8/2007 12:05'!
selectedChangesAsDelta
	^ DSDelta new changes: self shownSelectedChanges! !

!DSChangeSorter methodsFor: 'class list' stamp: 'mtf 10/2/2007 14:15'!
selectedClassGroup
	"find all changes for the selected class"
	
	^ delta classGroupNamed: self selectedClassName! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/3/2007 11:29'!
shownChangeAt: externalIndex
	^ self allChanges at: (self internalChangeIndexFor: externalIndex)! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/3/2007 11:22'!
shownChanges
	shownChanges ifNotNil: [^ shownChanges
		collect: [:changeIndex | self allChanges at: changeIndex]].
	^ self allChanges reversed! !

!DSChangeSorter methodsFor: 'change list' stamp: 'mtf 10/8/2007 11:29'!
shownSelectedChanges
	"Answer the changes that are both visible and selected"
	
	changeListSelection ifNil: [^ Array new].
	shownChanges ifNotNil: [^ shownChanges
		select: [:changeIndex | self changeListSelectionInternalAt: changeIndex]
		thenCollect: [:changeIndex | self allChanges at: changeIndex]].
	^ (1 to: changeListSelection)
		select: [:changeIndex | changeListSelection at: changeIndex]
		thenCollect: [:changeIndex | self allChanges at: changeIndex]! !

!DSChangeSorter methodsFor: 'delta list functions' stamp: 'mtf 9/7/2007 10:59'!
startLogging
	delta startLogging.
	self changed: #deltaList "So the delta will be drown bold now"! !

!DSChangeSorter methodsFor: 'delta list functions' stamp: 'mtf 9/7/2007 11:00'!
stopLogging
	delta stopLogging.
	self changed: #deltaList "So the delta will be drown normal now"! !

!DSChangeSorter methodsFor: 'as yet unclassified' stamp: 'mtf 9/29/2007 00:43'!
systemCategoryMenu: aMenu
	delta ifNil: [^ super systemCategoryMenu: aMenu].
	
	aMenu add: 'select package changes (s)' action: #selectClassCategoryPackageChanges.
	aMenu add: 'only show package changes (o)' action: #onlyClassCategoryPackageChanges.
	aMenu add: 'hide package changes (h)' action: #hideClassCategoryPackageChanges.
	aMenu addLine.
	aMenu add: 'select category changes' action: #selectClassCategoryChanges.
	aMenu add: 'only show category changes' action: #onlyClassCategoryPackageChanges.
	aMenu add: 'hide category changes' action: #hideClassCategoryPackageChanges.
	aMenu addLine.
	aMenu add: 'browse all' action: #browseAllClasses.
	aMenu add: 'browse' action: #buildSystemCategoryBrowser.
	aMenu addLine.
	aMenu add: 'update' action: #updateSystemCategories.
	aMenu add: 'add item...' action: #addSystemCategories.
	aMenu add: 'rename...' action: #renameSystemCategory.
	aMenu add: 'remove' action: #removeSystemCategory.
	
	^ aMenu! !

!DSChangeSorter methodsFor: 'version list' stamp: 'mtf 9/6/2007 08:45'!
versionList
	^ #()! !

!DSChangeSorter methodsFor: 'version list' stamp: 'mtf 10/3/2007 10:56'!
versionListIndex
	^ 0! !

!DSChangeSorter methodsFor: 'delta tree' stamp: 'mtf 9/26/2007 17:41'!
virtualDeltaActions
	"Answers an array of associations. each Value is the text to display, and
	each key is the message to send when it is selected"
	^ Array with: #selectLiveImage -> 'Live Image' asText allBold! !

!SystemEditorBrowser methodsFor: 'accessing' stamp: 'mtf 9/6/2007 14:01'!
editor
	^ editor! !

!SystemEditorBrowser methodsFor: 'accessing' stamp: 'mtf 9/20/2007 17:34'!
editor: anEditor 
	"Changes the system editor being shown, Attempts to preserve as much
	of the selection as possible"
	editor := anEditor.
	self systemOrganizer: (anEditor
		ifNil: [SystemOrganization]
		ifNotNil: [anEditor organization]).
	self systemCategoryListIndex: 0. "Deselect all classes"! !

!SystemEditorBrowser methodsFor: 'accessing' stamp: 'mtf 9/7/2007 20:23'!
selectedEnvironment
	"Overridden to support using a SystemEditor as the environment"
	
	self editor ifNotNil: [^ self editor].
	^ super selectedEnvironment! !

!SystemEditorBrowser methodsFor: 'message list' stamp: 'mtf 9/20/2007 14:14'!
selectedMessage
	"Answer a copy of the source code for the selected message."

	| class selector method |
	editor ifNil: [^ super selectedMessage].
	contents == nil ifFalse: [^ contents copy].

	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	method :=  class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod := method.

	self showingDocumentation
		ifFalse: [contents := self sourceStringPrettifiedAndDiffed]
		ifTrue: [contents := self commentContents].
	^ contents := contents copy asText makeSelectorBoldIn: class! !

!SystemEditorBrowser methodsFor: 'accessing' stamp: 'mtf 9/6/2007 14:01'!
showChangesOnly
	^ showChangesOnly! !

!SystemEditorBrowser methodsFor: 'accessing' stamp: 'mtf 9/6/2007 14:04'!
showChangesOnly: aBool
	showChangesOnly := true! !
DSDeltaReader initialize!
